#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>

#include "main.h"
#include "Bluecell_operate.h"
#include "PE43711.h"
#include "eeprom.h"




#define CPU_VERSION0 0
#define CPU_VERSION1 0
#define CPU_VERSION2 3
#define ANSI_COLOR_RED     "\x1b[31m"
#define ANSI_COLOR_GREEN   "\x1b[32m"
#define ANSI_COLOR_YELLOW  "\x1b[33m"
#define ANSI_COLOR_BLUE    "\x1b[34m"
#define ANSI_COLOR_MAGENTA "\x1b[35m"
#define ANSI_COLOR_CYAN    "\x1b[36m"
#define ANSI_COLOR_RESET   "\x1b[0m" "\r\n"    



/***************************************************************************************/
/*                                  Extern Function                                    */
/***************************************************************************************/
extern uint32_t ShutdownCnt_Get(uint8_t index);
extern void ShutdownCnt_Set(uint8_t index,uint8_t val);

extern void PE43711_atten_ctrl(PE43711_st ATT ,uint8_t data);
extern void Uart1_Data_Send(uint8_t* data,uint16_t size);
extern HAL_StatusTypeDef EEPROM_M24C08_Zerowrite(uint8_t devid,uint16_t Address);
extern uint8_t Chksum_Create(uint8_t *data);
extern uint16_t CRC16_Generate(uint8_t* buf_ptr, int len);
/***************************************************************************************/
/*                                  Extern Valuable                                    */
/***************************************************************************************/
extern volatile bool     AlarmTimerOnSet;
extern volatile bool     AlarmTimerOffSet;
extern volatile uint32_t AlarmTimerOnCnt;
extern volatile uint32_t AlarmTimerOffCnt;

extern volatile uint32_t AGC_On_AlarmTimerCnt[AGC_Alarm_DL_Index_MAX];
extern volatile uint32_t AGC_Off_AlarmTimerCnt[AGC_Alarm_DL_Index_MAX];

extern volatile uint32_t DET_DL_Normal_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL_Index_MAX] ;
extern volatile uint32_t DET_UL_Normal_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL_Index_MAX] ;

extern ALL_PE43711_st ALL_ATT;
extern volatile uint16_t ADC1valuearray[4][ADC_AVERAGECNT];
extern volatile uint16_t ADC3valuearray[5][ADC_AVERAGECNT];

extern volatile uint32_t LedTimerCnt;
extern volatile uint32_t AdcTimerCnt;
extern volatile uint32_t DET_UL_On_AlarmTimerCnt[DET_Alarm_UL_Index_MAX];
extern volatile uint32_t DET_UL_Off_AlarmTimerCnt[DET_Alarm_UL_Index_MAX];
extern volatile uint32_t DET_DL_Low_On_AlarmTimerCnt[DET_Alarm_DL_Index_MAX];
extern volatile uint32_t DET_DL_High_On_AlarmTimerCnt[DET_Alarm_DL_Index_MAX];
extern volatile uint32_t DET_DL_Low_Off_AlarmTimerCnt[DET_Alarm_DL_Index_MAX];
extern volatile uint32_t DET_DL_High_Off_AlarmTimerCnt[DET_Alarm_DL_Index_MAX];
extern volatile uint32_t ALC_On_AlarmTimerCnt[ALC_Alarm_UL_Index_MAX];
extern volatile uint32_t ALC_Off_AlarmTimerCnt[ALC_Alarm_UL_Index_MAX];
extern volatile uint32_t DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL_Index_MAX];
extern volatile uint32_t DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL_Index_MAX];
extern volatile uint32_t DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL_Index_MAX];
extern volatile uint32_t DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL_Index_MAX];
extern uint8_t* MBIC_UL_ShutdownCount;
extern uint8_t* MBIC_DL_ShutdownCount;
extern uint8_t* PrevMBIC_UL_ShutdownCount;
extern uint8_t* PrevMBIC_DL_ShutdownCount;

extern volatile uint32_t Alarm_Temp_TimerOffCnt ;
extern volatile uint32_t Alarm_Temp_TimerOnCnt ;

extern volatile uint32_t Alarm_DL_Level_TimerOffCnt ;
extern volatile uint32_t Alarm_DL_Level_TimerOnCnt ;

extern volatile uint32_t Alarm_UL_Level_TimerOffCnt ;
extern volatile uint32_t Alarm_UL_Level_TimerOnCnt ;

/***************************************************************************************/
/*                                  Function                                           */
/***************************************************************************************/

double AutoControl_ADC_Compare(double CurrentAdc,uint8_t* CompareAdc,uint8_t size,int8_t* RefTable_Data);
void Bluecell_StructCpy(uint8_t* dst,uint8_t* src,uint16_t size);
void DataStatusSet(void);
void Alarm_Check();
double TableAtteGuarantee(uint8_t* Table,double AttenValue);
uint8_t PE43711_Calc(uint8_t* Table,uint8_t high_bit,uint8_t low_bit,uint8_t offset_h,uint8_t offset_l);
uint8_t PE43711_Calc_NoTable(uint8_t high_bit,uint8_t low_bit);
double PE43711_Double(uint8_t high_bit,uint8_t low_bit);
void Booting_LED_Check(void);
double Bluecell_TestPro(double value );
double AGC_AutoControl_ADC_Compare(double CurrentAdc,uint8_t* CompareAdc,uint8_t size,int8_t* RefTable_Data);
void ALC_Alarm_TimerSet(uint8_t num,int16_t threshold,int16_t CurrDet,int16_t CurrAtten);



/***************************************************************************************/
/*                                  Valuable                                           */
/***************************************************************************************/

bool Alarm_LED_OnSet = false;
uint8_t DataWrite[sizeof(BLUESTATUS_st)] = {0,};
uint8_t Txdata[512];
int8_t AutoControl_Save[MBIC_Table_INDEX][sizeof(ALC_dBm_t)];
uint16_t ADC1Ret[4];
bool UL_ALC_GainAttenSet[ALC_Alarm_UL_Index_MAX] = {false,};
int16_t ALC_Level_Save[ALC_Alarm_UL_Index_MAX] = {0,};
bool Initialize = false;

uint16_t ADC3Ret[5];

ATT_TABLE_st Att_DL1;
ATT_TABLE_st Att_DL2;
ATT_TABLE_st Att_DL3;
ATT_TABLE_st Att_DL4;
ATT_TABLE_st Att_UL1;
ATT_TABLE_st Att_UL2;
ATT_TABLE_st Att_UL3;
ATT_TABLE_st Att_UL4;

DET_TABLEDL_st Det_DL1;
DET_TABLEDL_st Det_DL2;
DET_TABLEDL_st Det_DL3;
DET_TABLEDL_st Det_DL4;

DET_TABLEUL_st Det_UL1;
DET_TABLEUL_st Det_UL2;
DET_TABLEUL_st Det_UL3;
DET_TABLEUL_st Det_UL4;

TEMP_TABLE_st Temp_DL1;
TEMP_TABLE_st Temp_DL2;
TEMP_TABLE_st Temp_DL3;
TEMP_TABLE_st Temp_DL4;

TEMP_TABLE_st Temp_UL1;
TEMP_TABLE_st Temp_UL2;
TEMP_TABLE_st Temp_UL3;
TEMP_TABLE_st Temp_UL4;

BLUESTATUS_st bluecell_Currdatastatus;
BLUESTATUS_st bluecell_Prevdatastatus;
int16_t DL_AGC_StartAtten[AGC_Alarm_DL_Index_MAX] = {0,};
int16_t DL_PrevIwillgiveAtten[AGC_Alarm_DL_Index_MAX] = {0,};


#if 0 // PYJ.2020.05.21_BEGIN -- 
int8_t AGC_ATTEN_ref[16] = {
    15  ,
    14  ,
    13  ,
    12  ,
    11  ,
    10  ,
    9   ,
    8   ,
    7   ,
    6   ,
    5   ,
    4   ,
    3   ,
    2   ,
    1   ,
    0   ,
};
#endif // PYJ.2020.05.21_END -- 









uint8_t MBIC_TxDataArray[256] = {0,};
void Booting_LedInit(void){
    HAL_GPIO_WritePin(BOOT_LED_GPIO_Port,BOOT_LED_Pin,GPIO_PIN_RESET);
    HAL_GPIO_WritePin(LED_ACT_GPIO_Port,LED_ACT_Pin,GPIO_PIN_SET);
    HAL_GPIO_WritePin(LED_FAIL_GPIO_Port,LED_FAIL_Pin,GPIO_PIN_SET);

}
void Booting_LED_Check(void){
    for(int i = 0; i < 6; i ++){
        HAL_GPIO_TogglePin(BOOT_LED_GPIO_Port,BOOT_LED_Pin);
        HAL_GPIO_TogglePin(LED_ACT_GPIO_Port,LED_ACT_Pin);
        HAL_GPIO_TogglePin(LED_FAIL_GPIO_Port,LED_FAIL_Pin);
        HAL_Delay(500);
    }
}

void MBIC_Table_Load_TempValueConvert(uint8_t* data){
    uint16_t tempvalue = 0;
    for(int i = 0; i < sizeof(TEMP_TABLE_st); i+=2){
        tempvalue = data[i + MBIC_TableIndex_Max] << 8 | data[i + MBIC_TableIndex_Max + 1];
        tempvalue *= 4;
        data[i + MBIC_TableIndex_Max + 0] = ((tempvalue & 0xFF00) >> 8);
        data[i + MBIC_TableIndex_Max + 1] = ((tempvalue & 0x00FF)) ;           
    }

}
void MBIC_Table_Save_TempValueConvert(uint8_t* data){
    uint16_t tempvalue = 0;
    double  ret = 0;
    for(int i = 0; i < sizeof(TEMP_TABLE_st); i+=2){
        tempvalue = data[i + MBIC_TableIndex_Max] << 8 | data[i + MBIC_TableIndex_Max + 1];
        ret = tempvalue / 4;
        ret *=1000;
        tempvalue = (uint16_t)ret;
//        printf("ret : %f\r\n",ret);
        data[i + MBIC_TableIndex_Max + 0] = ((tempvalue & 0xFF00) >> 8);
        data[i + MBIC_TableIndex_Max + 1] = ((tempvalue & 0x00FF)) ;           
    }

}

uint8_t LED_TestCnt = 0;
void Boot_LED_Toggle(void){
  if(bluecell_Currdatastatus.LED_TEST == false){
      if(LedTimerCnt > 1000){
        HAL_GPIO_TogglePin(BOOT_LED_GPIO_Port,BOOT_LED_Pin);
        HAL_GPIO_TogglePin(LED_ACT_GPIO_Port,LED_ACT_Pin);
    //    if(AlarmTimerOnCnt > 3000){
        if(Alarm_LED_OnSet == true){
            HAL_GPIO_WritePin(LED_FAIL_GPIO_Port,LED_FAIL_Pin,GPIO_PIN_SET);
        }else{
            HAL_GPIO_WritePin(LED_FAIL_GPIO_Port,LED_FAIL_Pin,GPIO_PIN_RESET);
        }
    //    printf("Alarm_LED_OnSet : %d\r\n",Alarm_LED_OnSet);
        LedTimerCnt = 0;
      }
  }else{
        if(LedTimerCnt > 500){
            printf("LED TESTING....\r\n");
            if(LED_TestCnt % 2){
                HAL_GPIO_WritePin(BOOT_LED_GPIO_Port,BOOT_LED_Pin,GPIO_PIN_SET);
                HAL_GPIO_WritePin(LED_FAIL_GPIO_Port,LED_FAIL_Pin,GPIO_PIN_RESET);
                HAL_GPIO_WritePin(LED_ACT_GPIO_Port,LED_ACT_Pin,GPIO_PIN_RESET);
            }else{
                HAL_GPIO_WritePin(BOOT_LED_GPIO_Port,BOOT_LED_Pin,GPIO_PIN_RESET);
                HAL_GPIO_WritePin(LED_FAIL_GPIO_Port,LED_FAIL_Pin,GPIO_PIN_SET);
                HAL_GPIO_WritePin(LED_ACT_GPIO_Port,LED_ACT_Pin,GPIO_PIN_SET);

            }
            LED_TestCnt++;
            LedTimerCnt = 0;

          /*Set LED TEST variable False after 12 flashes*/
          if(LED_TestCnt >= 10){
            bluecell_Currdatastatus.LED_TEST = false;
            LED_TestCnt = 0;

          }
        }


  }
  
}





uint8_t PE43711_Calc(uint8_t* Table,uint8_t high_bit,uint8_t low_bit,uint8_t offset_h,uint8_t offset_l){

    double ret = 0;
    uint8_t Result = 0;
    ret = PE43711_Double(high_bit,low_bit); // Hidden Atten Calc
    ret += PE43711_Double(offset_h,offset_l);//Plus User Atten Calc
//    ret += PE43711_Double(0,HIDDENATTEN);//Plus Default Atten 5
//    printf("ret1 : %f \r\n",ret); // 2
    ret = TableAtteGuarantee(Table,ret);//Table Guarantee
//    printf("ret2 : %f \r\n",ret);
    Result = PE43711_DataToHexConvert(ret);
    return Result;
}
uint8_t MBIC_DL_PE43711_Calc(uint8_t* Table,int16_t MBIC_Val,int16_t MBIC_UserVal){

    double ret = 0;
    double ret2 = 0;
    uint8_t Result = 0;
    //ret = PE43711_Double(MBIC_Val); // Hidden Atten Calc
//    printf("MBIC_UserVal : %x \r\n",MBIC_UserVal); // 2
    ret = MBIC_Val;
    ret /= 10;
    ret *= -1;
    ret2 += MBIC_UserVal;
    ret2 /= 10;
    ret2 *= -1;
    ret += ret2;
    
    ret += HIDDENATTEN/10;//PE43711_Double(0,HIDDENATTEN);//Plus Default Atten 5
//    printf("ret1 : %f \r\n",ret); // 2
    ret = TableAtteGuarantee(Table,ret);//Table Guarantee
//    printf("ret2 : %f \r\n",ret);
    Result = PE43711_DataToHexConvert(ret);
    return Result;
}
uint8_t MBIC_UL_PE43711_Calc(uint8_t* Table,int16_t MBIC_Val,int16_t MBIC_UserVal,int16_t MBIC_ALCVal){

    double ret = 0;
    double ret2 = 0;
    double ret3 = 0;
    
    uint8_t Result = 0;
    if(MBIC_Val + MBIC_ALCVal < -200)
        MBIC_Val = MBIC_ALCVal = -100;
    //ret = PE43711_Double(MBIC_Val); // Hidden Atten Calc
//    printf("MBIC_UserVal : %x \r\n",MBIC_UserVal); // 2
    ret = MBIC_Val;
    ret /= 10;
    ret *= -1;
    ret2 += MBIC_UserVal;
    ret2 /= 10;
    ret2 *= -1;
    ret3 += MBIC_ALCVal;
    ret3 /= 10;
    ret3 *= -1;    
    ret =ret + ret2 + ret3;
//    printf("ret1 : %f \r\n",ret); // 2
    ret = TableAtteGuarantee(Table,ret);//Table Guarantee
//    printf("ret2 : %f \r\n",ret);
    Result = PE43711_DataToHexConvert(ret);
    return Result;
}

uint8_t PE43711_Calc_NoTable(uint8_t high_bit,uint8_t low_bit){

    double ret = 0;

    ret = PE43711_Double(high_bit,low_bit);

    PE43711_DataToHexConvert(ret);
    return 0;
}
/*2 byte Data Double Convert Function*/


double PE43711_Double(uint8_t high_bit,uint8_t low_bit){
    //uint16_t tmp_h = 0,tmp_l = 0;
    double ret = 0;
    int16_t tmp = 0;
#if 0 // PYJ.2020.05.22_BEGIN -- 
    tmp_h = high_bit;
    tmp_l = low_bit;
    ret = ((tmp_h << 8) & 0xFF00);
    ret += (tmp_l & 0x00FF);
    /*Minus Convert*/
    if((((tmp_h << 8) & 0xFF00) & 0xF000) == 0xF000){
//        printf("minus Calc Start\r\n");
        ret = 0xFFFF - ret;
//        printf("0xFFFF - %x\r\n",ret);
        ret += 0x01;
        ret = ret - (2*ret);
//        printf("ret :  %x\r\n",ret);        
    }
//    printf("%s 1: ret : %x\r\n",__func__,ret);
    ret /= 10;
#else
    tmp = (int16_t)((high_bit << 8) | low_bit);
    ret = tmp / 10;
#if 0 // PYJ.2020.05.22_BEGIN -- 
    if(ret > 6000){
        printf("high_bit : %x LOW BIT : %x \r\n",high_bit,low_bit);
    }
#endif // PYJ.2020.05.22_END -- 
#endif // PYJ.2020.05.22_END -- 
//    printf("%s 2: ret : %f\r\n",__func__,ret);
    return ret;

}
double TableAtteGuarantee(uint8_t* Table,double AttenValue){
    int8_t GuaranteeData[256];
    double ret = 0;
    //double ref = 0;
    uint8_t cnt = 0;
    for(double ref = 0; ref < AttenValue; ref += 0.5){
        cnt++;       
    }
    
    Bluecell_StructCpy(&GuaranteeData[0],&Table[0],sizeof(ATT_TABLE_st));
    
//    printf("H  : %x   L : %x \r\n",GuaranteeData[cnt * 2],GuaranteeData[cnt * 2 + 1]);
    ret =  GuaranteeData[cnt] / 2;
//= PE43711_Double(GuaranteeData[cnt * 2],GuaranteeData[cnt * 2 + 1]);
//    printf("ret3 : %f \r\n",ret); //1
    ret += AttenValue;
//    printf("ret4 : %f \r\n",ret);
    return ret;

}

#if 1 // PYJ.2020.05.25_BEGIN -- 
AGC_dBm_t AGC_Table_ref[AGC_Table_DL_Ref_Index_MAX] = {
     5   ,    4   ,    3   ,    2   ,
     1   ,    0   ,    -1  ,    -2  ,
    -3  ,    -4  ,    -5  ,    -6  ,
    -7  ,    -8  ,    -9  ,    -10 ,
    -11 ,    -12 ,    -13 ,    -14 ,
    -15 ,    -16 ,    -17 ,    -18 ,
    -19 ,    -20 ,    -21 ,    -22 ,
    -23 ,    -24 ,    -25 ,  

}; 
ALC_dBm_t ALC_Table_ref[ALC_Table_UL_Ref_Index_MAX] = {
    -15 ,    -16 ,    -17 ,    -18 ,  -19 ,
    -20 ,    -21 ,    -22 ,    -23 ,  -24 ,    -25 ,    -26 ,    -27 ,
    -28 ,    -29 ,    -30 ,    -31 ,  -32 ,    -33 ,    -34 ,    -35 ,
    -36 ,    -37 ,    -38 ,    -39 ,  -40 ,    -41 ,    -42 ,    -43 ,
    -44 ,    -45 ,    -46 ,    -47 ,  -48 ,    -49 ,    -50 ,    -51 ,
    -52 ,    -53 ,    -54 ,    -55 ,  -56 ,    -57 ,    -58 ,    -59 ,
    -60 ,

};
#define TableDataSetting_ATT_DL_STEP 0.5 * 10
#define TableDataSetting_ATT_UL_STEP 0.5 * 10
#define TableDataSetting_ATT_DL_LEVEL_STEP 1 * 10
#define TableDataSetting_ATT_UL_LEVEL_STEP 1 * 10
#define TableDataSetting_ATT_DL_TEMP_STEP 5 * 10
#define TableDataSetting_ATT_UL_TEMP_STEP 5 * 10

void Table_DataSetting(int8_t* pdata,int8_t RefData,double step,uint8_t size){
    step /= 10;
    for(int i =0; i < size; i++){
        pdata[i] = RefData;
        RefData -= step; 
    }
}
void Table_Init(){
    Table_DataSetting(&AGC_Table_ref[AGC_Table_DL1_Ref_Index].DET_DL_0,Det_DL1.Table_Ref,TableDataSetting_ATT_DL_LEVEL_STEP,Det_DL1.Table_Length);
    Table_DataSetting(&AGC_Table_ref[AGC_Table_DL2_Ref_Index].DET_DL_0,Det_DL2.Table_Ref,TableDataSetting_ATT_DL_LEVEL_STEP,Det_DL2.Table_Length);
    Table_DataSetting(&AGC_Table_ref[AGC_Table_DL3_Ref_Index].DET_DL_0,Det_DL3.Table_Ref,TableDataSetting_ATT_DL_LEVEL_STEP,Det_DL3.Table_Length);
    Table_DataSetting(&AGC_Table_ref[AGC_Table_DL4_Ref_Index].DET_DL_0,Det_DL4.Table_Ref,TableDataSetting_ATT_DL_LEVEL_STEP,Det_DL4.Table_Length);
    Table_DataSetting(&ALC_Table_ref[ALC_Table_UL1_Ref_Index].DET_UL_0,Det_UL1.Table_Ref,TableDataSetting_ATT_UL_LEVEL_STEP,Det_UL1.Table_Length);
    Table_DataSetting(&ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_0,Det_UL2.Table_Ref,TableDataSetting_ATT_UL_LEVEL_STEP,Det_UL2.Table_Length);
    Table_DataSetting(&ALC_Table_ref[ALC_Table_UL3_Ref_Index].DET_UL_0,Det_UL3.Table_Ref,TableDataSetting_ATT_UL_LEVEL_STEP,Det_UL3.Table_Length);
    Table_DataSetting(&ALC_Table_ref[ALC_Table_UL4_Ref_Index].DET_UL_0,Det_UL4.Table_Ref,TableDataSetting_ATT_UL_LEVEL_STEP,Det_UL4.Table_Length);


}
void Bluecell_AttenInitialize(){
    uint8_t val = 0;
    int16_t MBIC_ALC_Val = 0,MBIC_Val = 0,MBIC_UserVal = 0;
    Table_Init();
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_DL1_INDEX],&AGC_Table_ref[AGC_Table_DL1_Ref_Index].DET_DL_0,sizeof(AGC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_DL2_INDEX],&AGC_Table_ref[AGC_Table_DL2_Ref_Index].DET_DL_0,sizeof(AGC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_DL3_INDEX],&AGC_Table_ref[AGC_Table_DL3_Ref_Index].DET_DL_0,sizeof(AGC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_DL4_INDEX],&AGC_Table_ref[AGC_Table_DL4_Ref_Index].DET_DL_0,sizeof(AGC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_UL1_INDEX],&ALC_Table_ref[ALC_Table_UL1_Ref_Index].DET_UL_0,sizeof(ALC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_UL2_INDEX],&ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_0,sizeof(ALC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_UL3_INDEX],&ALC_Table_ref[ALC_Table_UL3_Ref_Index].DET_UL_0,sizeof(ALC_dBm_t));
//    Bluecell_StructCpy(&AutoControl_Save[MBIC_Table_UL4_INDEX],&ALC_Table_ref[ALC_Table_UL4_Ref_Index].DET_UL_0,sizeof(ALC_dBm_t));

//    for(int a = 0; a < MBIC_Table_INDEX; a++){
//        printf("========================START %d=====================================\r\n",a);
//        if(a <= MBIC_Table_DL4_INDEX)
//        for(int i = 0; i < sizeof(AGC_dBm_t); i++)
//            printf("ref Tabe: %d \r\n",AutoControl_Save[MBIC_Table_DL1_INDEX + a][i]      );         
//        else
//            for(int i = 0; i < sizeof(ALC_dBm_t); i++)
//                printf("ref Tabe: %d \r\n",AutoControl_Save[MBIC_Table_DL1_INDEX + a][i]      ); 
//    }
////    
//    

    bluecell_Prevdatastatus.ATT_DL1_H = bluecell_Currdatastatus.ATT_DL1_H;
    bluecell_Prevdatastatus.ATT_DL1_L = bluecell_Currdatastatus.ATT_DL1_L;
    bluecell_Prevdatastatus.bluecell_User_DL1_H = bluecell_Currdatastatus.bluecell_User_DL1_H;
    bluecell_Prevdatastatus.bluecell_User_DL1_L = bluecell_Currdatastatus.bluecell_User_DL1_L;

    MBIC_Val = bluecell_Currdatastatus.ATT_DL1_H << 8 | bluecell_Currdatastatus.ATT_DL1_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_DL1_H << 8 | bluecell_Currdatastatus.bluecell_User_DL1_L;
    val = MBIC_DL_PE43711_Calc(&Att_DL1.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal);// User Atten Low Bit

    PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    bluecell_Prevdatastatus.ATT_DL2_H = bluecell_Currdatastatus.ATT_DL2_H;
    bluecell_Prevdatastatus.ATT_DL2_L = bluecell_Currdatastatus.ATT_DL2_L;
    bluecell_Prevdatastatus.bluecell_User_DL2_H = bluecell_Currdatastatus.bluecell_User_DL2_H;
    bluecell_Prevdatastatus.bluecell_User_DL2_L = bluecell_Currdatastatus.bluecell_User_DL2_L;
    MBIC_Val = bluecell_Currdatastatus.ATT_DL2_H << 8 | bluecell_Currdatastatus.ATT_DL2_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_DL2_H << 8 | bluecell_Currdatastatus.bluecell_User_DL2_L;
    val = MBIC_DL_PE43711_Calc(&Att_DL2.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal);// User Atten Low Bit

    PE43711_atten_ctrl(ALL_ATT.ATT_DL2,val);

    bluecell_Prevdatastatus.ATT_DL3_H = bluecell_Currdatastatus.ATT_DL3_H;
    bluecell_Prevdatastatus.ATT_DL3_L = bluecell_Currdatastatus.ATT_DL3_L;
    bluecell_Prevdatastatus.bluecell_User_DL3_H = bluecell_Currdatastatus.bluecell_User_DL3_H;
    bluecell_Prevdatastatus.bluecell_User_DL3_L = bluecell_Currdatastatus.bluecell_User_DL3_L;
    MBIC_Val = bluecell_Currdatastatus.ATT_DL3_H << 8 | bluecell_Currdatastatus.ATT_DL3_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_DL3_H << 8 | bluecell_Currdatastatus.bluecell_User_DL3_L;
    printf("DL 3 MAIN ATTEN :%x \r\n",MBIC_Val);
    printf("DL 3 OFSSET ATTEN :%x \r\n",MBIC_UserVal);    
    val = MBIC_DL_PE43711_Calc(&Att_DL3.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal);// User Atten Low Bit

    PE43711_atten_ctrl(ALL_ATT.ATT_DL3,val);

    bluecell_Prevdatastatus.ATT_DL4_H = bluecell_Currdatastatus.ATT_DL4_H;
    bluecell_Prevdatastatus.ATT_DL4_L = bluecell_Currdatastatus.ATT_DL4_L;
    bluecell_Prevdatastatus.bluecell_User_DL4_H = bluecell_Currdatastatus.bluecell_User_DL4_H;
    bluecell_Prevdatastatus.bluecell_User_DL4_L = bluecell_Currdatastatus.bluecell_User_DL4_L;
    MBIC_Val = bluecell_Currdatastatus.ATT_DL4_H << 8 | bluecell_Currdatastatus.ATT_DL4_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_DL4_H << 8 | bluecell_Currdatastatus.bluecell_User_DL4_L;
    val = MBIC_DL_PE43711_Calc(&Att_DL4.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal);// User Atten Low Bit

    PE43711_atten_ctrl(ALL_ATT.ATT_DL4,val);



    bluecell_Prevdatastatus.ATT_UL1_H = bluecell_Currdatastatus.ATT_UL1_H;
    bluecell_Prevdatastatus.ATT_UL1_L = bluecell_Currdatastatus.ATT_UL1_L;
    bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
    bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten1_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten1_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L;    
    MBIC_Val = bluecell_Currdatastatus.ATT_UL1_H << 8 | bluecell_Currdatastatus.ATT_UL1_L;
    MBIC_ALC_Val = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_UL1_H << 8 | bluecell_Currdatastatus.bluecell_User_UL1_L;
    val = MBIC_UL_PE43711_Calc(&Att_UL1.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal,
                       MBIC_ALC_Val);// User Atten Low Bit

//        printf("%d val = %x \r\n",__LINE__,val);
    PE43711_atten_ctrl(ALL_ATT.ATT_UL1,val);

                bluecell_Prevdatastatus.ATT_UL2_H = bluecell_Currdatastatus.ATT_UL2_H;
    bluecell_Prevdatastatus.ATT_UL2_L = bluecell_Currdatastatus.ATT_UL2_L;
    bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
    bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten2_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten2_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L;    
    MBIC_Val = bluecell_Currdatastatus.ATT_UL2_H << 8 | bluecell_Currdatastatus.ATT_UL2_L;
    bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H = 0;
    bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L = 0;
    MBIC_ALC_Val = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_UL2_H << 8 | bluecell_Currdatastatus.bluecell_User_UL2_L;
    printf("MBIC_ALC_Val : %x \r\n",MBIC_ALC_Val);
    val = MBIC_UL_PE43711_Calc(&Att_UL1.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal,
                       MBIC_ALC_Val);// User Atten Low Bit

//        printf("%d val = %x \r\n",__LINE__,val);
    PE43711_atten_ctrl(ALL_ATT.ATT_UL2,val);


                bluecell_Prevdatastatus.ATT_UL3_H = bluecell_Currdatastatus.ATT_UL3_H;
    bluecell_Prevdatastatus.ATT_UL3_L = bluecell_Currdatastatus.ATT_UL3_L;
    bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
    bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten3_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten3_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L;    
    MBIC_Val = bluecell_Currdatastatus.ATT_UL3_H << 8 | bluecell_Currdatastatus.ATT_UL3_L;
    MBIC_ALC_Val = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_UL3_H << 8 | bluecell_Currdatastatus.bluecell_User_UL3_L;
    val = MBIC_UL_PE43711_Calc(&Att_UL3.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal,
                       MBIC_ALC_Val);// User Atten Low Bit

//        printf("%d val = %x \r\n",__LINE__,val);
    PE43711_atten_ctrl(ALL_ATT.ATT_UL3,val);


    bluecell_Prevdatastatus.ATT_UL4_H = bluecell_Currdatastatus.ATT_UL4_H;
    bluecell_Prevdatastatus.ATT_UL4_L = bluecell_Currdatastatus.ATT_UL4_L;
    bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
    bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten4_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten4_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;    
    MBIC_Val = bluecell_Currdatastatus.ATT_UL4_H << 8 | bluecell_Currdatastatus.ATT_UL4_L;
    MBIC_ALC_Val = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;
    MBIC_UserVal = bluecell_Currdatastatus.bluecell_User_UL4_H << 8 | bluecell_Currdatastatus.bluecell_User_UL4_L;
    val = MBIC_UL_PE43711_Calc(&Att_UL4.Table_0_0_dBm, // Table Offset
                        MBIC_Val,
                       MBIC_UserVal,
                       MBIC_ALC_Val);// User Atten Low Bit

//        printf("%d val = %x \r\n",__LINE__,val);
    PE43711_atten_ctrl(ALL_ATT.ATT_UL4,val);

}
void CompareAttenData(BLUESTATUS_st Curr,BLUESTATUS_st Prev){
     uint8_t val = 0;
     uint16_t MBIC_Val = 0;
     uint16_t MBIC_ALC_Val = 0;
     uint16_t MBIC_UserVal = 0;

    if((Curr.ATT_DL1_H != Prev.ATT_DL1_H
     ||Curr.ATT_DL1_L != Prev.ATT_DL1_L)
     ||(Curr.bluecell_User_DL1_H != Prev.bluecell_User_DL1_H
     ||Curr.bluecell_User_DL1_L != Prev.bluecell_User_DL1_L)){
//         printf("%s : %d \r\n",__func__,__LINE__);   
            bluecell_Prevdatastatus.ATT_DL1_H = bluecell_Currdatastatus.ATT_DL1_H;
            bluecell_Prevdatastatus.ATT_DL1_L = bluecell_Currdatastatus.ATT_DL1_L;
            bluecell_Prevdatastatus.bluecell_User_DL1_H = bluecell_Currdatastatus.bluecell_User_DL1_H;
            bluecell_Prevdatastatus.bluecell_User_DL1_L = bluecell_Currdatastatus.bluecell_User_DL1_L;
            MBIC_Val =Curr.ATT_DL1_H << 8 | Curr.ATT_DL1_L;
            MBIC_UserVal = Curr.bluecell_User_DL1_H << 8 | Curr.bluecell_User_DL1_L;
            val = MBIC_DL_PE43711_Calc(&Att_DL1.Table_0_0_dBm, // Table Offset
                                MBIC_Val,
                               MBIC_UserVal);// User Atten Low Bit
            printf("%s : %d \r\n",__func__,__LINE__);   
            printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);
    }
    if((Curr.ATT_DL2_H != Prev.ATT_DL2_H)
     ||(Curr.ATT_DL2_L != Prev.ATT_DL2_L)
     ||(Curr.bluecell_User_DL2_H != Prev.bluecell_User_DL2_H
     ||Curr.bluecell_User_DL2_L != Prev.bluecell_User_DL2_L)){
            bluecell_Prevdatastatus.ATT_DL2_H = bluecell_Currdatastatus.ATT_DL2_H;
            bluecell_Prevdatastatus.ATT_DL2_L = bluecell_Currdatastatus.ATT_DL2_L;
            bluecell_Prevdatastatus.bluecell_User_DL2_H = bluecell_Currdatastatus.bluecell_User_DL2_H;
            bluecell_Prevdatastatus.bluecell_User_DL2_L = bluecell_Currdatastatus.bluecell_User_DL2_L;
            MBIC_Val = Curr.ATT_DL2_H << 8 | Curr.ATT_DL2_L;
            MBIC_UserVal = Curr.bluecell_User_DL2_H << 8 | Curr.bluecell_User_DL2_L;
            val = MBIC_DL_PE43711_Calc(&Att_DL2.Table_0_0_dBm,// Table Offset
                                MBIC_Val,// Hidden Atten Low bit
                               MBIC_UserVal);// User Atten Low Bit
//            printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_DL2,val);
    }
     
    if((Curr.ATT_DL3_H != Prev.ATT_DL3_H
     ||Curr.ATT_DL3_L != Prev.ATT_DL3_L)
     ||(Curr.bluecell_User_DL3_H != Prev.bluecell_User_DL3_H
     ||Curr.bluecell_User_DL3_L != Prev.bluecell_User_DL3_L)){

        bluecell_Prevdatastatus.ATT_DL3_H = bluecell_Currdatastatus.ATT_DL3_H;
        bluecell_Prevdatastatus.ATT_DL3_L = bluecell_Currdatastatus.ATT_DL3_L;
        bluecell_Prevdatastatus.bluecell_User_DL3_H = bluecell_Currdatastatus.bluecell_User_DL3_H;
        bluecell_Prevdatastatus.bluecell_User_DL3_L = bluecell_Currdatastatus.bluecell_User_DL3_L;
            MBIC_Val = Curr.ATT_DL3_H << 8 | Curr.ATT_DL3_L;
            MBIC_UserVal = Curr.bluecell_User_DL3_H << 8 | Curr.bluecell_User_DL3_L;
            
            val = MBIC_DL_PE43711_Calc(&Att_DL3.Table_0_0_dBm,// Table Offset
                            MBIC_Val,// Hidden Atten Low bit
                           MBIC_UserVal);// User Atten Low Bit
        printf("%d val = %x \r\n",__LINE__,val);
        PE43711_atten_ctrl(ALL_ATT.ATT_DL3,val);
    }
//    printf("\r\nCurr.ATT_DL3_H : %x \r\nPrev.ATT_DL3_H : %x \r\n",Curr.ATT_DL3_H,Prev.ATT_DL3_H);
//     printf("\r\nCurr.ATT_DL3_L : %x \r\nPrev.ATT_DL3_L : %x \r\n",Curr.ATT_DL3_L,Prev.ATT_DL3_L);
     
    if((Curr.ATT_DL4_H != Prev.ATT_DL4_H
     ||Curr.ATT_DL4_L != Prev.ATT_DL4_L)
     ||(Curr.bluecell_User_DL4_H != Prev.bluecell_User_DL4_H
     ||Curr.bluecell_User_DL4_L != Prev.bluecell_User_DL4_L)){
     
            bluecell_Prevdatastatus.ATT_DL4_H = bluecell_Currdatastatus.ATT_DL4_H;
            bluecell_Prevdatastatus.ATT_DL4_L = bluecell_Currdatastatus.ATT_DL4_L;
            bluecell_Prevdatastatus.bluecell_User_DL4_H = bluecell_Currdatastatus.bluecell_User_DL4_H;
            bluecell_Prevdatastatus.bluecell_User_DL4_L = bluecell_Currdatastatus.bluecell_User_DL4_L;
            MBIC_Val = (Curr.ATT_DL4_H << 8 | Curr.ATT_DL4_L);
            printf("\r\nCurr.ATT_DL4_H : %x \rCurr.ATT_DL4_L : %x \r\n",Curr.ATT_DL4_H,Curr.ATT_DL4_L);
            MBIC_UserVal = Curr.bluecell_User_DL4_H << 8 | Curr.bluecell_User_DL4_L;
            printf("\r\nCurr.bluecell_User_DL4_H : %x \rCurr.bluecell_User_DL4_L : %x \r\n",Curr.bluecell_User_DL4_H,Curr.bluecell_User_DL4_L);
            printf("MBIC_UserVal : %x \r\n",MBIC_UserVal); // 2
            val = MBIC_DL_PE43711_Calc(&Att_DL4.Table_0_0_dBm,// Table Offset
                                MBIC_Val,// Hidden Atten Low bit
                               MBIC_UserVal);// User Atten Low Bit
            printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_DL4,val);
    }

    
    if((Curr.ATT_UL1_H != Prev.ATT_UL1_H
     ||Curr.ATT_UL1_L != Prev.ATT_UL1_L)
     ||(Curr.bluecell_User_UL1_H != Prev.bluecell_User_UL1_H
     ||Curr.bluecell_User_UL1_L != Prev.bluecell_User_UL1_L)
     ||(Curr.MBIC_ULO_ALC_Atten1_H != Prev.MBIC_ULO_ALC_Atten1_H
     ||Curr.MBIC_ULO_ALC_Atten1_L != Prev.MBIC_ULO_ALC_Atten1_L)){
     
            bluecell_Prevdatastatus.ATT_UL1_H = bluecell_Currdatastatus.ATT_UL1_H;
            bluecell_Prevdatastatus.ATT_UL1_L = bluecell_Currdatastatus.ATT_UL1_L;
            bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
            bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten1_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H;
    bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten1_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L;    
    
            MBIC_Val = Curr.ATT_UL1_H << 8 | Curr.ATT_UL1_L;
            MBIC_ALC_Val = Curr.MBIC_ULO_ALC_Atten1_H << 8 | Curr.MBIC_ULO_ALC_Atten1_L;
            MBIC_UserVal = Curr.bluecell_User_UL1_H << 8 | Curr.bluecell_User_UL1_L;
            val = MBIC_UL_PE43711_Calc(&Att_UL1.Table_0_0_dBm,// Table Offset
                                MBIC_Val,// Hidden Atten Low bit
                               MBIC_UserVal,
                               MBIC_ALC_Val);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_UL1,val);
    }
    if((Curr.ATT_UL2_H != Prev.ATT_UL2_H
     ||Curr.ATT_UL2_L != Prev.ATT_UL2_L)
     ||(Curr.bluecell_User_UL2_H != Prev.bluecell_User_UL2_H
     ||Curr.bluecell_User_UL2_L != Prev.bluecell_User_UL2_L)
     ||(Curr.MBIC_ULO_ALC_Atten2_H != Prev.MBIC_ULO_ALC_Atten2_H
     ||Curr.MBIC_ULO_ALC_Atten2_L != Prev.MBIC_ULO_ALC_Atten2_L)){
            bluecell_Prevdatastatus.ATT_UL2_H = bluecell_Currdatastatus.ATT_UL2_H;
            bluecell_Prevdatastatus.ATT_UL2_L = bluecell_Currdatastatus.ATT_UL2_L;
            bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
            bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten2_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten2_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L;    
            

            MBIC_Val = Curr.ATT_UL2_H << 8 | Curr.ATT_UL2_L;
            MBIC_ALC_Val = Curr.MBIC_ULO_ALC_Atten2_H << 8 | Curr.MBIC_ULO_ALC_Atten2_L;
            MBIC_UserVal = Curr.bluecell_User_UL2_H << 8 | Curr.bluecell_User_UL2_L;
            printf("MBIC_Val : %02X \r\n",MBIC_Val);
            printf("MBIC_UserVal : %02X \r\n",MBIC_UserVal);
            printf("MBIC_ALC_Val : %02X \r\n",MBIC_ALC_Val);
            val = MBIC_UL_PE43711_Calc(&Att_UL2.Table_0_0_dBm,// Table Offset
                                MBIC_Val,// Hidden Atten Low bit
                               MBIC_UserVal,
                               MBIC_ALC_Val);// User Atten Low Bit
            printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_UL2,val);
    }
     
    if((Curr.ATT_UL3_H != Prev.ATT_UL3_H
     ||Curr.ATT_UL3_L != Prev.ATT_UL3_L)
     ||(Curr.bluecell_User_UL3_H != Prev.bluecell_User_UL3_H
     ||Curr.bluecell_User_UL3_L != Prev.bluecell_User_UL3_L)
     ||(Curr.MBIC_ULO_ALC_Atten3_H != Prev.MBIC_ULO_ALC_Atten3_H
     ||Curr.MBIC_ULO_ALC_Atten3_L != Prev.MBIC_ULO_ALC_Atten3_L))
     {
            bluecell_Prevdatastatus.ATT_UL3_H = bluecell_Currdatastatus.ATT_UL3_H;
            bluecell_Prevdatastatus.ATT_UL3_L = bluecell_Currdatastatus.ATT_UL3_L;
            bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
            bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten3_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten3_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L;    
            MBIC_Val = Curr.ATT_UL3_H << 8 | Curr.ATT_UL3_L;
            MBIC_ALC_Val = Curr.MBIC_ULO_ALC_Atten3_H << 8 | Curr.MBIC_ULO_ALC_Atten3_L;
            MBIC_UserVal = Curr.bluecell_User_UL3_H << 8 | Curr.bluecell_User_UL3_L;
            val = MBIC_UL_PE43711_Calc(&Att_UL3.Table_0_0_dBm,// Table Offset
                                MBIC_Val,// Hidden Atten Low bit
                               MBIC_UserVal,
                               MBIC_ALC_Val);// User Atten Low Bit
//    printf("bluecell_Currdatastatus.ATT_UL3_H : %0x \r\n",bluecell_Currdatastatus.ATT_UL3_H);
//    printf("bluecell_Currdatastatus.ATT_UL3_L : %0x \r\n",bluecell_Currdatastatus.ATT_UL3_L);
//    printf("bluecell_Currdatastatus.bluecell_User_UL3_H : %0x \r\n",bluecell_Currdatastatus.bluecell_User_UL3_H);
//    printf("bluecell_Currdatastatus.bluecell_User_UL3_L : %0x \r\n",bluecell_Currdatastatus.bluecell_User_UL3_L);        
        printf("%d val = %x \r\n",__LINE__,val);

            PE43711_atten_ctrl(ALL_ATT.ATT_UL3,val);
    }
    if((Curr.ATT_UL4_H != Prev.ATT_UL4_H
     ||Curr.ATT_UL4_L != Prev.ATT_UL4_L)
     ||(Curr.bluecell_User_UL4_H != Prev.bluecell_User_UL4_H
     ||Curr.bluecell_User_UL4_L != Prev.bluecell_User_UL4_L)
     ||(Curr.MBIC_ULO_ALC_Atten4_H != Prev.MBIC_ULO_ALC_Atten4_H
     ||Curr.MBIC_ULO_ALC_Atten4_L != Prev.MBIC_ULO_ALC_Atten4_L))
     {
            bluecell_Prevdatastatus.ATT_UL4_H = bluecell_Currdatastatus.ATT_UL4_H;
            bluecell_Prevdatastatus.ATT_UL4_L = bluecell_Currdatastatus.ATT_UL4_L;
            bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
            bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten4_H = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H;
            bluecell_Prevdatastatus.MBIC_ULO_ALC_Atten4_L = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;
            MBIC_Val = Curr.ATT_UL4_H << 8 | Curr.ATT_UL4_L;
            MBIC_ALC_Val = Curr.MBIC_ULO_ALC_Atten4_H << 8 | Curr.MBIC_ULO_ALC_Atten4_L;
            MBIC_UserVal = Curr.bluecell_User_UL4_H << 8 | Curr.bluecell_User_UL4_L;
            val = MBIC_UL_PE43711_Calc(&Att_UL4.Table_0_0_dBm,// Table Offset
                                MBIC_Val,// Hidden Atten Low bit
                               MBIC_UserVal,
                               MBIC_ALC_Val);// User Atten Low Bit
        printf("bluecell_Currdatastatus.ATT_UL4_H : %0x \r\n",bluecell_Currdatastatus.ATT_UL4_H);
        printf("bluecell_Currdatastatus.ATT_UL4_L : %0x \r\n",bluecell_Currdatastatus.ATT_UL4_L);
        printf("bluecell_Currdatastatus.bluecell_User_UL4_H : %0x \r\n",bluecell_Currdatastatus.bluecell_User_UL4_H);
        printf("bluecell_Currdatastatus.bluecell_User_UL4_L : %0x \r\n",bluecell_Currdatastatus.bluecell_User_UL4_L);        
            printf("%d val = %x \r\n",__LINE__,val);
            PE43711_UL4_atten_ctrl(val);
    }
//    memcpy(&bluecell_Prevdatastatus.ATT_DL1_H,&bluecell_Currdatastatus.ATT_DL1_H,32);
#if 0 // PYJ.2020.04.21_BEGIN -- 
    
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL1_H != Prev.bluecell_User_UL1_H
     ||Curr.bluecell_User_UL1_L != Prev.bluecell_User_UL1_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
        bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
        val = PE43711_Calc(&Att_UL1.Table_0_0_dBm,
                            Curr.ULO_ALC_Threshold_H,
                            Curr.ULO_ALC_Threshold_L,
                           bluecell_Currdatastatus.bluecell_User_DL1_H,
                           bluecell_Currdatastatus.bluecell_User_DL1_L);
//        val = PE43711_DataToHexConvert(ret);
//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL2_H != Prev.bluecell_User_UL2_H
     ||Curr.bluecell_User_UL2_L != Prev.bluecell_User_UL2_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
        bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
        val = PE43711_Calc(&Att_UL2.Table_0_0_dBm,
                            Curr.ATT_ALC2_MAX_H,
                            Curr.ATT_ALC2_MAX_L,
                           bluecell_Currdatastatus.bluecell_User_DL2_H,
                           bluecell_Currdatastatus.bluecell_User_DL2_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL3_H != Prev.bluecell_User_UL3_H
     ||Curr.bluecell_User_UL3_L != Prev.bluecell_User_UL3_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
        bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
        val = PE43711_Calc(&Att_UL3.Table_0_0_dBm,
            Curr.ATT_ALC3_MAX_H,
            Curr.ATT_ALC3_MAX_L,
                           bluecell_Currdatastatus.bluecell_User_DL3_H,
                           bluecell_Currdatastatus.bluecell_User_DL3_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL4_H != Prev.bluecell_User_UL4_H
     ||Curr.bluecell_User_UL4_L != Prev.bluecell_User_UL4_L)){
//        printf("Curr.ATT_ALC4_MAX_H : %x Curr.ATT_ALC4_MAX_L : %x\r\n",Curr.ATT_ALC4_MAX_H,Curr.ATT_ALC4_MAX_L);
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
        bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
        val = PE43711_Calc(&Att_UL4.Table_0_0_dBm,
            Curr.ATT_ALC4_MAX_H,
            Curr.ATT_ALC4_MAX_L,
                           bluecell_Currdatastatus.bluecell_User_DL4_H,
                           bluecell_Currdatastatus.bluecell_User_DL4_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }

    
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL1_H != Prev.bluecell_User_UL1_H
     ||Curr.bluecell_User_UL1_L != Prev.bluecell_User_UL1_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
        bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
        val = PE43711_Calc(&Att_DL1.Table_0_0_dBm,
            Curr.ULO_ALC_Threshold_H,
            Curr.ULO_ALC_Threshold_L,
                           bluecell_Currdatastatus.bluecell_User_UL1_H,
                           bluecell_Currdatastatus.bluecell_User_UL1_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ATT_ALC2_MIN_H != Prev.ATT_ALC2_MIN_H
     ||Curr.ATT_ALC2_MIN_L != Prev.ATT_ALC2_MIN_L)
     ||(Curr.bluecell_User_UL2_H != Prev.bluecell_User_UL2_H
     ||Curr.bluecell_User_UL2_L != Prev.bluecell_User_UL2_L)){
        bluecell_Prevdatastatus.ATT_ALC2_MIN_H = bluecell_Currdatastatus.ATT_ALC2_MIN_H;
        bluecell_Prevdatastatus.ATT_ALC2_MIN_L = bluecell_Currdatastatus.ATT_ALC2_MIN_L;
        bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
        bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
        val = PE43711_Calc(&Att_DL2.Table_0_0_dBm,
            Curr.ATT_ALC2_MIN_H,
            Curr.ATT_ALC2_MIN_L,
                           bluecell_Currdatastatus.bluecell_User_UL2_H,
                           bluecell_Currdatastatus.bluecell_User_UL2_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ATT_ALC3_MIN_H != Prev.ATT_ALC3_MIN_H
     ||Curr.ATT_ALC3_MIN_L != Prev.ATT_ALC3_MIN_L)
     ||(Curr.bluecell_User_UL3_H != Prev.bluecell_User_UL3_H
     ||Curr.bluecell_User_UL3_L != Prev.bluecell_User_UL3_L)){
        bluecell_Prevdatastatus.ATT_ALC3_MIN_H = bluecell_Currdatastatus.ATT_ALC3_MIN_H;
        bluecell_Prevdatastatus.ATT_ALC3_MIN_L = bluecell_Currdatastatus.ATT_ALC3_MIN_L;
        bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
        bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
        val = PE43711_Calc(&Att_DL3.Table_0_0_dBm,
            Curr.ATT_ALC3_MIN_H,
            Curr.ATT_ALC3_MIN_L,
                           bluecell_Currdatastatus.bluecell_User_UL3_H,
                           bluecell_Currdatastatus.bluecell_User_UL3_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ATT_ALC4_MIN_H != Prev.ATT_ALC4_MIN_H
     ||Curr.ATT_ALC4_MIN_L != Prev.ATT_ALC4_MIN_L)
     ||(Curr.bluecell_User_UL4_H != Prev.bluecell_User_UL4_H
     ||Curr.bluecell_User_UL4_L != Prev.bluecell_User_UL4_L)){
        bluecell_Prevdatastatus.ATT_ALC4_MIN_H = bluecell_Currdatastatus.ATT_ALC4_MIN_H;
        bluecell_Prevdatastatus.ATT_ALC4_MIN_L = bluecell_Currdatastatus.ATT_ALC4_MIN_L;
        bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
        bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
        val = PE43711_Calc(&Att_DL4.Table_0_0_dBm,
            Curr.ATT_ALC4_MIN_H,
            Curr.ATT_ALC4_MIN_L,
                           bluecell_Currdatastatus.bluecell_User_UL4_H,
                           bluecell_Currdatastatus.bluecell_User_UL4_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
#endif // PYJ.2020.04.21_END -- 
     
}
#else
void CompareAttenData(BLUESTATUS_st Curr,BLUESTATUS_st Prev){
     uint8_t val = 0;


            bluecell_Prevdatastatus.ATT_DL1_H = bluecell_Currdatastatus.ATT_DL1_H;
            bluecell_Prevdatastatus.ATT_DL1_L = bluecell_Currdatastatus.ATT_DL1_L;
            bluecell_Prevdatastatus.bluecell_User_DL1_H = bluecell_Currdatastatus.bluecell_User_DL1_H;
            bluecell_Prevdatastatus.bluecell_User_DL1_L = bluecell_Currdatastatus.bluecell_User_DL1_L;
            val = PE43711_Calc(&Att_DL1.Table_0_0_dBm, // Table Offset
                                Curr.ATT_DL1_H,         // Hidden Atten High bit
                                Curr.ATT_DL1_L,         // Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_DL1_H, // User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_DL1_L);// User Atten Low Bit
//            printf("%s : %d \r\n",__func__,__LINE__);   
//            printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);
     
            bluecell_Prevdatastatus.ATT_DL2_H = bluecell_Currdatastatus.ATT_DL2_H;
            bluecell_Prevdatastatus.ATT_DL2_L = bluecell_Currdatastatus.ATT_DL2_L;
            bluecell_Prevdatastatus.bluecell_User_DL2_H = bluecell_Currdatastatus.bluecell_User_DL2_H;
            bluecell_Prevdatastatus.bluecell_User_DL2_L = bluecell_Currdatastatus.bluecell_User_DL2_L;
            val = PE43711_Calc(&Att_DL2.Table_0_0_dBm,// Table Offset
                                Curr.ATT_DL2_H,// Hidden Atten High bit
                                Curr.ATT_DL2_L,// Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_DL2_H,// User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_DL2_L);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_DL2,val);
     

        bluecell_Prevdatastatus.ATT_DL3_H = bluecell_Currdatastatus.ATT_DL3_H;
        bluecell_Prevdatastatus.ATT_DL3_L = bluecell_Currdatastatus.ATT_DL3_L;
        bluecell_Prevdatastatus.bluecell_User_DL3_H = bluecell_Currdatastatus.bluecell_User_DL3_H;
        bluecell_Prevdatastatus.bluecell_User_DL3_L = bluecell_Currdatastatus.bluecell_User_DL3_L;
        val = PE43711_Calc(&Att_DL3.Table_0_0_dBm,// Table Offset
                            Curr.ATT_DL3_H,// Hidden Atten High bit
                            Curr.ATT_DL3_L,// Hidden Atten Low bit
                           bluecell_Currdatastatus.bluecell_User_DL3_H,// User Atten High Bit
                           bluecell_Currdatastatus.bluecell_User_DL3_L);// User Atten Low Bit
//        printf("%d val = %x \r\n",__LINE__,val);
        PE43711_atten_ctrl(ALL_ATT.ATT_DL3,val);
//    printf("\r\nCurr.ATT_DL3_H : %x \r\nPrev.ATT_DL3_H : %x \r\n",Curr.ATT_DL3_H,Prev.ATT_DL3_H);
//     printf("\r\nCurr.ATT_DL3_L : %x \r\nPrev.ATT_DL3_L : %x \r\n",Curr.ATT_DL3_L,Prev.ATT_DL3_L);
     
     
            bluecell_Prevdatastatus.ATT_DL4_H = bluecell_Currdatastatus.ATT_DL4_H;
            bluecell_Prevdatastatus.ATT_DL4_L = bluecell_Currdatastatus.ATT_DL4_L;
            bluecell_Prevdatastatus.bluecell_User_DL4_H = bluecell_Currdatastatus.bluecell_User_DL4_H;
            bluecell_Prevdatastatus.bluecell_User_DL4_L = bluecell_Currdatastatus.bluecell_User_DL4_L;
            val = PE43711_Calc(&Att_DL4.Table_0_0_dBm,// Table Offset
                                Curr.ATT_DL4_H,// Hidden Atten High bit
                                Curr.ATT_DL4_L,// Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_DL4_H,// User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_DL4_L);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_DL4,val);

    
     
            bluecell_Prevdatastatus.ATT_UL1_H = bluecell_Currdatastatus.ATT_UL1_H;
            bluecell_Prevdatastatus.ATT_UL1_L = bluecell_Currdatastatus.ATT_UL1_L;
            bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
            bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
            val = PE43711_Calc(&Att_UL1.Table_0_0_dBm,// Table Offset
                                Curr.ATT_UL1_H,// Hidden Atten High bit
                                Curr.ATT_UL1_L,// Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_UL1_H,// User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_UL1_L);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_UL1,val);
            bluecell_Prevdatastatus.ATT_UL2_H = bluecell_Currdatastatus.ATT_UL2_H;
            bluecell_Prevdatastatus.ATT_UL2_L = bluecell_Currdatastatus.ATT_UL2_L;
            bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
            bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
            val = PE43711_Calc(&Att_UL2.Table_0_0_dBm,// Table Offset
                                Curr.ATT_UL2_H,// Hidden Atten High bit
                                Curr.ATT_UL2_L,// Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_UL2_H,// User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_UL2_L);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_UL2,val);
     
            bluecell_Prevdatastatus.ATT_UL3_H = bluecell_Currdatastatus.ATT_UL3_H;
            bluecell_Prevdatastatus.ATT_UL3_L = bluecell_Currdatastatus.ATT_UL3_L;
            bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
            bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
            
            val = PE43711_Calc(&Att_UL3.Table_0_0_dBm,// Table Offset
                                Curr.ATT_UL3_H,// Hidden Atten High bit
                                Curr.ATT_UL3_L,// Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_UL3_H,// User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_UL3_L);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_UL3,val);
            bluecell_Prevdatastatus.ATT_UL4_H = bluecell_Currdatastatus.ATT_UL4_H;
            bluecell_Prevdatastatus.ATT_UL4_L = bluecell_Currdatastatus.ATT_UL4_L;
            bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
            bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
            
            val = PE43711_Calc(&Att_UL4.Table_0_0_dBm,// Table Offset
                                Curr.ATT_UL4_H,// Hidden Atten High bit
                                Curr.ATT_UL4_L,// Hidden Atten Low bit
                               bluecell_Currdatastatus.bluecell_User_UL4_H,// User Atten High Bit
                               bluecell_Currdatastatus.bluecell_User_UL4_L);// User Atten Low Bit
    //        printf("%d val = %x \r\n",__LINE__,val);
            PE43711_atten_ctrl(ALL_ATT.ATT_UL4,val);
//    memcpy(&bluecell_Prevdatastatus.ATT_DL1_H,&bluecell_Currdatastatus.ATT_DL1_H,32);
#if 0 // PYJ.2020.04.21_BEGIN -- 
    
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL1_H != Prev.bluecell_User_UL1_H
     ||Curr.bluecell_User_UL1_L != Prev.bluecell_User_UL1_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
        bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
        val = PE43711_Calc(&Att_UL1.Table_0_0_dBm,
                            Curr.ULO_ALC_Threshold_H,
                            Curr.ULO_ALC_Threshold_L,
                           bluecell_Currdatastatus.bluecell_User_DL1_H,
                           bluecell_Currdatastatus.bluecell_User_DL1_L);
//        val = PE43711_DataToHexConvert(ret);
//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL2_H != Prev.bluecell_User_UL2_H
     ||Curr.bluecell_User_UL2_L != Prev.bluecell_User_UL2_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
        bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
        val = PE43711_Calc(&Att_UL2.Table_0_0_dBm,
                            Curr.ATT_ALC2_MAX_H,
                            Curr.ATT_ALC2_MAX_L,
                           bluecell_Currdatastatus.bluecell_User_DL2_H,
                           bluecell_Currdatastatus.bluecell_User_DL2_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL3_H != Prev.bluecell_User_UL3_H
     ||Curr.bluecell_User_UL3_L != Prev.bluecell_User_UL3_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
        bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
        val = PE43711_Calc(&Att_UL3.Table_0_0_dBm,
            Curr.ATT_ALC3_MAX_H,
            Curr.ATT_ALC3_MAX_L,
                           bluecell_Currdatastatus.bluecell_User_DL3_H,
                           bluecell_Currdatastatus.bluecell_User_DL3_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL4_H != Prev.bluecell_User_UL4_H
     ||Curr.bluecell_User_UL4_L != Prev.bluecell_User_UL4_L)){
//        printf("Curr.ATT_ALC4_MAX_H : %x Curr.ATT_ALC4_MAX_L : %x\r\n",Curr.ATT_ALC4_MAX_H,Curr.ATT_ALC4_MAX_L);
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
        bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
        val = PE43711_Calc(&Att_UL4.Table_0_0_dBm,
            Curr.ATT_ALC4_MAX_H,
            Curr.ATT_ALC4_MAX_L,
                           bluecell_Currdatastatus.bluecell_User_DL4_H,
                           bluecell_Currdatastatus.bluecell_User_DL4_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }

    
    if((Curr.ULO_ALC_Threshold_H != Prev.ULO_ALC_Threshold_H
     ||Curr.ULO_ALC_Threshold_L != Prev.ULO_ALC_Threshold_L)
     ||(Curr.bluecell_User_UL1_H != Prev.bluecell_User_UL1_H
     ||Curr.bluecell_User_UL1_L != Prev.bluecell_User_UL1_L)){
        bluecell_Prevdatastatus.ULO_ALC_Threshold_H = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
        bluecell_Prevdatastatus.ULO_ALC_Threshold_L = bluecell_Currdatastatus.ULO_ALC_Threshold_L;
        bluecell_Prevdatastatus.bluecell_User_UL1_H = bluecell_Currdatastatus.bluecell_User_UL1_H;
        bluecell_Prevdatastatus.bluecell_User_UL1_L = bluecell_Currdatastatus.bluecell_User_UL1_L;
        val = PE43711_Calc(&Att_DL1.Table_0_0_dBm,
            Curr.ULO_ALC_Threshold_H,
            Curr.ULO_ALC_Threshold_L,
                           bluecell_Currdatastatus.bluecell_User_UL1_H,
                           bluecell_Currdatastatus.bluecell_User_UL1_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ATT_ALC2_MIN_H != Prev.ATT_ALC2_MIN_H
     ||Curr.ATT_ALC2_MIN_L != Prev.ATT_ALC2_MIN_L)
     ||(Curr.bluecell_User_UL2_H != Prev.bluecell_User_UL2_H
     ||Curr.bluecell_User_UL2_L != Prev.bluecell_User_UL2_L)){
        bluecell_Prevdatastatus.ATT_ALC2_MIN_H = bluecell_Currdatastatus.ATT_ALC2_MIN_H;
        bluecell_Prevdatastatus.ATT_ALC2_MIN_L = bluecell_Currdatastatus.ATT_ALC2_MIN_L;
        bluecell_Prevdatastatus.bluecell_User_UL2_H = bluecell_Currdatastatus.bluecell_User_UL2_H;
        bluecell_Prevdatastatus.bluecell_User_UL2_L = bluecell_Currdatastatus.bluecell_User_UL2_L;
        val = PE43711_Calc(&Att_DL2.Table_0_0_dBm,
            Curr.ATT_ALC2_MIN_H,
            Curr.ATT_ALC2_MIN_L,
                           bluecell_Currdatastatus.bluecell_User_UL2_H,
                           bluecell_Currdatastatus.bluecell_User_UL2_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ATT_ALC3_MIN_H != Prev.ATT_ALC3_MIN_H
     ||Curr.ATT_ALC3_MIN_L != Prev.ATT_ALC3_MIN_L)
     ||(Curr.bluecell_User_UL3_H != Prev.bluecell_User_UL3_H
     ||Curr.bluecell_User_UL3_L != Prev.bluecell_User_UL3_L)){
        bluecell_Prevdatastatus.ATT_ALC3_MIN_H = bluecell_Currdatastatus.ATT_ALC3_MIN_H;
        bluecell_Prevdatastatus.ATT_ALC3_MIN_L = bluecell_Currdatastatus.ATT_ALC3_MIN_L;
        bluecell_Prevdatastatus.bluecell_User_UL3_H = bluecell_Currdatastatus.bluecell_User_UL3_H;
        bluecell_Prevdatastatus.bluecell_User_UL3_L = bluecell_Currdatastatus.bluecell_User_UL3_L;
        val = PE43711_Calc(&Att_DL3.Table_0_0_dBm,
            Curr.ATT_ALC3_MIN_H,
            Curr.ATT_ALC3_MIN_L,
                           bluecell_Currdatastatus.bluecell_User_UL3_H,
                           bluecell_Currdatastatus.bluecell_User_UL3_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
    if((Curr.ATT_ALC4_MIN_H != Prev.ATT_ALC4_MIN_H
     ||Curr.ATT_ALC4_MIN_L != Prev.ATT_ALC4_MIN_L)
     ||(Curr.bluecell_User_UL4_H != Prev.bluecell_User_UL4_H
     ||Curr.bluecell_User_UL4_L != Prev.bluecell_User_UL4_L)){
        bluecell_Prevdatastatus.ATT_ALC4_MIN_H = bluecell_Currdatastatus.ATT_ALC4_MIN_H;
        bluecell_Prevdatastatus.ATT_ALC4_MIN_L = bluecell_Currdatastatus.ATT_ALC4_MIN_L;
        bluecell_Prevdatastatus.bluecell_User_UL4_H = bluecell_Currdatastatus.bluecell_User_UL4_H;
        bluecell_Prevdatastatus.bluecell_User_UL4_L = bluecell_Currdatastatus.bluecell_User_UL4_L;
        val = PE43711_Calc(&Att_DL4.Table_0_0_dBm,
            Curr.ATT_ALC4_MIN_H,
            Curr.ATT_ALC4_MIN_L,
                           bluecell_Currdatastatus.bluecell_User_UL4_H,
                           bluecell_Currdatastatus.bluecell_User_UL4_L);

//        PE43711_atten_ctrl(ALL_ATT.ATT_DL1,val);

    }
#endif // PYJ.2020.04.21_END -- 
     
}

#endif // PYJ.2020.05.25_END -- 



void Bluecell_StructCpy(uint8_t* dst,uint8_t* src,uint16_t size){
  for(int i = 0; i < size; i++){
    dst[i]  = src[i];
  }
}


int16_t ConvertTo2byte(uint8_t highbit, uint8_t lowbit){
    int16_t ret = 0;
    ret += ((highbit << 8) & 0xFF00);
    ret += (lowbit & 0x00FF);

    return ret;
}

OneByteToTwoByte ConvertTo1Byte(uint16_t data){
    OneByteToTwoByte ret;
    
    ret.High_bit = ((data & 0xFF00) >> 8);
    ret.Low_bit = ((data & 0x00FF));
    
    return ret;
}

void Bluecell_Struct2byteCopyFunction(uint8_t* dst,uint16_t* src,uint8_t size){
  OneByteToTwoByte convert;
  for(int i = 0; i < (size / 2); i++){
    convert = ConvertTo1Byte(src[i]);
    dst[i * 2]      = convert.High_bit;
    dst[i * 2 + 1]  = convert.Low_bit;
  }
}
void Bluecell_structprintf(uint8_t* dst,uint8_t size){
  for(int i = 0; i < size; i++){
    printf("Index[%d]%x \r\n",i,dst[i]);
  }
}



void Bluecell_DataCopy(uint8_t* dst,uint8_t* src,uint16_t size){
//    printf("Setting Start Value\r\n");
       for(int i = 0; i < size; i++){
         dst[i] = src[i];
//         printf("%02x ",src[i]);
       }
//        printf("Setting Start Value END \r\n");

}
void MBIC_TableDataCopy(uint8_t* dst,uint8_t* src,uint16_t size){
    dst[MBIC_TableIndex_Unit_Type] = UNIT_TYPE_MBIC;
    dst[MBIC_TableIndex_Reseved] = 0;
    dst[MBIC_TableIndex_Number] = dst[MBIC_TableIndex_Number];
//    printf("\r\dst[MBIC_TableIndex_Number] : %d \r\n",dst[MBIC_TableIndex_Number]);   
    dst[MBIC_TableIndex_Ref] = UNIT_TYPE_MBIC;    // Check
    if(dst[MBIC_TableIndex_Number] >= DLI_P1_ATT_Accuracy_Table_Number
     &&dst[MBIC_TableIndex_Number] <= ULO_P4_ATT_Accuracy_Table_Number ){
        dst[MBIC_TableIndex_Length] = size;   //
     }
     else{
         dst[MBIC_TableIndex_Length] = size / 2;   //
     }


    for(int i = 0; i < size; i++){
      dst[i + MBIC_TableIndex_Max] = src[i];
//      printf("SRC : %x \r\n",src[i]);
    }
 

}
void MBIC_TableHeaderCopy(uint8_t* src,uint16_t size){
    src[MBIC_TableIndex_Unit_Type] = UNIT_TYPE_MBIC;
    src[MBIC_TableIndex_Reseved] = 0;
    src[MBIC_TableIndex_Number] = src[MBIC_PAYLOADSTART + 2];
    
    if(src[MBIC_TableIndex_Number] >= DLI_P1_ATT_Accuracy_Table_Number
     &&src[MBIC_TableIndex_Number] <= ULO_P4_ATT_Accuracy_Table_Number ){
        src[MBIC_TableIndex_Length] = size;   //
     }
     else{
         src[MBIC_TableIndex_Length] = size / 2;   //
     }
}


void Bluecell_TableLoad(uint8_t* data,uint8_t type){
    uint8_t tabletype = type;
//    OneByteToTwoByte data;
//    printf("%s : %x \r\n",__func__,tabletype);
//INDEX :5 COpy Start
    switch(tabletype){
        case DLI_P1_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,EEPROM_ATT_BASE              ,&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st) );
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
//            Bluecell_structprintf(&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
          break;
        case DLI_P2_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL1_TABLE_ADDRESDS),&Att_DL2.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_DL2.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
            break;
        case DLI_P3_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL2_TABLE_ADDRESDS),&Att_DL3.Table_0_0_dBm,sizeof(ATT_TABLE_st) );               
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_DL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
            break;
        case DLI_P4_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL3_TABLE_ADDRESDS),&Att_DL4.Table_0_0_dBm,sizeof(ATT_TABLE_st) );               
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_DL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
            break;
        case ULO_P1_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL4_TABLE_ADDRESDS),&Att_UL1.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_UL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
        break;
        case ULO_P2_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL1_TABLE_ADDRESDS),&Att_UL2.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_UL2.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
            break;
        case ULO_P3_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL2_TABLE_ADDRESDS),&Att_UL3.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_UL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
            break;
        case ULO_P4_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL3_TABLE_ADDRESDS),&Att_UL4.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Att_UL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(ATT_TABLE_st)) + 7 - 3;
            break;
        case DLI_P1_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL4_TABLE_ADDRESDS),&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEDL_st)) + 7 - 3;
            break;
        case DLI_P2_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL1_TABLE_ADDRESDS),&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEDL_st)) + 7 - 3;
            break;
        case DLI_P3_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL2_TABLE_ADDRESDS),&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEDL_st)) + 7 - 3;
            break;
        case DLI_P4_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL3_TABLE_ADDRESDS),&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEDL_st)) + 7 - 3;
            break;
        case ULO_P1_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL4_TABLE_ADDRESDS),&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEUL_st)) + 7 - 3;
            break;
        case ULO_P2_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL1_TABLE_ADDRESDS),&Det_UL2.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_UL2.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEUL_st)) + 7 - 3;
        
            break;
        case ULO_P3_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL2_TABLE_ADDRESDS),&Det_UL3.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_UL3.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEUL_st)) + 7 - 3;
            break;
        case ULO_P4_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL3_TABLE_ADDRESDS),&Det_UL4.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Det_UL4.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[BLUECELL_LENGTH] = (sizeof(DET_TABLEUL_st)) + 7 - 3;

            break;
        case DLI_P1_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL4_TABLE_ADDRESDS),&Temp_DL1.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_DL1.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;

            break;
        case DLI_P2_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL1_TABLE_ADDRESDS),&Temp_DL2.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_DL2.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;

            break;
        case DLI_P3_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL2_TABLE_ADDRESDS),&Temp_DL3.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_DL3.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;

            break;
        case DLI_P4_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL3_TABLE_ADDRESDS),&Temp_DL4.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_DL4.Table_1_Temp,sizeof(TEMP_TABLE_st));
        data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;

        break;
        case ULO_P1_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL4_TABLE_ADDRESDS),&Temp_UL1.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_UL1.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;
        
            break;
        case ULO_P2_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL1_TABLE_ADDRESDS),&Temp_UL2.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_UL2.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;
        
            break;
        case ULO_P3_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL2_TABLE_ADDRESDS),&Temp_UL3.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_UL3.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;
        
            break;
        case ULO_P4_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL3_TABLE_ADDRESDS),&Temp_UL4.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            Bluecell_DataCopy(&data[BLUECELL_DATA + 1],&Temp_UL4.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[BLUECELL_LENGTH] = (sizeof(TEMP_TABLE_st)) + 7 - 3;
            break;
    }
        
    
}


void Bluecell_TableSave(uint8_t* data,uint8_t type){
    uint8_t tabletype = type;
    
    //printf("%s : %x \r\n",__func__,tabletype);
    switch(tabletype){
        case DLI_P1_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL1.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_BASE)              ,&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st) );
            break;
        case DLI_P2_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL2.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL1_TABLE_ADDRESDS),&Att_DL2.Table_0_0_dBm,sizeof(ATT_TABLE_st) );
//            printf("ADDRESS : %d \r\n",EEPROM_ATT_DL1_TABLE_ADDRESDS );
            break;
        case DLI_P3_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL3.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL2_TABLE_ADDRESDS),&Att_DL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            break;
        case DLI_P4_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL4.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL3_TABLE_ADDRESDS),&Att_DL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            break;
        case ULO_P1_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL1.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL4_TABLE_ADDRESDS),&Att_UL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            break;
        case ULO_P2_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL2.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL1_TABLE_ADDRESDS),&Att_UL2.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            break;
        case ULO_P3_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL3.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL2_TABLE_ADDRESDS),&Att_UL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            break;
        case ULO_P4_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL4.Table_0_0_dBm,&data[BLUECELL_DATA + 1],sizeof(ATT_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL3_TABLE_ADDRESDS),&Att_UL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            break;
        case DLI_P1_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL1.Table_Det5_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEDL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL4_TABLE_ADDRESDS),&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            break;
        case DLI_P2_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL2.Table_Det5_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEDL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL1_TABLE_ADDRESDS),&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            break;
        case DLI_P3_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL3.Table_Det5_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEDL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL2_TABLE_ADDRESDS),&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            break;
        case DLI_P4_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL4.Table_Det5_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEDL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL3_TABLE_ADDRESDS),&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            break;
        case ULO_P1_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL1.Table_Det_15_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEUL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL4_TABLE_ADDRESDS),&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            break;
        case ULO_P2_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL2.Table_Det_15_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEUL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL1_TABLE_ADDRESDS),&Det_UL2.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
        break;
        case ULO_P3_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL3.Table_Det_15_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEUL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL2_TABLE_ADDRESDS),&Det_UL3.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            break;
        case ULO_P4_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL4.Table_Det_15_dBm_H,&data[BLUECELL_DATA + 1],sizeof(DET_TABLEUL_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL3_TABLE_ADDRESDS),&Det_UL4.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            break;
        case DLI_P1_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL1.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL4_TABLE_ADDRESDS),&Temp_DL1.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            break;
        case DLI_P2_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL2.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL1_TABLE_ADDRESDS),&Temp_DL2.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            break;
        case DLI_P3_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL3.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL2_TABLE_ADDRESDS),&Temp_DL3.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            
            break;
        case DLI_P4_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL4.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL3_TABLE_ADDRESDS),&Temp_DL4.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            break;
        case ULO_P1_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL1.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL4_TABLE_ADDRESDS),&Temp_UL1.Table_1_Temp,sizeof(TEMP_TABLE_st));                            
            break;
        case ULO_P2_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL2.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL1_TABLE_ADDRESDS),&Temp_UL2.Table_1_Temp,sizeof(TEMP_TABLE_st));                            
            break;
        case ULO_P3_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL3.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL2_TABLE_ADDRESDS),&Temp_UL3.Table_1_Temp,sizeof(TEMP_TABLE_st));                            
            break;
        case ULO_P4_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL4.Table_1_Temp,&data[BLUECELL_DATA + 1],sizeof(TEMP_TABLE_st));
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL3_TABLE_ADDRESDS),&Temp_UL4.Table_1_Temp,sizeof(TEMP_TABLE_st));                                
            break;
    }
    
}

//uint8_t tableTest[sizeof(DET_TABLEDL_st)] ={0,};
void MBIC_TableLoad(uint8_t* data,uint8_t type){
    uint8_t tabletype = type;
//    OneByteToTwoByte data;
//    printf("%s : %x \r\n",__func__,tabletype);
//INDEX :5 COpy Start
    data[MBIC_CMD_0] = 0xC0;
    if( data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] > 70)
         data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = 70;
    printf("\r\ntabletype : %d \r\n",tabletype);
    switch(tabletype){
        case DLI_P1_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,EEPROM_ATT_BASE              ,&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st) );
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_DL1.Table_Ref; 
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_DL1.Table_Length;
          break;
        case DLI_P2_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL1_TABLE_ADDRESDS),&Att_DL2.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_DL2.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_DL2.Table_Ref; 
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_DL2.Table_Length;
            break;
        case DLI_P3_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL2_TABLE_ADDRESDS),&Att_DL3.Table_0_0_dBm,sizeof(ATT_TABLE_st) );               
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_DL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_DL3.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_DL3.Table_Length;        
            break;
        case DLI_P4_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL3_TABLE_ADDRESDS),&Att_DL4.Table_0_0_dBm,sizeof(ATT_TABLE_st) );               
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_DL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_DL4.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_DL4.Table_Length;        
            break;
        case ULO_P1_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_DL4_TABLE_ADDRESDS),&Att_UL1.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_UL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_UL1.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_UL1.Table_Length;

        break;
        case ULO_P2_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL1_TABLE_ADDRESDS),&Att_UL2.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_UL2.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_UL2.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_UL2.Table_Length;

            break;
        case ULO_P3_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL2_TABLE_ADDRESDS),&Att_UL3.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_UL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_UL3.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_UL3.Table_Length;

            break;
        case ULO_P4_ATT_Accuracy_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL3_TABLE_ADDRESDS),&Att_UL4.Table_0_0_dBm,sizeof(ATT_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Att_UL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Att_UL4.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Att_UL4.Table_Length;

            break;
        case DLI_P1_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL4_TABLE_ADDRESDS),&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_DL1.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_DL1.Table_Length;
        break;
        case DLI_P2_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL1_TABLE_ADDRESDS),&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_DL2.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_DL2.Table_Length;        
            break;
        case DLI_P3_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL2_TABLE_ADDRESDS),&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_DL3.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_DL3.Table_Length;        

            break;
        case DLI_P4_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL3_TABLE_ADDRESDS),&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_DL4.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_DL4.Table_Length;        

            break;
        case ULO_P1_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_DL4_TABLE_ADDRESDS),&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_UL1.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_UL1.Table_Length;

            break;
        case ULO_P2_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL1_TABLE_ADDRESDS),&Det_UL2.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_UL2.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_UL2.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_UL2.Table_Length;
            printf("Det_UL2.Table_Length : %d \r\n",Det_UL2.Table_Length);

        
            break;
        case ULO_P3_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL2_TABLE_ADDRESDS),&Det_UL3.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_UL3.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_UL3.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_UL3.Table_Length;

            break;
        case ULO_P4_Level_Table_Number: 
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL3_TABLE_ADDRESDS),&Det_UL4.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Det_UL4.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Det_UL4.Table_Ref;
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Det_UL4.Table_Length;


            break;
        case DLI_P1_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_DET_UL4_TABLE_ADDRESDS),&Temp_DL1.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_DL1.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_DL1.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_DL1.Table_Length;
            break;
        case DLI_P2_ATT_Temp_guarantee_Table_Number:
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_DL2.Table_1_Temp,sizeof(TEMP_TABLE_st));
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_DL2.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_DL2.Table_Length;        
            break;
        case DLI_P3_ATT_Temp_guarantee_Table_Number:
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_DL3.Table_1_Temp,sizeof(TEMP_TABLE_st));
//            MBIC_Table_Load_TempValueConvert(&data[MBIC_PAYLOADSTART]);
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_DL3.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_DL3.Table_Length;        
            break;
        case DLI_P4_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL3_TABLE_ADDRESDS),&Temp_DL4.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_DL4.Table_1_Temp,sizeof(TEMP_TABLE_st));
//            MBIC_Table_Load_TempValueConvert(&data[MBIC_PAYLOADSTART]);
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_DL4.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_DL4.Table_Length;        
            break;
        case ULO_P1_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL4_TABLE_ADDRESDS),&Temp_UL1.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_UL1.Table_1_Temp,sizeof(TEMP_TABLE_st));
//            MBIC_Table_Load_TempValueConvert(&data[MBIC_PAYLOADSTART]);
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_UL1.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_UL1.Table_Length;        
            break;
        case ULO_P2_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL1_TABLE_ADDRESDS),&Temp_UL2.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_UL2.Table_1_Temp,sizeof(TEMP_TABLE_st));
//            MBIC_Table_Load_TempValueConvert(&data[MBIC_PAYLOADSTART]);
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_UL2.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_UL2.Table_Length;                
            break;
        case ULO_P3_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL2_TABLE_ADDRESDS),&Temp_UL3.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_UL3.Table_1_Temp,sizeof(TEMP_TABLE_st));
//            MBIC_Table_Load_TempValueConvert(&data[MBIC_PAYLOADSTART]);
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_UL3.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_UL3.Table_Length;                
            break;
        case ULO_P4_ATT_Temp_guarantee_Table_Number:
            EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL3_TABLE_ADDRESDS),&Temp_UL4.Table_1_Temp,sizeof(TEMP_TABLE_st) );   
            MBIC_TableDataCopy(&data[MBIC_PAYLOADSTART],&Temp_UL4.Table_1_Temp,sizeof(TEMP_TABLE_st));
//            MBIC_Table_Load_TempValueConvert(&data[MBIC_PAYLOADSTART]);
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref] = Temp_UL4.Table_Ref;        
            data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = Temp_UL4.Table_Length;                
            break;
    }
     printf("\rLoad MBIC_TableIndex_Number : %x \r\n",data[MBIC_PAYLOADSTART + MBIC_TableIndex_Number]);
    
}


void MBIC_TableSave(uint8_t* data,uint8_t type){
    uint8_t tabletype = type;
    
    //printf("%s : %x \r\n",__func__,tabletype);
    if( data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] > 70)
         data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] = 70;
//    printf("data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length]  : %d \r\n",sizeof(ATT_TABLE_st)  );
    data[MBIC_CMD_0] = 0xC1;
    switch(tabletype){
        case DLI_P1_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL1.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_DL1.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
            Att_DL1.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];            
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_BASE)              ,&Att_DL1.Table_0_0_dBm,sizeof(ATT_TABLE_st) );
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   
            break;
        case DLI_P2_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL2.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_DL2.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];   // Check
            Att_DL2.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];            
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL1_TABLE_ADDRESDS),&Att_DL2.Table_0_0_dBm,sizeof(ATT_TABLE_st) );
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   
//            printf("ADDRESS : %d \r\n",EEPROM_ATT_DL1_TABLE_ADDRESDS );
            break;
        case DLI_P3_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL3.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_DL3.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
            Att_DL3.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];            
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL2_TABLE_ADDRESDS),&Att_DL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   
            break;
        case DLI_P4_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_DL4.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_DL4.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
            Att_DL4.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL3_TABLE_ADDRESDS),&Att_DL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P1_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL1.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));
            Att_UL1.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
            Att_UL1.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_DL4_TABLE_ADDRESDS),&Att_UL1.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   
            break;
        case ULO_P2_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL2.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_UL2.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Att_UL2.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL1_TABLE_ADDRESDS),&Att_UL2.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   
            break;
        case ULO_P3_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL3.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_UL3.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
             Att_UL3.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL2_TABLE_ADDRESDS),&Att_UL3.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P4_ATT_Accuracy_Table_Number: 
            Bluecell_DataCopy(&Att_UL4.Table_0_0_dBm,&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st) );
            Att_UL4.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
            Att_UL4.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL3_TABLE_ADDRESDS),&Att_UL4.Table_0_0_dBm,sizeof(ATT_TABLE_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P1_Level_Table_Number: 

            Bluecell_DataCopy(&Det_DL1.Table_Det5_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEDL_st));
            Det_DL1.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];    // Check
            Det_DL1.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_UL4_TABLE_ADDRESDS),&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
  //          EEPROM_M24C08_Read(EEPROM_M24C08_ID ,(EEPROM_ATT_UL4_TABLE_ADDRESDS),tableTest,sizeof(DET_TABLEDL_st));
//            for(int i = 0 ; i < sizeof(DET_TABLEDL_st); i++)
//                printf("tableTest : %x\r\n",tableTest[i]);
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P2_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL2.Table_Det5_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEDL_st));
            Det_DL2.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];   // Check
            Det_DL2.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL1_TABLE_ADDRESDS),&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P3_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL3.Table_Det5_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEDL_st));
            Det_DL3.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Det_DL3.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL2_TABLE_ADDRESDS),&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P4_Level_Table_Number: 
            Bluecell_DataCopy(&Det_DL4.Table_Det5_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEDL_st));
            Det_DL4.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Det_DL4.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL3_TABLE_ADDRESDS),&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st));
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P1_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL1.Table_Det_15_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEUL_st));
            Det_UL1.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Det_UL1.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_DL4_TABLE_ADDRESDS),&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P2_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL2.Table_Det_15_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEUL_st));
            Det_UL2.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Det_UL2.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL1_TABLE_ADDRESDS),&Det_UL2.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

        break;
        case ULO_P3_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL3.Table_Det_15_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEUL_st));
            Det_UL3.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Det_UL3.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL2_TABLE_ADDRESDS),&Det_UL3.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P4_Level_Table_Number: 
            Bluecell_DataCopy(&Det_UL4.Table_Det_15_dBm_H,&data[MBIC_PAYLOADSTART + 5],sizeof(DET_TABLEUL_st));
            Det_UL4.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Det_UL4.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL3_TABLE_ADDRESDS),&Det_UL4.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P1_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL1.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_DL1.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_DL1.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_DET_UL4_TABLE_ADDRESDS),&Temp_DL1.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   
            printf("Temp_DL1 Length : %d \r\n",data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length]);

            break;
        case DLI_P2_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL2.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_DL2.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_DL2.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL1_TABLE_ADDRESDS),&Temp_DL2.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P3_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL3.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_DL3.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_DL3.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL2_TABLE_ADDRESDS),&Temp_DL3.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case DLI_P4_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_DL4.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_DL4.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_DL4.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL3_TABLE_ADDRESDS),&Temp_DL4.Table_1_Temp,sizeof(TEMP_TABLE_st));                    
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P1_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL1.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_UL1.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_UL1.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_DL4_TABLE_ADDRESDS),&Temp_UL1.Table_1_Temp,sizeof(TEMP_TABLE_st));                            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P2_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL2.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_UL2.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_UL2.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL1_TABLE_ADDRESDS),&Temp_UL2.Table_1_Temp,sizeof(TEMP_TABLE_st));                            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P3_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL3.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st) );
            Temp_UL3.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_UL3.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            printf("Temp_UL3.Table_Length : %d \r\n",Temp_UL3.Table_Length);
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL2_TABLE_ADDRESDS),&Temp_UL3.Table_1_Temp,sizeof(TEMP_TABLE_st));                            
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
        case ULO_P4_ATT_Temp_guarantee_Table_Number:
            Bluecell_DataCopy(&Temp_UL4.Table_1_Temp,&data[MBIC_PAYLOADSTART + 5],sizeof(TEMP_TABLE_st));
            Temp_UL4.Table_Ref = data[MBIC_PAYLOADSTART + MBIC_TableIndex_Ref];
            Temp_UL4.Table_Length= data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length];
            EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_TEMP_UL3_TABLE_ADDRESDS),&Temp_UL4.Table_1_Temp,sizeof(TEMP_TABLE_st));                                
            MBIC_TableHeaderCopy(&data[MBIC_PAYLOADSTART + 5],sizeof(ATT_TABLE_st));   

            break;
    }
    printf("\rSetting MBIC_TableIndex_Number : %x \r\n",data[MBIC_PAYLOADSTART + MBIC_TableIndex_Number]);
}




bool Bluecell_Operate(uint8_t* data){
  uint8_t datatype = data[BLUECELL_TYPE];


  //double ret = 0 ,tmp = 0.1;
  int16_t tempdata = 0;
  
  uint8_t i = 0;
  switch(datatype){
    case BLUECELL_SOFTWARERESET:
        NVIC_SystemReset();
        break;
	case Bluecell_ATT_DL1 				 :
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
       

        bluecell_Currdatastatus.ATT_DL1_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_DL1_L = data[BLUECELL_DATA + i++];
//        printf("bluecell_Currdatastatus.ATT_DL1_H : %x\r\n",bluecell_Currdatastatus.ATT_DL1_H);
//        printf("bluecell_Currdatastatus.ATT_DL1_L : %x\r\n",bluecell_Currdatastatus.ATT_DL1_L);        
        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
    case Bluecell_ATT_DL2                :
        bluecell_Currdatastatus.ATT_DL2_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_DL2_L = data[BLUECELL_DATA + i++];

        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
	case Bluecell_ATT_DL3 				 :        
        bluecell_Currdatastatus.ATT_DL3_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_DL3_L = data[BLUECELL_DATA + i++];

        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
	case Bluecell_ATT_DL4 				 :    
        bluecell_Currdatastatus.ATT_DL4_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_DL4_L = data[BLUECELL_DATA + i++];

        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
    case Bluecell_ATT_UL1                :

        bluecell_Currdatastatus.ATT_UL1_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_UL1_L = data[BLUECELL_DATA + i++];
        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        //        printf("bluecell_Currdatastatus.ATT_UL1_H : %x\r\n",bluecell_Currdatastatus.ATT_UL1_H);
        //        printf("bluecell_Currdatastatus.ATT_UL1_L : %x\r\n",bluecell_Currdatastatus.ATT_UL1_L);        
        break;
    case Bluecell_ATT_UL2                :        
        bluecell_Currdatastatus.ATT_UL2_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_UL2_L = data[BLUECELL_DATA + i++];

        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;    
    case Bluecell_ATT_UL3                :    
        bluecell_Currdatastatus.ATT_UL3_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_UL3_L = data[BLUECELL_DATA + i++];

        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;    
    case Bluecell_ATT_UL4                :        
        bluecell_Currdatastatus.ATT_UL4_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ATT_UL4_L = data[BLUECELL_DATA + i++];
        printf("bluecell_Currdatastatus.ATT_UL4_H : %0x \r\n",bluecell_Currdatastatus.ATT_UL4_H);
        printf("bluecell_Currdatastatus.ATT_UL4_L : %0x \r\n",bluecell_Currdatastatus.ATT_UL4_L);        


        /*
            Atten Ctrl Function
        */
		CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
//		printf("ret : %f ,tmp %f \r\n",ret,tmp );
		break;
	case ATT_DL1_PATH 		 :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_RESET);//CLOCK        
            printf("ATT_DL1_PATH OFF\r\n");
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 = 0;
        }
        else{
            
            HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_DL1_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL1_L =0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
            printf("ATT_DL1_PATH ON\r\n");
        }
        bluecell_Currdatastatus.ATT_DL1_PATH = data[BLUECELL_DATA];
	

		break;
	case ATT_UL1_PATH        :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 = 0;
        }
        else{
            HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_UL1_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL1_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

        }
        bluecell_Currdatastatus.ATT_UL1_PATH = data[BLUECELL_DATA];
//        printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_SelfTest1		 :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(_PATH_SW1_GPIO_Port,_PATH_SW1_Pin,GPIO_PIN_RESET);//CLOCK
            HAL_GPIO_WritePin(PATH_SW1_GPIO_Port,PATH_SW1_Pin,GPIO_PIN_SET);//CLOCK
        }
        else{
            HAL_GPIO_WritePin(_PATH_SW1_GPIO_Port,_PATH_SW1_Pin,GPIO_PIN_SET);//CLOCK
            HAL_GPIO_WritePin(PATH_SW1_GPIO_Port,PATH_SW1_Pin,GPIO_PIN_RESET);//CLOCK
        }
        bluecell_Currdatastatus.Selftest1 = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_DL2_PATH		 :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 = 0;            
        }
        else{
            HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_DL2_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL2_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
        bluecell_Currdatastatus.ATT_DL2_PATH = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;

		break;
	case ATT_UL2_PATH 		 :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2 = 0;
        }
        else{
            HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_UL2_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL2_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
        bluecell_Currdatastatus.ATT_UL2_PATH = data[BLUECELL_DATA];
         printf("UL SHUTDOWN RECOVERY OPERATE PATH : %d \r\n",bluecell_Currdatastatus.ATT_UL2_PATH);
		break;
	case ATT_SelfTest2		 :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(_PATH_SW2_GPIO_Port,_PATH_SW2_Pin,GPIO_PIN_RESET);//CLOCK
            HAL_GPIO_WritePin(PATH_SW2_GPIO_Port,PATH_SW2_Pin,GPIO_PIN_SET);//CLOCK
        }else{
            HAL_GPIO_WritePin(PATH_SW2_GPIO_Port,PATH_SW2_Pin,GPIO_PIN_RESET);//CLOCK
            HAL_GPIO_WritePin(_PATH_SW2_GPIO_Port,_PATH_SW2_Pin,GPIO_PIN_SET);//CLOCK
        }
        bluecell_Currdatastatus.Selftest2 = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_DL3_PATH :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 = 0;
        }
        else{
            HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_DL3_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL3_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
        bluecell_Currdatastatus.ATT_DL3_PATH = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_UL3_PATH :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3 = 0;
        }
        else{
            HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_UL3_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL3_L =0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
        bluecell_Currdatastatus.ATT_UL3_PATH = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_SelfTest3 :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(_PATH_SW3_GPIO_Port,_PATH_SW3_Pin,GPIO_PIN_RESET);//CLOCK
            HAL_GPIO_WritePin(PATH_SW3_GPIO_Port,PATH_SW3_Pin,GPIO_PIN_SET);//CLOCK
        }
        else{
            HAL_GPIO_WritePin(PATH_SW3_GPIO_Port,PATH_SW3_Pin,GPIO_PIN_RESET);//CLOCK
            HAL_GPIO_WritePin(_PATH_SW3_GPIO_Port,_PATH_SW3_Pin,GPIO_PIN_SET);//CLOCK
        }
        bluecell_Currdatastatus.Selftest3 = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_DL4_PATH :
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 = 0;
        }else{
            HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_SET);//CLOCK    
            bluecell_Prevdatastatus.ATT_DL4_H =0xFF;
            bluecell_Prevdatastatus.ATT_DL4_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

        }
        bluecell_Currdatastatus.ATT_DL4_PATH = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_UL4_PATH:
        if(data[BLUECELL_DATA]==0){
            HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_RESET);//CLOCK
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4 = 0;
        }
        else{
            HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Prevdatastatus.ATT_UL4_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL4_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
        bluecell_Currdatastatus.ATT_UL4_PATH = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
	case ATT_SelfTest4 :
	  if(data[BLUECELL_DATA]==0){
	    HAL_GPIO_WritePin(_PATH_SW4_GPIO_Port,_PATH_SW4_Pin,GPIO_PIN_RESET);//CLOCK
	    HAL_GPIO_WritePin(PATH_SW4_GPIO_Port,PATH_SW4_Pin,GPIO_PIN_SET);//CLOCK
	  }else{
	    HAL_GPIO_WritePin(_PATH_SW4_GPIO_Port,_PATH_SW4_Pin,GPIO_PIN_SET);//CLOCK
	    HAL_GPIO_WritePin(PATH_SW4_GPIO_Port,PATH_SW4_Pin,GPIO_PIN_RESET);//CLOCK
	  }
      bluecell_Currdatastatus.Selftest4 = data[BLUECELL_DATA];
//		printf("Function : %s Line %d \r\n",__func__,__LINE__);
		break;
    case ATT_TableSet:
        Bluecell_TableSave(data, data[BLUECELL_DATA]);
    case ATT_TableGet:
        Bluecell_TableLoad(data, data[BLUECELL_DATA]);
        data[BLUECELL_TYPE] = ATT_TableGet;

        data[data[BLUECELL_LENGTH] + 1] = ((CRC16_Generate(&data[BLUECELL_TYPE], data[BLUECELL_LENGTH]) & 0xFF00) >> 8);
        data[data[BLUECELL_LENGTH] + 2] = CRC16_Generate(&data[BLUECELL_TYPE], data[BLUECELL_LENGTH]) & 0x00FF;        
        data[data[BLUECELL_LENGTH] + 3] = 0xEB;
        
        Bluecell_StructCpy(&Txdata[0],&data[0],data[BLUECELL_LENGTH] + 4);
        Uart1_Data_Send(&Txdata[0], Txdata[BLUECELL_LENGTH] + 4);
#if 0 // PYJ.2020.04.22_BEGIN -- 
        for(int i = 0 ; i < data[BLUECELL_LENGTH] + 3; i++ ){
//            printf("%x ",data[i]);
        Txdata[0] = data[0];        }
        printf("\r\n");
#endif // PYJ.2020.04.22_END -- 
//        printf("\r\nuint8_t data : %x  data[BLUECELL_LENGTH] + 6 : %d\r\n",data[0],data[BLUECELL_LENGTH] + 6);
        break;
    case Bluecell_StatusReq:
        if(Initialize == false){
            printf("Atten Init \r\n");
            Bluecell_DataInit();
            HAL_Delay(10);
            Bluecell_AttenInitialize();
            Initialize = true;
        }
        DataStatusSet();
#if 0 // PYJ.2020.05.28_BEGIN -- 
        Bluecell_StructCpy(&Txdata[0],&bluecell_Currdatastatus.bluecell_header,94);
        Txdata[BLUECELL_LENGTH] = 94 - 3;//sizeof(BLUESTATUS_st) - 3;
        Txdata[94 - 2] = STH30_CreateCrc(&Txdata[BLUECELL_TYPE], Txdata[BLUECELL_LENGTH]);
#else        
//        printf("bluecell_Currdatastatus.ATT_UL4_H : %0x \r\n",bluecell_Currdatastatus.ATT_UL4_H);
//        printf("bluecell_Currdatastatus.ATT_UL4_L : %0x \r\n",bluecell_Currdatastatus.ATT_UL4_L);

        Bluecell_StructCpy(&Txdata[0],&bluecell_Currdatastatus.bluecell_header,sizeof(BLUESTATUS_st));
        Txdata[BLUECELL_LENGTH] = sizeof(BLUESTATUS_st) - 3;//sizeof(BLUESTATUS_st) - 3;
//        Txdata[sizeof(BLUESTATUS_st) - 2] = STH30_CreateCrc(&Txdata[BLUECELL_TYPE], Txdata[BLUECELL_LENGTH]);
        Txdata[sizeof(BLUESTATUS_st) - 3] = ((CRC16_Generate(&data[BLUECELL_TYPE], data[BLUECELL_LENGTH]) & 0xFF00) >> 8);
        Txdata[sizeof(BLUESTATUS_st) - 2] = CRC16_Generate(&data[BLUECELL_TYPE], data[BLUECELL_LENGTH]) & 0x00FF;        
        
#endif // PYJ.2020.05.28_END -- 
#if 0 // PYJ.2020.04.22_BEGIN -- 
        for(int i = 0 ; i < sizeof(BLUESTATUS_st); i++ ){
            printf("%x ",Txdata[i]);
        }
        printf("\r\n");
#endif // PYJ.2020.04.22_END --    
        Uart1_Data_Send(&Txdata[0], sizeof(BLUESTATUS_st));
        break;
    case Bluecell_StatusSave:
//        printf("Copy Complete");
//        Bluecell_StructCpy(&DataWrite[0],&bluecell_Currdatastatus.bluecell_header,sizeof(BLUESTATUS_st));
//        EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_ATT_BASE),&DataWrite[0],sizeof(BLUESTATUS_st));
//        Bluecell_StructCpy(&Txdata[0],&DataWrite[0],sizeof(BLUESTATUS_st));
//        Txdata[sizeof(BLUESTATUS_st) - 2] = STH30_CreateCrc(&Txdata[BLUECELL_TYPE], sizeof(BLUESTATUS_st) - 3);
//        Uart1_Data_Send(&Txdata[0], sizeof(BLUESTATUS_st));        
        EEPROM_M24C08_Zerowrite(EEPROM_M24C08_ID ,(EEPROM_ATT_BASE));
//        printf("ZERO WRITE COMPLETE");
         NVIC_SystemReset();

        break;

    case Bluecell_DL1_USER: 
        bluecell_Currdatastatus.bluecell_User_DL1_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_DL1_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
    case Bluecell_DL2_USER:
        bluecell_Currdatastatus.bluecell_User_DL2_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_DL2_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
    case Bluecell_DL3_USER:
        bluecell_Currdatastatus.bluecell_User_DL3_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_DL3_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
    case Bluecell_DL4_USER:     
        bluecell_Currdatastatus.bluecell_User_DL4_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_DL4_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;

    case Bluecell_UL1_USER: 
        bluecell_Currdatastatus.bluecell_User_UL1_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_UL1_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;
    case Bluecell_UL2_USER:
        bluecell_Currdatastatus.bluecell_User_UL2_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_UL2_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

        break;
    case Bluecell_UL3_USER:
        bluecell_Currdatastatus.bluecell_User_UL3_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_UL3_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

        break;
    case Bluecell_UL4_USER:
        bluecell_Currdatastatus.bluecell_User_UL4_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.bluecell_User_UL4_L = data[BLUECELL_DATA + i++];
        CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        break;

        /*******/
#if 0 // PYJ.2020.05.13_BEGIN -- 
    수정 및 추가
#endif // PYJ.2020.05.13_END -- 
        /*******/
    case Bluecell_TEMP_USER                               :
        bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET = data[BLUECELL_DATA + i++];
//        bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET_L = data[BLUECELL_DATA + i++];
        break;
    case Bluecell_DLI_AGC_ON_OFF:                             
        bluecell_Currdatastatus.DLI_AGC_ON_OFF = data[BLUECELL_DATA + i];
        printf("AGC ON OFF SET : %d \r\n",bluecell_Currdatastatus.DLI_AGC_ON_OFF);
        break;
    case Bluecell_ULO_ALC_ON_OFF:                             
        bluecell_Currdatastatus.ULO_ALC_ON_OFF = data[BLUECELL_DATA + i];
        printf("ALC ON OFF SET : %d \r\n",bluecell_Currdatastatus.DLI_AGC_ON_OFF);
        break;
    case Bluecell_DLI_AGC_Threshold:                          
        bluecell_Currdatastatus.DLI_AGC_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.DLI_AGC_Threshold_L = data[BLUECELL_DATA + i++];
//        tempdata = (( bluecell_Currdatastatus.DLI_AGC_Threshold_H << 8) & 0xFF00) ;
//        tempdata +=  bluecell_Currdatastatus.DLI_AGC_Threshold_L ;
//        tempdata /= 10;
//        printf("tempdata : %d\r\n",tempdata);
//        bluecell_Currdatastatus.DLI_AGC_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.DLI_AGC_Threshold_L = (tempdata & 0x00FF); 

        break;
    case Bluecell_DLI_AGC_Threshold_Default:                  
        bluecell_Currdatastatus.DLI_AGC_Threshold_default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.DLI_AGC_Threshold_default == true){
            bluecell_Currdatastatus.DLI_AGC_Threshold_H = MBIC_DLI_AGC_Threshold_Default_H;
            bluecell_Currdatastatus.DLI_AGC_Threshold_L = MBIC_DLI_AGC_Threshold_Default_L;
            bluecell_Currdatastatus.DLI_AGC_Threshold_default = false;
            printf("%s : %d \r\n",__func__,__LINE__);
        }
        break;
    case Bluecell_DLI_Shutdown_ON_OFF:                        
        bluecell_Currdatastatus.DLI_Shutdown_ON_OFF = data[BLUECELL_DATA + i++];
        printf("%s : %d  DLI_Shutdown_ON_OFF : %d \r\n",__func__,__LINE__,bluecell_Currdatastatus.DLI_Shutdown_ON_OFF);
        
        break;
    case Bluecell_DLI_Shutdown_Threshold:                     
        bluecell_Currdatastatus.DLI_Shutdown_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.DLI_Shutdown_Threshold_L = data[BLUECELL_DATA + i++];
        tempdata = (( bluecell_Currdatastatus.DLI_Shutdown_Threshold_H << 8) & 0xFF00) ;
        tempdata +=  bluecell_Currdatastatus.DLI_Shutdown_Threshold_L ;
        tempdata /= 10;
        printf("tempdata : %d\r\n",tempdata);
//        bluecell_Currdatastatus.DLI_Shutdown_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.DLI_Shutdown_Threshold_L = (tempdata & 0x00FF);
//        printf("tempdata %d \r\n",tempdata);

        break;
    case Bluecell_DLI_Shutdown_Threshold_Default:             
        bluecell_Currdatastatus.DLI_Shutdown_Threshold_Default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.DLI_Shutdown_Threshold_Default == true){
            bluecell_Currdatastatus.DLI_Shutdown_Threshold_H = MBIC_DLI_Shutdown_Threshold_Default_H;
            bluecell_Currdatastatus.DLI_Shutdown_Threshold_L = MBIC_DLI_Shutdown_Threshold_Default_L;
        }
        printf("%s : %d \r\n",__func__,__LINE__);
//      printf("bluecell_Currdatastatus.DLI_Shutdown_Threshold_Default : %d \r\n",bluecell_Currdatastatus.DLI_Shutdown_Threshold_Default);
        break;
    case Bluecell_DLI_Shutdown_Count:                         
        /*NOP*/
        break;
    case Bluecell_DLI_Level_High_Threshold                :   
        bluecell_Currdatastatus.DLI_Level_High_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.DLI_Level_High_Threshold_L = data[BLUECELL_DATA + i++];
        tempdata = (( bluecell_Currdatastatus.DLI_Level_High_Threshold_H << 8) & 0xFF00) ;
        tempdata +=  bluecell_Currdatastatus.DLI_Level_High_Threshold_L ;
        tempdata /= 10;
        printf("DLI_Level_High_Threshold : %d\r\n",tempdata);
//        bluecell_Currdatastatus.DLI_Level_High_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.DLI_Level_High_Threshold_L = (tempdata & 0x00FF);
        break;
    case Bluecell_DLI_Level_Low_Threshold                 :   
        bluecell_Currdatastatus.DLI_Level_Low_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.DLI_Level_Low_Threshold_L = data[BLUECELL_DATA + i++];
        tempdata = (( bluecell_Currdatastatus.DLI_Level_Low_Threshold_H << 8) & 0xFF00) ;
        tempdata +=  bluecell_Currdatastatus.DLI_Level_Low_Threshold_L ;
        tempdata /= 10;
         printf("DLI_Level_Low_Threshold : %d\r\n",tempdata);
//        bluecell_Currdatastatus.DLI_Level_Low_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.DLI_Level_Low_Threshold_L = (tempdata & 0x00FF);    

        break;
    case Bluecell_DLI_Level_High_Low_Threshold_default    :   
        bluecell_Currdatastatus.DLI_Level_High_Low_Threshold_default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.DLI_Level_High_Low_Threshold_default == true){
            bluecell_Currdatastatus.DLI_Level_High_Threshold_H = MBIC_DLI_Level_High_Threshold_default_H;
            bluecell_Currdatastatus.DLI_Level_High_Threshold_L = MBIC_DLI_Level_High_Threshold_default_L;
            bluecell_Currdatastatus.DLI_Level_Low_Threshold_H = MBIC_DLI_Level_Low_Threshold_default_H;
            bluecell_Currdatastatus.DLI_Level_Low_Threshold_L = MBIC_DLI_Level_Low_Threshold_default_L;
        }
        break;
    case Bluecell_LED_TEST  :                                 
        bluecell_Currdatastatus.LED_TEST = data[BLUECELL_DATA + i++];
        printf("%s : %d Value : %d \r\n",__func__,__LINE__,bluecell_Currdatastatus.LED_TEST);
        break;
    case Bluecell_Temperature_Offset                      :   
        bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET = data[BLUECELL_DATA + i++];
        printf("%s : %d \r\n",__func__,__LINE__);

        break;
    case Bluecell_Temp_High_Threshold                     :   
        bluecell_Currdatastatus.Temp_High_Threshold = data[BLUECELL_DATA + i++];
        printf("%s : %d Temps_High_Threshold :  %d\r\n",__func__,__LINE__,bluecell_Currdatastatus.Temp_High_Threshold);
        printf("Temp_High_Threshold : %d :   Det %d \r\n",bluecell_Currdatastatus.Temp_High_Threshold,bluecell_Currdatastatus.DET_TEMP);
        printf("Temp_High_Alarm : %d \r\n",bluecell_Currdatastatus.Temp_High_Alarm);
        printf("Txdata : %d \r\n",Txdata[201]);
    
        break;
    case Bluecell_Temp_High_Threshold_Default             :   
        bluecell_Currdatastatus.Temp_High_Threshold_Default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.Temp_High_Threshold_Default == true){
            bluecell_Currdatastatus.Temp_High_Threshold_Default = MBIC_Temp_High_Threshold_Default;
        }
        printf("%s : %d \r\n",__func__,__LINE__);
        break;
    case Bluecell_ULO_Level_High_Threshold                :   
        bluecell_Currdatastatus.ULO_Level_High_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ULO_Level_High_Threshold_L = data[BLUECELL_DATA + i++];
        tempdata = (( bluecell_Currdatastatus.ULO_Level_High_Threshold_H << 8) & 0xFF00) ;
        tempdata +=  bluecell_Currdatastatus.ULO_Level_High_Threshold_L ;
        tempdata /= 10;
         printf("ULO_Level_High_Threshold : %d\r\n",tempdata);
//        bluecell_Currdatastatus.ULO_Level_High_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.ULO_Level_High_Threshold_L = (tempdata & 0x00FF);   

       

        break;
    case Bluecell_ULO_Level_High_Threshold_default        :   
        bluecell_Currdatastatus.ULO_Level_High_Threshold_default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.ULO_Level_High_Threshold_default == true){
            bluecell_Currdatastatus.ULO_Level_High_Threshold_H = MBIC_ULO_Level_High_Threshold_Default_H;
            bluecell_Currdatastatus.ULO_Level_High_Threshold_L = MBIC_ULO_Level_High_Threshold_Default_L;
        }
        break;
    case Bluecell_ULO_ALC_Threshold                       :   
        bluecell_Currdatastatus.ULO_ALC_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ULO_ALC_Threshold_L = data[BLUECELL_DATA + i++];
        tempdata = (( bluecell_Currdatastatus.ULO_ALC_Threshold_H << 8) & 0xFF00) ;
        tempdata +=  bluecell_Currdatastatus.ULO_ALC_Threshold_L ;
        tempdata /= 10;
        printf("ULO_ALC_Threshold : %d\r\n",tempdata);
//        bluecell_Currdatastatus.ULO_ALC_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.ULO_ALC_Threshold_L = (tempdata & 0x00FF);

        break;
    case Bluecell_ULO_ALC_Threshold_Default               :   
        bluecell_Currdatastatus.ULO_ALC_Threshold_Default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.ULO_Level_High_Threshold_default == true){
            bluecell_Currdatastatus.ULO_ALC_Threshold_H = MBIC_ULO_ALC_Threshold_Default_H;
            bluecell_Currdatastatus.ULO_ALC_Threshold_L = MBIC_ULO_ALC_Threshold_Default_L;
        }
        printf("%s : %d \r\n",__func__,__LINE__);

        break;
    case Bluecell_ULO_Shutdown_ON_OFF                     :   
        bluecell_Currdatastatus.ULO_Shutdown_ON_OFF = data[BLUECELL_DATA + i++];
         printf("%s Bluecell_ULO_Shutdown_ON_OFF : %d \r\n",__func__,bluecell_Currdatastatus.ULO_Shutdown_ON_OFF );

        break;
    case Bluecell_ULO_Shutdown_Threshold                  :   
        bluecell_Currdatastatus.ULO_Shutdown_Threshold_H = data[BLUECELL_DATA + i++];
        bluecell_Currdatastatus.ULO_Shutdown_Threshold_L = data[BLUECELL_DATA + i++];
        tempdata = (( bluecell_Currdatastatus.ULO_Shutdown_Threshold_H << 8) & 0xFF00) ;
        tempdata +=  bluecell_Currdatastatus.ULO_Shutdown_Threshold_L ;
        tempdata /= 10;
        printf("tempdata : %d\r\n",tempdata);
//        bluecell_Currdatastatus.ULO_Shutdown_Threshold_H = ((tempdata & 0xFF00) >> 8);
//        bluecell_Currdatastatus.ULO_Shutdown_Threshold_L = (tempdata & 0x00FF);     

        break;
    case Bluecell_ULO_Shutdown_Threshold_Default          :   
        bluecell_Currdatastatus.ULO_Shutdown_Threshold_Default = data[BLUECELL_DATA + i++];
        if(bluecell_Currdatastatus.ULO_Level_High_Threshold_default == true){
            bluecell_Currdatastatus.ULO_Shutdown_Threshold_H = MBIC_ULO_Shutdown_Threshold_Default_H;
            bluecell_Currdatastatus.ULO_Shutdown_Threshold_L = MBIC_ULO_Shutdown_Threshold_Default_L;
        }
         printf("%s : %d \r\n",__func__,__LINE__);
        break;
    case Bluecell_ULO_Shutdown_Retry_Count                :   
        break;
    case Bluecell_Alarm_Mask:
        bluecell_Currdatastatus.ALARM_MASK1 = (data[BLUECELL_DATA + i+ 1] / 10);
        printf("%s : %d   ALARM_MASK1 : %d \r\n",__func__,__LINE__,bluecell_Currdatastatus.ALARM_MASK1);
        printf("Txdata : %d \r\n",Txdata[107]);
        break;
  }
  
  if(datatype != Bluecell_StatusReq
  && datatype != Bluecell_LED_TEST)
  {
      Bluecell_StructCpy(&DataWrite[0],&bluecell_Currdatastatus.bluecell_header,sizeof(BLUESTATUS_st));
      EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_WINDOW_STATUS_ADDRESDS),&DataWrite[0],sizeof(BLUESTATUS_st));
//      EEPROM_M24C08_Read(EEPROM_M24C08_ID,EEPROM_WINDOW_STATUS_ADDRESDS,&bluecell_Currdatastatus.bluecell_header,sizeof(BLUESTATUS_st) );
//      Bluecell_StructCpy(&Txdata[0],&DataWrite[0],sizeof(BLUESTATUS_st));
//      Txdata[sizeof(BLUESTATUS_st) - 2] = STH30_CreateCrc(&Txdata[BLUECELL_TYPE], sizeof(BLUESTATUS_st) - 3);
//      Uart1_Data_Send(&Txdata[0], sizeof(BLUESTATUS_st));        
  }
  return true;
}
void DataStatusSet(void){
        bluecell_Currdatastatus.bluecell_header = 0xbe;
        bluecell_Currdatastatus.bluecell_type = Bluecell_StatusReq;
        bluecell_Currdatastatus.bluecell_length = 94 - 3;
        bluecell_Currdatastatus.bluecell_crcindex = 94 - 2 + 1;
//        bluecell_Currdatastatus.ATT_DL1_PATH = HAL_GPIO_ReadPin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin);//CLOCK;        
//        bluecell_Currdatastatus.ATT_DL2_PATH = HAL_GPIO_ReadPin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin);//CLOCK;
//        bluecell_Currdatastatus.ATT_DL3_PATH = HAL_GPIO_ReadPin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin);//CLOCK;
//        bluecell_Currdatastatus.ATT_DL4_PATH = HAL_GPIO_ReadPin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin);//CLOCK;
//        bluecell_Currdatastatus.ATT_UL1_PATH = HAL_GPIO_ReadPin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin);//CLOCK;
//        bluecell_Currdatastatus.ATT_UL2_PATH = HAL_GPIO_ReadPin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin);//CLOCK;
//        bluecell_Currdatastatus.ATT_UL3_PATH = HAL_GPIO_ReadPin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin);//CLOCK;
//        bluecell_Currdatastatus.ATT_UL4_PATH = HAL_GPIO_ReadPin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin);//CLOCK;
        bluecell_Currdatastatus.bluecell_etx = 0xeb;
             
        
     
}
typedef enum{
 MBIC_3_7G  =     0x80  ,              
 MBIC_3_8G     ,
 MBIC_3_9G     ,
 MBIC_4_0G     ,
 MBIC_4_1G     ,
 MBIC_4_6G     ,
 MBIC_4_7G     ,
 MBIC_4_8G     ,
 MBIC_4_9G     ,
 MBIC_5_0G     ,
 MBIC_DCM      ,
 MBIC_SBM      ,
 MBIC_KDDI     ,
}MBIC_CardType_t;
 typedef enum{
    BLUECELL_3_7G  =     0  ,              
    BLUECELL_3_8G     ,
    BLUECELL_3_9G     ,
    BLUECELL_4_0G     ,
    BLUECELL_4_1G     ,
    BLUECELL_4_6G     ,
    BLUECELL_SBM      ,
    BLUECELL_DCM      ,
    BLUECELL_KDDI     ,
    BLUECELL_4_7G     ,
    BLUECELL_4_8G     ,
    BLUECELL_4_9G     ,
    BLUECELL_5_0G     ,
 }Bluecell_CardType_t;

 void Bluecell_DataInit(){
/*
    MBIC Request List 
    Turn off all paths at boot
 */
   bluecell_Currdatastatus.CPUVERSION1 = CPU_VERSION0;
   bluecell_Currdatastatus.CPUVERSION2 = CPU_VERSION1;
   bluecell_Currdatastatus.CPUVERSION3 = CPU_VERSION2;
   
   bluecell_Currdatastatus.Type =  HAL_GPIO_ReadPin(UNIT_ID0_GPIO_Port,UNIT_ID0_Pin);
   bluecell_Currdatastatus.Type +=  HAL_GPIO_ReadPin(UNIT_ID1_GPIO_Port,UNIT_ID1_Pin) << 1;
   bluecell_Currdatastatus.Type +=  HAL_GPIO_ReadPin(UNIT_ID2_GPIO_Port,UNIT_ID2_Pin) << 2;
   bluecell_Currdatastatus.Type +=  HAL_GPIO_ReadPin(UNIT_ID3_GPIO_Port,UNIT_ID3_Pin) << 3;
    switch(bluecell_Currdatastatus.Type){
     case BLUECELL_3_7G: bluecell_Currdatastatus.Type = 0x80;bluecell_Currdatastatus.Carrier_ID = 0x01 ;printf("Type BLUECELL_3_7G  Ready....\r\n"); break;
     case BLUECELL_3_8G: bluecell_Currdatastatus.Type = 0x81;bluecell_Currdatastatus.Carrier_ID = 0x02; printf("Type BLUECELL_3_8G  Ready....\r\n");break;
     case BLUECELL_3_9G: bluecell_Currdatastatus.Type = 0x82;bluecell_Currdatastatus.Carrier_ID = 0x04;printf("Type BLUECELL_3_9G  Ready....\r\n"); break;
     case BLUECELL_4_0G: bluecell_Currdatastatus.Type = 0x83;bluecell_Currdatastatus.Carrier_ID = 0x03; printf("Type BLUECELL_4_0G  Ready....\r\n");break;
     case BLUECELL_4_1G: bluecell_Currdatastatus.Type = 0x84;bluecell_Currdatastatus.Carrier_ID = 0x02; printf("Type BLUECELL_4_1G  Ready....\r\n");break;
     case BLUECELL_4_6G: bluecell_Currdatastatus.Type = 0x85;bluecell_Currdatastatus.Carrier_ID = 0x01; printf("Type BLUECELL_4_6G  Ready....\r\n");break;
     case BLUECELL_4_7G: bluecell_Currdatastatus.Type = 0x86;bluecell_Currdatastatus.Carrier_ID = 0x00; printf("Type BLUECELL_4_7G  Ready....\r\n");break;
     case BLUECELL_4_8G: bluecell_Currdatastatus.Type = 0x87;bluecell_Currdatastatus.Carrier_ID = 0x00; printf("Type BLUECELL_3_7G  Ready....\r\n");break;
     case BLUECELL_4_9G: bluecell_Currdatastatus.Type = 0x88;bluecell_Currdatastatus.Carrier_ID = 0x00; printf("Type BLUECELL_4_9G  Ready....\r\n");break;
     case BLUECELL_5_0G: bluecell_Currdatastatus.Type = 0x89;bluecell_Currdatastatus.Carrier_ID = 0x00; printf("Type BLUECELL_5_0G  Ready....\r\n");break;
     case BLUECELL_DCM : bluecell_Currdatastatus.Type = 0x8A;bluecell_Currdatastatus.Carrier_ID = 0x01;printf("Type BLUECELL_DCM  Ready....\r\n"); break;
     case BLUECELL_SBM : bluecell_Currdatastatus.Type = 0x8B;bluecell_Currdatastatus.Carrier_ID = 0x03; printf("Type BLUECELL_SBM  Ready....\r\n");break;
     case BLUECELL_KDDI: bluecell_Currdatastatus.Type = 0x8C;bluecell_Currdatastatus.Carrier_ID = 0x02;printf("Type BLUECELL_KDDI  Ready....\r\n"); break;
     default : printf("Type %x  Ready....\r\n",bluecell_Currdatastatus.Type);break;
    }
    MBIC_UL_ShutdownCount = &bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1;   
    MBIC_DL_ShutdownCount = &bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1;   
    
    PrevMBIC_UL_ShutdownCount = &bluecell_Prevdatastatus.ULO_Shutdown_Retry_Count1;
    PrevMBIC_DL_ShutdownCount = &bluecell_Prevdatastatus.DLI_Shutdown_Retry_Count1;   
    for(int i = 0; i < DET_Alarm_DL_Shutdown_Index_MAX; i++){
        PrevMBIC_UL_ShutdownCount[i] = 0xFF;
        PrevMBIC_DL_ShutdownCount[i] = 0xFF;        
    }

  printf("Type %x  Ready....\r\n",bluecell_Currdatastatus.Type);

  //  printf("bluecell_Currdatastatus.Type : %02x \r\n",bluecell_Currdatastatus.Type);
#if 1 // PYJ.2020.06.17_BEGIN -- 
    HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,bluecell_Currdatastatus.ATT_DL1_PATH);   
    HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,bluecell_Currdatastatus.ATT_DL2_PATH);   
    HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,bluecell_Currdatastatus.ATT_DL3_PATH);   
    HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,bluecell_Currdatastatus.ATT_DL4_PATH);   

    
    HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,bluecell_Currdatastatus.ATT_UL1_PATH);   
    HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,bluecell_Currdatastatus.ATT_UL2_PATH);   
    printf("bluecell_Currdatastatus.ATT_UL2_PATH : %d \r\n",bluecell_Currdatastatus.ATT_UL2_PATH);
    HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,bluecell_Currdatastatus.ATT_UL3_PATH);   
    HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,bluecell_Currdatastatus.ATT_UL4_PATH);   
#endif // PYJ.2020.06.17_END -- 
    
    HAL_Delay(10);

    CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
    if(bluecell_Currdatastatus.Selftest1==0){
//        printf("Selftest1 : 0 \r\n");
        HAL_GPIO_WritePin(_PATH_SW1_GPIO_Port,_PATH_SW1_Pin,GPIO_PIN_RESET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW1_GPIO_Port,PATH_SW1_Pin,GPIO_PIN_SET);//CLOCK
    }else{
//        printf("Selftest1 : 1 \r\n");
        HAL_GPIO_WritePin(_PATH_SW1_GPIO_Port,_PATH_SW1_Pin,GPIO_PIN_SET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW1_GPIO_Port,PATH_SW1_Pin,GPIO_PIN_RESET);//CLOCK
    }
    if(bluecell_Currdatastatus.Selftest2==0){
//                printf("Selftest1 : 0 \r\n");
        HAL_GPIO_WritePin(_PATH_SW2_GPIO_Port,_PATH_SW2_Pin,GPIO_PIN_RESET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW2_GPIO_Port,PATH_SW2_Pin,GPIO_PIN_SET);//CLOCK
    }else{
//        printf("Selftest2 : 1 \r\n");
        HAL_GPIO_WritePin(_PATH_SW2_GPIO_Port,_PATH_SW2_Pin,GPIO_PIN_SET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW2_GPIO_Port,PATH_SW2_Pin,GPIO_PIN_RESET);//CLOCK
    }
    if(bluecell_Currdatastatus.Selftest3==0){
//                printf("Selftest1 : 0 \r\n");
        HAL_GPIO_WritePin(_PATH_SW3_GPIO_Port,_PATH_SW3_Pin,GPIO_PIN_RESET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW3_GPIO_Port,PATH_SW3_Pin,GPIO_PIN_SET);//CLOCK
    }else{
//                printf("Selftest3 : 1 \r\n");
        HAL_GPIO_WritePin(_PATH_SW3_GPIO_Port,_PATH_SW3_Pin,GPIO_PIN_SET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW3_GPIO_Port,PATH_SW3_Pin,GPIO_PIN_RESET);//CLOCK
    }
    if(bluecell_Currdatastatus.Selftest4==0){
//                printf("Selftest4 : 0 \r\n");
        HAL_GPIO_WritePin(_PATH_SW4_GPIO_Port,_PATH_SW4_Pin,GPIO_PIN_RESET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW4_GPIO_Port,PATH_SW4_Pin,GPIO_PIN_SET);//CLOCK
    }else{
//                printf("Selftest4 : 1 \r\n");
        HAL_GPIO_WritePin(_PATH_SW4_GPIO_Port,_PATH_SW4_Pin,GPIO_PIN_SET);//CLOCK
        HAL_GPIO_WritePin(PATH_SW4_GPIO_Port,PATH_SW4_Pin,GPIO_PIN_RESET);//CLOCK
    }


    DataStatusSet();
}
//uint8_t ResultData[1024] = {0,}; 
uint8_t* MBIC_HeaderDataSetting(uint8_t* data){
    static uint8_t ResultData[1024] = {0,}; /*NOT YET*/ 



	return ResultData;
}
void ALARM_Value_Get(uint8_t datatype,uint8_t* ret){
    switch(datatype){

    }

}

#define MBIC_AID  0xE0
uint32_t MBIC_DataSend(uint8_t* data){
    uint32_t i = 0;
    uint32_t Length = 0;
    int16_t tmpdata = 0;

    
    /*ALARM BIT LIST*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Alarm_Bit_List;
    Length += 10;
    data[MBIC_PAYLOADSTART + i++] = 10; // LENGTH
    

    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK1 = 0x80;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK2 = 0xFF;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK3 = 0xFF;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK4 = 0x0F;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK5 = 0xFF;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_TEMP_HIGH;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_DLI_Level;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_ULO_Level;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm;
    
    /*ALARM BIT MASK*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Alarm_Mask;
    Length += 5;
    data[MBIC_PAYLOADSTART + i++] = 5; // LENGTH
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK4;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_MASK5;
    /*Alarm_Test_Mode*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Alarm_Test_Mode;
    Length += 1;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_TESTMODE;
    /*Alarm_Test_Mode*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Alarm_Test_Dummy;
    Length += 5;
    data[MBIC_PAYLOADSTART + i++] = 5; // LENGTH
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_Test_Dummy1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_Test_Dummy2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_Test_Dummy3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_Test_Dummy4;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ALARM_Test_Dummy5;        

    /*CPU_Version*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Version;
    Length += 3;
    data[MBIC_PAYLOADSTART + i++] = 3; // LENGTH
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPUVERSION1 = CPU_VERSION0;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPUVERSION2 = CPU_VERSION1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPUVERSION3 = CPU_VERSION2;
    


    /*CPU_Current_Bank*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Current_Bank;
    Length += 1;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH        
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Current_Bank;


    /*CPU_Bank1_Image_Version*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Bank1_Image_Version;
    data[MBIC_PAYLOADSTART + i++] = 3; // LENGTH
    Length += 3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_Version1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_Version2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_Version3;

    /*CPU_Bank1_Image_BuildTime*/

    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Bank1_Image_BuildTime;
    data[MBIC_PAYLOADSTART + i++] = 6; // LENGTH
    Length += 6;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_BuildTime1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_BuildTime2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_BuildTime3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_BuildTime4;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_BuildTime5;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_BuildTime6;

    /*CPU_Bank1_Image_Name*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Bank1_Image_Name;
    data[MBIC_PAYLOADSTART + i++] = 32; // LENGTH
    Length += 32;  
    for(int a = 0; a < 32; a++){
        data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank1_Image_Name[a];
    }

    /*CPU_Bank2_Image_Version*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Bank2_Image_Version;
    data[MBIC_PAYLOADSTART + i++] = 3; // LENGTH
    Length += 3;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_Version1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_Version2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_Version3;

    /*CPU_Bank2_Image_BuildTime*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = CPU_Bank2_Image_BuildTime;
    data[MBIC_PAYLOADSTART + i++] = 6; // LENGTH
    Length += 6;          
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_BuildTime1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_BuildTime2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_BuildTime3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_BuildTime4;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_BuildTime5;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_BuildTime6;



   /*CPU_Bank2_Image_Name*/
   data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
   data[MBIC_PAYLOADSTART + i++] = CPU_Bank2_Image_Name;
   data[MBIC_PAYLOADSTART + i++] = 32; // LENGTH
    Length += 32;  
   for(int a = 0; a < 32; a++){
        data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.CPU_Bank2_Image_Name[a];
   }



    /*Temperature*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Temperature;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DET_TEMP;

    /*Temperature_Offset*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Temperature_Offset;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1  ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET;



    /*Temperature_Offset*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Temp_High_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Temp_High_Threshold;


    /*Node*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Node;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.NODE;



    /*Type*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Type;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Type;
    printf(" bluecell_Currdatastatus.Type : %x \r\n", bluecell_Currdatastatus.Type);
     /*PCB_Version*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = PCB_Version;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2;       
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.PCB_Version[0];
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.PCB_Version[1];


     /*Serial_Number*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Serial_Number;
    data[MBIC_PAYLOADSTART + i++] = 20; // LENGTH
    Length += 20     ;      
    for(int a = 0; a < 20; a++){
        data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Serial_Number[a];
    }

     /*Manufacture*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Manufacture;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1     ;      
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Manufacture;




     /*Manufacture_Date*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Manufacture_Date;
    data[MBIC_PAYLOADSTART + i++] = 3; // LENGTH
    Length += 3   ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Manufacture_Date[0];
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Manufacture_Date[1];
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Manufacture_Date[2];

     /*Freq_ID*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Freq_ID;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1     ;      
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Freq_ID;


     /*Carrier_ID*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Carrier_ID;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1       ;   
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Carrier_ID;



     /*Carrier_ID*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = Carrier_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1  ;          
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.Carrier_ON_OFF;


    
     /*DLI_P1_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_P1_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2     ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P1_Level1_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P1_Level1_L;
    
     /*DLI_P2_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_P2_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2   ;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P2_Level2_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P2_Level2_L;

     /*DLI_P3_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_P3_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2   ;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P3_Level3_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P3_Level3_L;

     /*DLI_P4_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_P4_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2   ;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P4_Level4_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_P4_Level4_L;

     /*ULO_P1_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_P1_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2     ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P1_Level1_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P1_Level1_L;
    
     /*ULO_P2_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_P2_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2     ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P2_Level2_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P2_Level2_L;

     /*ULO_P3_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_P3_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2   ;  
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P3_Level3_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P3_Level3_L;

     /*ULO_P4_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_P4_Level;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2     ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P4_Level4_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_P4_Level4_L;  

    


    
     /*DLI_RF_Path1_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
  //  printf("data[%d]:%x\r\n",MBIC_PAYLOADSTART + i-1,data[MBIC_PAYLOADSTART + i-1]);
    data[MBIC_PAYLOADSTART + i++] = DLI_RF_Path1_ON_OFF;
   //  printf("data[%d]:%x\r\n",MBIC_PAYLOADSTART + i-1,data[MBIC_PAYLOADSTART + i-1]);
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1  ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_DL1_PATH;
    // printf("data[%d]:%x\r\n",MBIC_PAYLOADSTART + i-1,data[MBIC_PAYLOADSTART + i-1]);
//    printf("bluecell_Currdatastatus.ATT_DL1_PATH : %d\r\n",bluecell_Currdatastatus.ATT_DL1_PATH);
     /*DLI_RF_Path2_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_RF_Path2_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1          ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_DL2_PATH;
     /*DLI_RF_Path3_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_RF_Path3_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1          ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_DL3_PATH;
     /*DLI_RF_Path4_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_RF_Path4_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1          ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_DL4_PATH;

    /*DLI_Gain_Atten1*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten1;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2  ;
    tmpdata = bluecell_Currdatastatus.ATT_DL1_H << 8 | bluecell_Currdatastatus.ATT_DL1_L;
//    tmpdata *= 10;
//    printf("tmpdata : %x \r\n",tmpdata);
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 


    /*DLI_Gain_Atten2*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten2;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2        ;  
    tmpdata = bluecell_Currdatastatus.ATT_DL2_H << 8 | bluecell_Currdatastatus.ATT_DL2_L;
//    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 


    /*DLI_Gain_Atten3*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten3;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2          ;
    tmpdata = bluecell_Currdatastatus.ATT_DL3_H << 8 | bluecell_Currdatastatus.ATT_DL3_L;
//    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 


    /*DLI_Gain_Atten4*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten4;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2        ;  
    tmpdata = bluecell_Currdatastatus.ATT_DL4_H << 8 | bluecell_Currdatastatus.ATT_DL4_L;
//    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 



     /*DLI_Gain_Atten_Offset1*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten_Offset1;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2;
    tmpdata = bluecell_Currdatastatus.bluecell_User_DL1_H << 8 | bluecell_Currdatastatus.bluecell_User_DL1_L;
//    tmpdata *= 10;    
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

     /*DLI_Gain_Atten_Offset2*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten_Offset2;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2        ;
    tmpdata = bluecell_Currdatastatus.bluecell_User_DL2_H << 8 | bluecell_Currdatastatus.bluecell_User_DL2_L;
//    tmpdata *= 10;   
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

     /*DLI_Gain_Atten_Offset3*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten_Offset3;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2        ;
    tmpdata = bluecell_Currdatastatus.bluecell_User_DL3_H << 8 | bluecell_Currdatastatus.bluecell_User_DL3_L;
//    tmpdata *= 10;   
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

     /*DLI_Gain_Atten_Offset4*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Gain_Atten_Offset4;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2        ;
    tmpdata = bluecell_Currdatastatus.bluecell_User_DL4_H << 8 | bluecell_Currdatastatus.bluecell_User_DL4_L;
//    tmpdata *= 10;   
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 



     /*DLI_Level_High_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Level_High_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2    ;          
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Level_High_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Level_High_Threshold_L;

     /*DLI_Level_Low_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Level_Low_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2     ;         
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Level_Low_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Level_Low_Threshold_L;



     /*DLI_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Level;
    data[MBIC_PAYLOADSTART + i++] = 8; // LENGTH
    Length += 8     ;
    tmpdata = bluecell_Currdatastatus.DLI_Level1_H << 8 | bluecell_Currdatastatus.DLI_Level1_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

    tmpdata = bluecell_Currdatastatus.DLI_Level2_H << 8 | bluecell_Currdatastatus.DLI_Level2_L;
    tmpdata *= 10;    
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

    tmpdata = bluecell_Currdatastatus.DLI_Level3_H << 8 | bluecell_Currdatastatus.DLI_Level3_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

    tmpdata = bluecell_Currdatastatus.DLI_Level4_H << 8 | bluecell_Currdatastatus.DLI_Level4_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 


    
     /*DLI_FRBT_Atten*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_FRBT_Atten;
    data[MBIC_PAYLOADSTART + i++] = 8; // LENGTH
    Length += 8   ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten1_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten1_L;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten2_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten2_L;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten3_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten3_L;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten4_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Atten4_L;



    
     /*DLI_FRBT_D_Day*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_FRBT_D_Day;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1   ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_D_Day;
     /*DLI_FRBT_Status*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_FRBT_Status;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1         ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_FRBT_Status;

    

     /*DLI_AGC_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_AGC_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1    ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_AGC_ON_OFF;




     /*DLI_AGC_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_AGC_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2    ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_AGC_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_AGC_Threshold_L;

     /*DLI_Shutdown_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Shutdown_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1       ;     
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_ON_OFF;

     /*DLI_Shutdown_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Shutdown_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2       ;   
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_Threshold_L;




     /*DLI_Shutdown_Count*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = DLI_Shutdown_Count;
    data[MBIC_PAYLOADSTART + i++] = 4; // LENGTH
    Length += 4        ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4;


     /*ULO_RF_Path1_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_RF_Path1_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1           ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL1_PATH;
     /*ULO_RF_Path3_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_RF_Path2_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1        ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL2_PATH;

     /*ULO_RF_Path3_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_RF_Path3_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1        ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL3_PATH;
     /*ULO_RF_Path4_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_RF_Path4_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1     ;   
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL4_PATH;


     /*ULO_Gain_Atten1*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten1;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2      ;     
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL1_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL1_L;


     /*ULO_Gain_Atten2*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten2;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2      ;     
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL2_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL2_L;

     /*ULO_Gain_Atten3*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten3;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2    ;      
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL3_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL3_L;

     /*ULO_Gain_Atten4*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten4;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2       ;    
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL4_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ATT_UL4_L;
    
     /*ULO_Gain_Atten_Offset1*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten_Offset1;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2      ;      

    tmpdata = bluecell_Currdatastatus.bluecell_User_UL1_H << 8 | bluecell_Currdatastatus.bluecell_User_UL1_L;
//    tmpdata *= 10;   
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

     /*ULO_Gain_Atten_Offset2*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten_Offset2;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2     ;         
    tmpdata = bluecell_Currdatastatus.bluecell_User_UL2_H << 8 | bluecell_Currdatastatus.bluecell_User_UL2_L;
//    tmpdata *= 10;   
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 


     /*ULO_Gain_Atten_Offset3*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten_Offset3;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2        ;      
    tmpdata = bluecell_Currdatastatus.bluecell_User_UL3_H << 8 | bluecell_Currdatastatus.bluecell_User_UL3_L;
//    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 


     /*ULO_Gain_Atten_Offset4*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Gain_Atten_Offset4;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2    ;          
    tmpdata = bluecell_Currdatastatus.bluecell_User_UL4_H << 8 | bluecell_Currdatastatus.bluecell_User_UL4_L;
//    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 





     /*ULO_Level_High_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Level_High_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2   ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Level_High_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Level_High_Threshold_L;





     /*ULO_Level*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Level;
    data[MBIC_PAYLOADSTART + i++] = 8; // LENGTH
    Length += 8           ;
    tmpdata = bluecell_Currdatastatus.ULO_Level1_H << 8 | bluecell_Currdatastatus.ULO_Level1_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 
    tmpdata = bluecell_Currdatastatus.ULO_Level2_H << 8 | bluecell_Currdatastatus.ULO_Level2_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

    tmpdata = bluecell_Currdatastatus.ULO_Level3_H << 8 | bluecell_Currdatastatus.ULO_Level3_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 

    tmpdata = bluecell_Currdatastatus.ULO_Level4_H << 8 | bluecell_Currdatastatus.ULO_Level4_L;
    tmpdata *= 10;
    data[MBIC_PAYLOADSTART + i++] = ((tmpdata & 0xFF00) >> 8);
    data[MBIC_PAYLOADSTART + i++] = (tmpdata & 0x00FF); 




    
     /*ULO_ALC_ON_OFF*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_ALC_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1 ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_ALC_ON_OFF;
     /*ULO_ALC_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_ALC_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2 ;        
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_ALC_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_ALC_Threshold_L;



     /*ULO_ALC_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Shutdown_ON_OFF;
    data[MBIC_PAYLOADSTART + i++] = 1; // LENGTH
    Length += 1;           
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_ON_OFF;

     /*ULO_ALC_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Shutdown_Threshold;
    data[MBIC_PAYLOADSTART + i++] = 2; // LENGTH
    Length += 2;                
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_Threshold_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_Threshold_L;



     /*ULO_Shutdown_Threshold*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_Shutdown_Retry_Count;
    data[MBIC_PAYLOADSTART + i++] = 4; // LENGTH
    Length += 4;          
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4;



#if 1 // PYJ.2020.06.20_BEGIN -- 
     /*ULO_ALC_Atten*/
    data[MBIC_PAYLOADSTART + i++] = MBIC_AID;
    data[MBIC_PAYLOADSTART + i++] = ULO_ALC_Atten;
    data[MBIC_PAYLOADSTART + i++] = 8; // LENGTH
    Length += 8   ;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H;
    data[MBIC_PAYLOADSTART + i++] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;
#endif // PYJ.2020.06.20_END -- 
    uint32_t HeaderLength = 75;
    data[MBIC_LENGTH_0] = (((Length + (3 * HeaderLength)) & 0xFF00) >> 8);    
    data[MBIC_LENGTH_1] = ((Length  + (3 * HeaderLength)) & 0x00FF);  


    data[MBIC_HEADERCHECKSUM_0] = Chksum_Create(data);        
    data[MBIC_PAYLOADSTART + i++] = ((CRC16_Generate(&data[MBIC_PAYLOADSTART], (Length  + (3 * HeaderLength))) & 0xFF00) >> 8);  
    data[MBIC_PAYLOADSTART + i++] = CRC16_Generate(&data[MBIC_PAYLOADSTART], (Length  + (3 * HeaderLength))) & 0x00FF;  
    data[MBIC_PAYLOADSTART + i++] = 0x03; 
    Uart1_Data_Send(data, (Length  + (3 * HeaderLength)) + 22 + 3);
    return Length;

}
bool MBIC_Operate(uint8_t* data){
	uint8_t datatype = data[MBIC_PAYLOADSTART + 1];
   // uint8_t Length = (data[MBIC_PROT_SUB_DATA_INDEX + 2]);
    uint8_t cmd = data[MBIC_PROT_CMD_INDEX];

    //uint16_t Temp_ADC = 0;
    //uint16_t i = 0;
    //double temp = 0;
    //int16_t Level = 0;
    //int16_t tmpdata = 0;
    uint16_t Length = data[MBIC_LENGTH_0] << 8 | data[MBIC_LENGTH_1];
    //uint8_t* Tempdata;
    uint16_t occurlen = Length;
    /*AID*/
  /* for(int i = 0; i < Length; i++){
        SubData[i] = (data[MBIC_PROT_SUB_DATA_INDEX + 3 + i]);
    }*/
    //    SubData     임시 데이터 변수 선언 Subdata로 데이터 전송


	data[MBIC_PREAMBLE_0] = MBIC_PREAMBLE0;
	data[MBIC_PREAMBLE_1] = MBIC_PREAMBLE1;
	data[MBIC_PREAMBLE_2] = MBIC_PREAMBLE2;
	data[MBIC_PREAMBLE_3] = MBIC_PREAMBLE3;

	data[MBIC_SUBUID_0] = MBIC_SUBUID0;
	data[MBIC_SUBUID_1] = MBIC_SUBUID1;

	data[MBIC_RCODE_0] = data[MBIC_RCODE_0];

	data[MBIC_TRID_0] = data[MBIC_TRID_0];
	data[MBIC_TRID_1] = data[MBIC_TRID_1];

	data[MBIC_SEQSUM_0] = data[MBIC_SEQSUM_0];

	data[MBIC_TTL_0] = data[MBIC_TTL_0];

    data[MBIC_TIME_0] = data[MBIC_TIME_0];
	data[MBIC_TIME_1] = data[MBIC_TIME_1];
	data[MBIC_TIME_2] = data[MBIC_TIME_2];
	data[MBIC_TIME_3] = data[MBIC_TIME_3];
	data[MBIC_TIME_4] = data[MBIC_TIME_4];
	data[MBIC_TIME_5] = data[MBIC_TIME_5];

    data[MBIC_ERRRESPONSE_0] = MBIC_ERRRESPONSE;

    if(cmd == MBIC_GET){
        if(Initialize == false){
            Bluecell_DataInit();
            Bluecell_AttenInitialize();
            Initialize = true;
        }
        data[MBIC_CMD_0] = 0x80;//MBIC_ERRRESPONSE;
        Length = MBIC_DataSend(data);
    }
    else if(cmd == MBIC_SET){
       protocolReplay:
        datatype = data[MBIC_PAYLOADSTART + 1];
        switch(datatype){
            case  Alarm_Mask                              :
                bluecell_Currdatastatus.ALARM_MASK1 = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ALARM_MASK2 = data[MBIC_PAYLOADSTART + 4]; 
                bluecell_Currdatastatus.ALARM_MASK3 = data[MBIC_PAYLOADSTART + 5]; 
                bluecell_Currdatastatus.ALARM_MASK4 = data[MBIC_PAYLOADSTART + 6]; 
                bluecell_Currdatastatus.ALARM_MASK5 = data[MBIC_PAYLOADSTART + 7]; 
                break;
            case  Alarm_Test_Mode                         :
                bluecell_Currdatastatus.ALARM_TESTMODE = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  Alarm_Test_Dummy                        :
                bluecell_Currdatastatus.ALARM_Test_Dummy1 = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ALARM_Test_Dummy2 = data[MBIC_PAYLOADSTART + 4];
                bluecell_Currdatastatus.ALARM_Test_Dummy3 = data[MBIC_PAYLOADSTART + 5];
                bluecell_Currdatastatus.ALARM_Test_Dummy4 = data[MBIC_PAYLOADSTART + 6];
                bluecell_Currdatastatus.ALARM_Test_Dummy5 = data[MBIC_PAYLOADSTART + 7];
                break;
            case  CPU_Bank_Select_Reboot_by               :
                bluecell_Currdatastatus.CPU_Bank_Select = data[MBIC_PAYLOADSTART + 3]; 

                break;
            case  SW_Reset                                :
                bluecell_Currdatastatus.S_W_Reset = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  Factory_Set_Initialization              :
                bluecell_Currdatastatus.Factory_Set_Initialization = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  Temperature_Offset                      :
                bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET = data[MBIC_PAYLOADSTART + 3]; 
                printf("%s : %d \r\n",__func__,__LINE__);
                break;
            case  Temp_High_Threshold                     :
                bluecell_Currdatastatus.Temp_High_Threshold = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  Temp_High_Threshold_Default             :
                bluecell_Currdatastatus.Temp_High_Threshold_Default = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  LED_TEST                                :
                bluecell_Currdatastatus.LED_TEST = data[MBIC_PAYLOADSTART + 3]; 
            case  Carrier_ON_OFF                          :

                break;
            case  DLI_RF_Path1_ON_OFF             :
                bluecell_Currdatastatus.ATT_DL1_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 = 0;
                bluecell_Prevdatastatus.DLI_Shutdown_Retry_Count1 = 0xFF;
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_DL1_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_DL1_L =0xFF;
                    HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_SET);//CLOCK
                }                
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                break;
            case  DLI_RF_Path2_ON_OFF                     :
                bluecell_Currdatastatus.ATT_DL2_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 = 0;
                bluecell_Prevdatastatus.DLI_Shutdown_Retry_Count2 = 0xFF;
               if(data[MBIC_PAYLOADSTART + 3]==0){
                   HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_RESET);//CLOCK        
               }
               else{
                    bluecell_Prevdatastatus.ATT_DL2_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_DL2_L = 0xFF;
                   HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_SET);//CLOCK
               }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_RF_Path3_ON_OFF                     :
                bluecell_Currdatastatus.ATT_DL3_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 = 0;
                bluecell_Prevdatastatus.DLI_Shutdown_Retry_Count3 = 0xFF;
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_DL3_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_DL3_L = 0xFF;
                    HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_SET);//CLOCK
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_RF_Path4_ON_OFF                     :
                bluecell_Currdatastatus.ATT_DL4_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 = 0;
                bluecell_Prevdatastatus.DLI_Shutdown_Retry_Count4 = 0xFF;
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_DL4_H =0xFF;
                    bluecell_Prevdatastatus.ATT_DL4_L = 0xFF;
                    HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_SET);//CLOCK
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Gain_Atten1                         :
                bluecell_Currdatastatus.ATT_DL1_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_DL1_L = data[MBIC_PAYLOADSTART + 4]; 
                printf("bluecell_Currdatastatus.ATT_DL1_H : %x\r\n",bluecell_Currdatastatus.ATT_DL1_H);
                printf("bluecell_Currdatastatus.ATT_DL1_L : %x\r\n",bluecell_Currdatastatus.ATT_DL1_L);              
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                break;
            case  DLI_Gain_Atten2                         :
                bluecell_Currdatastatus.ATT_DL2_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_DL2_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                printf("bluecell_Currdatastatus.ATT_DL2_H : %x\r\n",bluecell_Currdatastatus.ATT_DL2_H);
                printf("bluecell_Currdatastatus.ATT_DL2_L : %x\r\n",bluecell_Currdatastatus.ATT_DL2_L); 

                break;
            case  DLI_Gain_Atten3                         :
                bluecell_Currdatastatus.ATT_DL3_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_DL3_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Gain_Atten4                         :
                bluecell_Currdatastatus.ATT_DL4_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_DL4_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Gain_Atten_Offset1                  :
                bluecell_Currdatastatus.bluecell_User_DL1_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_DL1_L = data[MBIC_PAYLOADSTART + 4]; 
//                printf("bluecell_Currdatastatus.bluecell_User_DL1_H : %x\r\n",bluecell_Currdatastatus.bluecell_User_DL1_H);
//                printf("bluecell_Currdatastatus.bluecell_User_DL1_L : %x\r\n",bluecell_Currdatastatus.bluecell_User_DL1_L); 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Gain_Atten_Offset2                  :
                bluecell_Currdatastatus.bluecell_User_DL2_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_DL2_L = data[MBIC_PAYLOADSTART + 4];
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Gain_Atten_Offset3                  :
                bluecell_Currdatastatus.bluecell_User_DL3_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_DL3_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Gain_Atten_Offset4                  :
                bluecell_Currdatastatus.bluecell_User_DL4_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_DL4_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  DLI_Level_High_Threshold                :
                bluecell_Currdatastatus.DLI_Level_High_Threshold_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Level_High_Threshold_L = data[MBIC_PAYLOADSTART + 4]; 

                break;
            case  DLI_Level_Low_Threshold                 :
                bluecell_Currdatastatus.DLI_Level_Low_Threshold_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Level_Low_Threshold_L = data[MBIC_PAYLOADSTART + 4]; 

                break;
            case  DLI_Level_High_Low_Threshold_default    :
                bluecell_Currdatastatus.DLI_Level_High_Low_Threshold_default = data[MBIC_PAYLOADSTART + 3]; 

                break;
            case DLI_FRBT_D_Day: 
                bluecell_Currdatastatus.DLI_FRBT_D_Day = data[MBIC_PAYLOADSTART + 3];
                break;

            case  DLI_AGC_ON_OFF                   :
                /*AGC multi apply*/
                bluecell_Currdatastatus.DLI_AGC_ON_OFF = data[MBIC_PAYLOADSTART + 3];
                DL_AGC_StartAtten[AGC_Alarm_DL1_Index] 
                    = bluecell_Currdatastatus.ATT_DL1_H << 8 | bluecell_Currdatastatus.ATT_DL1_L;
                DL_AGC_StartAtten[AGC_Alarm_DL2_Index] 
                    = bluecell_Currdatastatus.ATT_DL2_H << 8 | bluecell_Currdatastatus.ATT_DL2_L;
                DL_AGC_StartAtten[AGC_Alarm_DL3_Index] 
                    = bluecell_Currdatastatus.ATT_DL3_H << 8 | bluecell_Currdatastatus.ATT_DL3_L;
                DL_AGC_StartAtten[AGC_Alarm_DL4_Index] 
                    = bluecell_Currdatastatus.ATT_DL4_H << 8 | bluecell_Currdatastatus.ATT_DL4_L;

                for(int i = 0; i < AGC_Alarm_DL_Index_MAX; i++){
                    DL_PrevIwillgiveAtten[i]= 0;                
                }
                break;
            case  DLI_AGC_Threshold                       :
                bluecell_Currdatastatus.DLI_AGC_Threshold_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_AGC_Threshold_L = data[MBIC_PAYLOADSTART + 4]; 

                break;
            case  DLI_AGC_Threshold_Default               :
                bluecell_Currdatastatus.DLI_AGC_Threshold_default = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  DLI_Shutdown_ON_OFF                     :
                bluecell_Currdatastatus.DLI_Shutdown_ON_OFF = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 = 0;
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 = 0;
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 = 0;
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 = 0;                
                for(int i = 0; i < DET_Alarm_DL_Index_MAX; i++){
                    DET_DL_Shutdown_Off_AlarmTimerCnt[i] = 0;
                    DET_DL_Shutdown_On_AlarmTimerCnt[i] = 0;
                    DET_DL_Normal_Shutdown_On_AlarmTimerCnt[i] = 0;
                }
                break;
            case  DLI_Shutdown_Threshold                  :
                bluecell_Currdatastatus.DLI_Shutdown_Threshold_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.DLI_Shutdown_Threshold_L = data[MBIC_PAYLOADSTART + 4]; 

                break;
            case  DLI_Shutdown_Threshold_Default          :
                bluecell_Currdatastatus.DLI_Shutdown_Threshold_Default = data[MBIC_PAYLOADSTART + 3]; 

                break;
            case  ULO_RF_Path1_ON_OFF            :
                bluecell_Currdatastatus.ATT_UL1_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 = 0;
                bluecell_Prevdatastatus.ULO_Shutdown_Retry_Count1 = 0xFF;
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_UL1_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL1_L = 0xFF;
                    HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_SET);//CLOCK
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                break;
            case  ULO_RF_Path2_ON_OFF                    :
                bluecell_Currdatastatus.ATT_UL2_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2 = 0;
                bluecell_Prevdatastatus.ULO_Shutdown_Retry_Count2 = 0xFF;
                printf("UL SHUTDOWN RECOVERY OPERATE PATH : %d \r\n",bluecell_Currdatastatus.ATT_UL2_PATH);
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_UL2_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL2_L = 0xFF;
                    HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_SET);//CLOCK
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);


                break;
            case  ULO_RF_Path3_ON_OFF                    :
                bluecell_Currdatastatus.ATT_UL3_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3 = 0;
                bluecell_Prevdatastatus.ULO_Shutdown_Retry_Count3 = 0xFF;
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_UL3_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL3_L =0xFF;
                    HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_SET);//CLOCK
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);


                break;
            case  ULO_RF_Path4_ON_OFF                    :
                bluecell_Currdatastatus.ATT_UL4_PATH = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4 = 0;
                bluecell_Prevdatastatus.ULO_Shutdown_Retry_Count4 = 0xFF;
                if(data[MBIC_PAYLOADSTART + 3]==0){
                    HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_RESET);//CLOCK        
                }
                else{
                    bluecell_Prevdatastatus.ATT_UL4_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL4_L = 0xFF;
                    HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_SET);//CLOCK
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);


                break;
            case  ULO_Gain_Atten1                         :
                bluecell_Currdatastatus.ATT_UL1_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_UL1_L = data[MBIC_PAYLOADSTART + 4]; 
                if(bluecell_Currdatastatus.ULO_ALC_ON_OFF == true){
                    UL_ALC_GainAttenSet[ALC_Alarm_UL1_Index] = true;
                    ALC_Level_Save[ALC_Alarm_UL1_Index] = bluecell_Currdatastatus.ULO_Level1_H << 8 | bluecell_Currdatastatus.ULO_Level1_L;
                }
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten2                         :
                bluecell_Currdatastatus.ATT_UL2_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_UL2_L = data[MBIC_PAYLOADSTART + 4]; 
                if(bluecell_Currdatastatus.ULO_ALC_ON_OFF == true){
                    UL_ALC_GainAttenSet[ALC_Alarm_UL2_Index] = true;
                    ALC_Level_Save[ALC_Alarm_UL2_Index] = bluecell_Currdatastatus.ULO_Level2_H << 8 | bluecell_Currdatastatus.ULO_Level2_L;
                }
            
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten3                         :
                bluecell_Currdatastatus.ATT_UL3_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_UL3_L = data[MBIC_PAYLOADSTART + 4]; 
                if(bluecell_Currdatastatus.ULO_ALC_ON_OFF == true){
                    UL_ALC_GainAttenSet[ALC_Alarm_UL3_Index] = true;
                    ALC_Level_Save[ALC_Alarm_UL3_Index] = bluecell_Currdatastatus.ULO_Level3_H << 8 | bluecell_Currdatastatus.ULO_Level3_L;
                }
            
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten4                         :
                bluecell_Currdatastatus.ATT_UL4_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ATT_UL4_L = data[MBIC_PAYLOADSTART + 4]; 
                if(bluecell_Currdatastatus.ULO_ALC_ON_OFF == true){
                    UL_ALC_GainAttenSet[ALC_Alarm_UL4_Index] = true;
                    ALC_Level_Save[ALC_Alarm_UL4_Index] = bluecell_Currdatastatus.ULO_Level4_H << 8 | bluecell_Currdatastatus.ULO_Level4_L  ;                      
                }
            
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten_Offset1                  :
                bluecell_Currdatastatus.bluecell_User_UL1_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_UL1_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten_Offset2                  :
                bluecell_Currdatastatus.bluecell_User_UL2_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_UL2_L = data[MBIC_PAYLOADSTART + 4]; 
                printf("ATTEN OFFSET START \r\n");
                printf("OFFSET HIGH : %02X \r\n", bluecell_Currdatastatus.bluecell_User_UL2_H);
                printf("OFFSET LOW : %02X \r\n", bluecell_Currdatastatus.bluecell_User_UL2_L);                
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten_Offset3                  :
                bluecell_Currdatastatus.bluecell_User_UL3_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_UL3_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Gain_Atten_Offset4                  :
                bluecell_Currdatastatus.bluecell_User_UL4_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.bluecell_User_UL4_L = data[MBIC_PAYLOADSTART + 4]; 
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);

                break;
            case  ULO_Level_High_Threshold                :
                bluecell_Currdatastatus.ULO_Level_High_Threshold_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ULO_Level_High_Threshold_L = data[MBIC_PAYLOADSTART + 4]; 
                printf("bluecell_Currdatastatus.ULO_Level_High_Threshold_H : %x \r\n",bluecell_Currdatastatus.ULO_Level_High_Threshold_H);
                printf("bluecell_Currdatastatus.ULO_Level_High_Threshold_L : %x \r\n",bluecell_Currdatastatus.ULO_Level_High_Threshold_L);    
                break;
            case  ULO_Level_High_Threshold_default        :
                bluecell_Currdatastatus.ULO_Level_High_Threshold_default = data[MBIC_PAYLOADSTART + 3]; 
                break;
            case  ULO_ALC_ON_OFF                   :
                bluecell_Currdatastatus.ULO_ALC_ON_OFF = data[MBIC_PAYLOADSTART + 3]; 


                break;
            case  ULO_ALC_Threshold                       :
                bluecell_Currdatastatus.ULO_ALC_Threshold_H = data[MBIC_PAYLOADSTART + 3];
                bluecell_Currdatastatus.ULO_ALC_Threshold_L = data[MBIC_PAYLOADSTART + 4];

                break;
            case  ULO_ALC_Threshold_Default               :
                bluecell_Currdatastatus.ULO_ALC_Threshold_Default = data[MBIC_PAYLOADSTART + 3]; 

                break;
            case  ULO_Shutdown_ON_OFF                     :
                bluecell_Currdatastatus.ULO_Shutdown_ON_OFF = data[MBIC_PAYLOADSTART + 3]; 

                break;
            case  ULO_Shutdown_Threshold                  :
                bluecell_Currdatastatus.ULO_Shutdown_Threshold_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.ULO_Shutdown_Threshold_L = data[MBIC_PAYLOADSTART + 4]; 

                break;
            case  ULO_Shutdown_Threshold_Default          :
                bluecell_Currdatastatus.ULO_Shutdown_Threshold_Default = data[MBIC_PAYLOADSTART + 3]; 

            
                break;
            case ULO_ALC_Atten:
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H = data[MBIC_PAYLOADSTART + 3]; 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L = data[MBIC_PAYLOADSTART + 4]; 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H = data[MBIC_PAYLOADSTART + 5]; 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L = data[MBIC_PAYLOADSTART + 6]; 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H = data[MBIC_PAYLOADSTART + 7]; 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L = data[MBIC_PAYLOADSTART + 8];                 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H = data[MBIC_PAYLOADSTART + 9]; 
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L = data[MBIC_PAYLOADSTART + 10];  
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                break;
        }
        if(data[MBIC_PAYLOADSTART + data[MBIC_PAYLOADSTART + 2] + 3 ] == 0xE0 && occurlen > 0){
            occurlen -= data[MBIC_PAYLOADSTART + 2] + 3;
            for(int k = 0; k < occurlen; k++){
                data[MBIC_PAYLOADSTART + k] = data[MBIC_PAYLOADSTART + data[MBIC_PAYLOADSTART + 2] + 3 + k];
                
            }
            goto protocolReplay;
        }
         data[MBIC_CMD_0] = 0x81;//MBIC_ERRRESPONSE;
        Length = MBIC_DataSend(data);
        Bluecell_StructCpy(&DataWrite[0],&bluecell_Currdatastatus.bluecell_header,sizeof(BLUESTATUS_st));
        EEPROM_M24C08_write(EEPROM_M24C08_ID ,(EEPROM_WINDOW_STATUS_ADDRESDS),&DataWrite[0],sizeof(BLUESTATUS_st));
    }
    else if(cmd == MBIC_Table_Get){
        /*Table Get */
//            data[MBIC_PAYLOADSTART + 1] //Reserve Data
        //protocolTableLoadReplay:

        switch(data[MBIC_PAYLOADSTART + 2]){
        	case DLI_P1_Level_Table_Number                 :
        	case DLI_P2_Level_Table_Number                 :
        	case DLI_P3_Level_Table_Number                 :
        	case DLI_P4_Level_Table_Number                 :
        	case ULO_P1_Level_Table_Number                 :
        	case ULO_P2_Level_Table_Number                 :
        	case ULO_P3_Level_Table_Number                 :
        	case ULO_P4_Level_Table_Number                 :
        	case DLI_P1_ATT_Temp_guarantee_Table_Number    :
        	case DLI_P2_ATT_Temp_guarantee_Table_Number    :
        	case DLI_P3_ATT_Temp_guarantee_Table_Number    :
        	case DLI_P4_ATT_Temp_guarantee_Table_Number    :
        	case ULO_P1_ATT_Temp_guarantee_Table_Number    :
        	case ULO_P2_ATT_Temp_guarantee_Table_Number    :
        	case ULO_P3_ATT_Temp_guarantee_Table_Number    :
        	case ULO_P4_ATT_Temp_guarantee_Table_Number    :
        	case DLI_P1_ATT_Accuracy_Table_Number          :
        	case DLI_P2_ATT_Accuracy_Table_Number          :
        	case DLI_P3_ATT_Accuracy_Table_Number          :
        	case DLI_P4_ATT_Accuracy_Table_Number          :
        	case ULO_P1_ATT_Accuracy_Table_Number          :
        	case ULO_P2_ATT_Accuracy_Table_Number          :
        	case ULO_P3_ATT_Accuracy_Table_Number          :
        	case ULO_P4_ATT_Accuracy_Table_Number          :
                MBIC_TableLoad(data,data[MBIC_PAYLOADSTART + 2]);
                if(data[MBIC_PAYLOADSTART + MBIC_TableIndex_Number] >= DLI_P1_ATT_Accuracy_Table_Number
                 &&data[MBIC_TableIndex_Number] <= ULO_P4_ATT_Accuracy_Table_Number ){                
                    MBIC_HeaderMergeFunction(data,data[MBIC_PAYLOADSTART + 4] + 5);
                    data[MBIC_HEADERCHECKSUM_0] = Chksum_Create(data);        
                    uint16_t crcret = ((CRC16_Generate(&data[MBIC_PAYLOADSTART], (data[MBIC_PAYLOADSTART + 4] ) + 5) ) );
                        printf("\r\n crc ret : %x \r\n",crcret);
                    data[MBIC_PAYLOADSTART + (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] ) + 5] =(( crcret  & 0xFF00)>> 8);  
                    data[MBIC_PAYLOADSTART + (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] ) + 6] = (( crcret  & 0x00FF));  
                    data[MBIC_PAYLOADSTART + (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] ) + 7] = 0x03; 

                    Uart1_Data_Send(data, (data[MBIC_PAYLOADSTART + 4]) + 5 + 22 + 3);

                }else{
                    MBIC_HeaderMergeFunction(data,(data[MBIC_PAYLOADSTART + 4] * 2) + 5);
                    data[MBIC_HEADERCHECKSUM_0] = Chksum_Create(data);        
                    uint16_t crcret = ((CRC16_Generate(&data[MBIC_PAYLOADSTART], (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] * 2) + 5) ) );
                        printf("\r\n crc ret : %x \r\n",crcret);
                    data[MBIC_PAYLOADSTART + (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] * 2) + 5] =(( crcret  & 0xFF00)>> 8);  
                    data[MBIC_PAYLOADSTART + (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] * 2) + 6] = (( crcret  & 0x00FF));  
                    data[MBIC_PAYLOADSTART + (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] * 2) + 7] = 0x03;
                    printf("==data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] : %d \r\n",data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length]);
                    Uart1_Data_Send(data, (data[MBIC_PAYLOADSTART + MBIC_TableIndex_Length] * 2)    + 5 + 22 + 3);
                }
                
            break;
       }
       
//        if(/*data[MBIC_PAYLOADSTART + data[MBIC_PAYLOADSTART + 4] + 2 ] == 0xE0 && */occurlen > 0){
//            occurlen -= data[MBIC_PAYLOADSTART + 2] + 5;
//            for(int k = 0; k < occurlen; k++){
//                data[MBIC_PAYLOADSTART + k] = data[MBIC_PAYLOADSTART + data[MBIC_PAYLOADSTART + 2] + 5 + k];
//            }
//            goto protocolTableLoadReplay;
//        }        
    }
    else if(cmd == MBIC_Table_Set){
           // protocolTableSaveReplay:
//            data[MBIC_PAYLOADSTART + 1] //Reserve Data
            switch(data[MBIC_PAYLOADSTART + 2]){
                case DLI_P1_Level_Table_Number                 :
                case DLI_P2_Level_Table_Number                 :
                case DLI_P3_Level_Table_Number                 :
                case DLI_P4_Level_Table_Number                 :
                case ULO_P1_Level_Table_Number                 :
                case ULO_P2_Level_Table_Number                 :
                case ULO_P3_Level_Table_Number                 :
                case ULO_P4_Level_Table_Number                 :
                case DLI_P1_ATT_Temp_guarantee_Table_Number    :
                case DLI_P2_ATT_Temp_guarantee_Table_Number    :
                case DLI_P3_ATT_Temp_guarantee_Table_Number    :
                case DLI_P4_ATT_Temp_guarantee_Table_Number    :
                case ULO_P1_ATT_Temp_guarantee_Table_Number    :
                case ULO_P2_ATT_Temp_guarantee_Table_Number    :
                case ULO_P3_ATT_Temp_guarantee_Table_Number    :
                case ULO_P4_ATT_Temp_guarantee_Table_Number    :
                case DLI_P1_ATT_Accuracy_Table_Number          :
                case DLI_P2_ATT_Accuracy_Table_Number          :
                case DLI_P3_ATT_Accuracy_Table_Number          :
                case DLI_P4_ATT_Accuracy_Table_Number          :
                case ULO_P1_ATT_Accuracy_Table_Number          :
                case ULO_P2_ATT_Accuracy_Table_Number          :
                case ULO_P3_ATT_Accuracy_Table_Number          :
                case ULO_P4_ATT_Accuracy_Table_Number          :
              //Header 문장 + sizeof(ATT_TABLE_st) + EXT 문장 Length 추가
                MBIC_TableSave(data,data[MBIC_PAYLOADSTART + 2]);
                if(data[MBIC_PAYLOADSTART + MBIC_TableIndex_Number] >= DLI_P1_ATT_Accuracy_Table_Number
                 &&data[MBIC_TableIndex_Number] <= ULO_P4_ATT_Accuracy_Table_Number ){                
                    MBIC_HeaderMergeFunction(data,1);
                    data[MBIC_HEADERCHECKSUM_0] = Chksum_Create(data);
//                    Uart1_Data_Send(data, (data[MBIC_PAYLOADSTART + 4]) + 5 + 22 + 3);
                    uint16_t crcret = (CRC16_Generate(&data[MBIC_PAYLOADSTART], 1 ) );
                        printf("\r\n crc ret : %x \r\n",crcret);
                    data[MBIC_PAYLOADSTART + 1] =(( crcret  & 0xFF00)>> 8);  
                    data[MBIC_PAYLOADSTART + 2] = (( crcret  & 0x00FF));  
                    data[MBIC_PAYLOADSTART + 3] = 0x03; 

                    Uart1_Data_Send(data,   22 + 3 + 1);

                }
                else{
                    MBIC_HeaderMergeFunction(data,1);
                    data[MBIC_HEADERCHECKSUM_0] = Chksum_Create(data);
                    uint16_t crcret = (CRC16_Generate(&data[MBIC_PAYLOADSTART], 1) ) ;
                    printf("\r\n crc ret : %x \r\n",crcret);
                    data[MBIC_PAYLOADSTART + 1] =(( crcret  & 0xFF00)>> 8);  
                    data[MBIC_PAYLOADSTART + 2] = (( crcret  & 0x00FF));  
                    data[MBIC_PAYLOADSTART + 3] = 0x03; 

                    Uart1_Data_Send(data,   22 + 3 + 1);
//                    Uart1_Data_Send(data, (data[MBIC_PAYLOADSTART + 4] * 2) + 5 + 22 + 3);
                }                
            }
//            if(/*data[MBIC_PAYLOADSTART + data[MBIC_PAYLOADSTART + 4] + 2 ] == 0xE0 && */occurlen > 0){
//                occurlen -= data[MBIC_PAYLOADSTART + 2] + 5;
//                for(int k = 0; k < occurlen; k++){
//                    data[MBIC_PAYLOADSTART + k] = data[MBIC_PAYLOADSTART + data[MBIC_PAYLOADSTART + 2] + 5 + k];
//                }
//                goto protocolTableSaveReplay;
//            }

    }
    else{
//        MBIC_Bootloader_FirmwareUpdate();
        /*NOP*/
        printf("DATA ERR\r\n");
    }
//    Uart1_Data_Send(&data[0], data[BLUECELL_LENGTH] + 3);
	return true;
}

uint16_t Ascendingcompare(const void *a, const void *b)    // 오름차순 비교 함수 구현
{
    uint16_t num1 = *(int *)a;    // void 포인터를 int 포인터로 변환한 뒤 역참조하여 값을 가져옴
    uint16_t num2 = *(int *)b;    // void 포인터를 int 포인터로 변환한 뒤 역참조하여 값을 가져옴

    if (num1 < num2)    // a가 b보다 작을 때는
        return -1;      // -1 반환
    
    if (num1 > num2)    // a가 b보다 클 때는
        return 1;       // 1 반환
    
    return 0;    // a와 b가 같을 때는 0 반환
}
uint16_t Descendingcompare(const void *a, const void *b)    // 내림차순 비교 함수 구현
{
    uint16_t num1 = *(uint16_t *)a;    // void 포인터를 uint16_t 포인터로 변환한 뒤 역참조하여 값을 가져옴
    uint16_t num2 = *(uint16_t *)b;    // void 포인터를 uint16_t 포인터로 변환한 뒤 역참조하여 값을 가져옴

    if (num1 > num2)    // a가 b보다 클 때는
        return -1;      // -1 반환
    
    if (num1 < num2)    // a가 b보다 작을 때는
        return 1;       // 1 반환
    
    return 0;           // a와 b가 같을 때는 0 반환
}
void DascendigFunc(uint16_t* data,uint32_t size ){
  int temp;
  for(int i = 0 ; i < size - 1 ; i ++) {
      for(int j = i+1 ; j < size ; j ++) {
          if(data[i] < data[j]) {
              temp = data[j];
              data[j] = data[i];
              data[i] = temp;
        }
      }
  }
}

uint32_t SumFunc(uint16_t* data,uint16_t size){
    uint32_t ret = 0;
    for (uint16_t i = 0; i < size; i++)    // 배열의 요소 개수만큼 반복
    {
        ret += data[i];    // sum과 배열의 요소를 더해서 다시 sum에 저장
    }
    return ret;
}




bool ADC_Alarm_DL_High_Set[DET_Alarm_DL_Index_MAX] = {false,} ;
bool ADC_Alarm_DL_Low_Set[DET_Alarm_DL_Index_MAX] = {false,} ;

bool ADC_Alarm_UL_Set[DET_Alarm_UL_Index_MAX] = {false,} ;
bool ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL_Shutdown_Index_MAX] = {false,} ;
bool ADC_Alarm_UL_Shutdown_Set[DET_Alarm_UL_Shutdown_Index_MAX] = {false,} ;

bool ADC_Alarm_DL_Normal_Shutdown_Set[DET_Alarm_DL_Shutdown_Index_MAX] = {false,} ;
bool ADC_Alarm_UL_Normal_Shutdown_Set[DET_Alarm_UL_Shutdown_Index_MAX] = {false,} ;

void DET_LevelAlarmCheck(){
    //int16_t DL[DET_Alarm_DL_Index_MAX] = {0,};
    //int16_t UL[DET_Alarm_UL_Index_MAX] = {0,};
    int16_t LimitData_UL_High = 0;
    
    int16_t LimitData_DL_High = 0;    
    int16_t LimitData_DL_Low = 0;    


    int16_t LimitData_DL_Shutdown = 0;    
    int16_t LimitData_UL_Shutdown = 0;    
    
    double ret = 0;
    int16_t Res_DL_dBm[DET_Alarm_DL_Index_MAX] = {0,};
    int16_t Res_UL_dBm[DET_Alarm_UL_Index_MAX] = {0,};    
    

    LimitData_UL_High = ConvertTo2byte(bluecell_Currdatastatus.ULO_Level_High_Threshold_H,bluecell_Currdatastatus.ULO_Level_High_Threshold_L) / 10;
//    printf("bluecell_Currdatastatus.ULO_Level_High_Threshold_H : %x \r\n",bluecell_Currdatastatus.ULO_Level_High_Threshold_H);
//    printf("bluecell_Currdatastatus.ULO_Level_High_Threshold_L : %x \r\n",bluecell_Currdatastatus.ULO_Level_High_Threshold_L);    
//    printf("LimitData_UL_High : %d \r\n",LimitData_UL_High);
    LimitData_DL_High = ConvertTo2byte(bluecell_Currdatastatus.DLI_Level_High_Threshold_H,bluecell_Currdatastatus.DLI_Level_High_Threshold_L)/ 10;
    LimitData_DL_Low = ConvertTo2byte(bluecell_Currdatastatus.DLI_Level_Low_Threshold_H,bluecell_Currdatastatus.DLI_Level_Low_Threshold_L)/ 10;

    LimitData_DL_Shutdown = ConvertTo2byte(bluecell_Currdatastatus.DLI_Shutdown_Threshold_H,bluecell_Currdatastatus.DLI_Shutdown_Threshold_L)/ 10;
    LimitData_UL_Shutdown = ConvertTo2byte(bluecell_Currdatastatus.ULO_Shutdown_Threshold_H,bluecell_Currdatastatus.ULO_Shutdown_Threshold_L)/ 10;
    
    /*DL[DET_Alarm_DL1_Index] = ConvertTo2byte(bluecell_Currdatastatus.DLI_P1_Level1_H,bluecell_Currdatastatus.DLI_P1_Level1_L);
    DL[DET_Alarm_DL2_Index] = ConvertTo2byte(bluecell_Currdatastatus.DLI_P2_Level2_H,bluecell_Currdatastatus.DLI_P2_Level2_L);
    DL[DET_Alarm_DL3_Index] = ConvertTo2byte(bluecell_Currdatastatus.DLI_P3_Level3_H,bluecell_Currdatastatus.DLI_P3_Level3_L);
    DL[DET_Alarm_DL4_Index] = ConvertTo2byte(bluecell_Currdatastatus.DLI_P4_Level4_H,bluecell_Currdatastatus.DLI_P4_Level4_L);
    UL[DET_Alarm_UL1_Index] = ConvertTo2byte(bluecell_Currdatastatus.ULO_P1_Level1_H,bluecell_Currdatastatus.ULO_P1_Level1_L);
    UL[DET_Alarm_UL2_Index] = ConvertTo2byte(bluecell_Currdatastatus.ULO_P2_Level2_H,bluecell_Currdatastatus.ULO_P2_Level2_L);
    UL[DET_Alarm_UL3_Index] = ConvertTo2byte(bluecell_Currdatastatus.ULO_P3_Level3_H,bluecell_Currdatastatus.ULO_P3_Level3_L);
    UL[DET_Alarm_UL4_Index] = ConvertTo2byte(bluecell_Currdatastatus.ULO_P4_Level4_H,bluecell_Currdatastatus.ULO_P4_Level4_L);*/

    ret  = bluecell_Currdatastatus.DLI_P1_Level1_H << 8;
    ret += bluecell_Currdatastatus.DLI_P1_Level1_L;   
    ret *= 0.001;    
    Res_DL_dBm[DET_Alarm_DL1_Index] 
        = bluecell_Currdatastatus.DLI_Level1_H << 8 | bluecell_Currdatastatus.DLI_Level1_L;//(int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL1.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st)));
    ret  = bluecell_Currdatastatus.DLI_P2_Level2_H << 8;
    ret += bluecell_Currdatastatus.DLI_P2_Level2_L;   
    ret *= 0.001;  

    Res_DL_dBm[DET_Alarm_DL2_Index] 
        = bluecell_Currdatastatus.DLI_Level2_H << 8 | bluecell_Currdatastatus.DLI_Level2_L;//(int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL2.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st)));
    ret  = bluecell_Currdatastatus.DLI_P3_Level3_H << 8;
    ret += bluecell_Currdatastatus.DLI_P3_Level3_L;   
    ret *= 0.001;  

    Res_DL_dBm[DET_Alarm_DL3_Index] 
        = bluecell_Currdatastatus.DLI_Level3_H << 8 | bluecell_Currdatastatus.DLI_Level3_L;//(int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL3.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st)));


    ret  = bluecell_Currdatastatus.DLI_P4_Level4_H << 8;
    ret += bluecell_Currdatastatus.DLI_P4_Level4_L;   
    ret *= 0.001;  

    Res_DL_dBm[DET_Alarm_DL4_Index] 
        = bluecell_Currdatastatus.DLI_Level4_H << 8 | bluecell_Currdatastatus.DLI_Level4_L;//(int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL4.Table_Det5_dBm_H,sizeof(DET_TABLEDL_st)));



    ret  = bluecell_Currdatastatus.ULO_P1_Level1_H << 8;
    ret += bluecell_Currdatastatus.ULO_P1_Level1_L;   
    ret *= 0.001;

    Res_UL_dBm[DET_Alarm_UL1_Index] 
        = bluecell_Currdatastatus.ULO_Level1_H << 8 | bluecell_Currdatastatus.ULO_Level1_L; //(int8_t)Bluecell_TestPro(AutoControl_ADC_Compare(ret,&Det_UL1.Table_Det_15_dBm_H,Det_UL1.Table_Length * 2));

//    printf("LimitData_UL_High : %d \r\n",LimitData_UL_High);
//    printf("DET_High : %d \r\n",Res_UL_dBm[DET_Alarm_UL1_Index] );

    ret  = bluecell_Currdatastatus.ULO_P2_Level2_H << 8;
    ret += bluecell_Currdatastatus.ULO_P2_Level2_L;   
    ret *= 0.001;

    Res_UL_dBm[DET_Alarm_UL2_Index] 
        = bluecell_Currdatastatus.ULO_Level2_H << 8 | bluecell_Currdatastatus.ULO_Level2_L;//(int8_t)Bluecell_TestPro(AutoControl_ADC_Compare(ret,&Det_UL2.Table_Det_15_dBm_H,Det_UL2.Table_Length * 2));

    ret  = bluecell_Currdatastatus.ULO_P3_Level3_H << 8;
    ret += bluecell_Currdatastatus.ULO_P3_Level3_L;   
    ret *= 0.001;

    Res_UL_dBm[DET_Alarm_UL3_Index] 
        = bluecell_Currdatastatus.ULO_Level3_H << 8 | bluecell_Currdatastatus.ULO_Level3_L;//(int8_t)Bluecell_TestPro(AutoControl_ADC_Compare(ret,&Det_UL3.Table_Det_15_dBm_H,Det_UL3.Table_Length * 2));


    ret  = bluecell_Currdatastatus.ULO_P4_Level4_H << 8;
    ret += bluecell_Currdatastatus.ULO_P4_Level4_L;   
    ret *= 0.001;

    Res_UL_dBm[DET_Alarm_UL4_Index] 
        = bluecell_Currdatastatus.ULO_Level4_H << 8 | bluecell_Currdatastatus.ULO_Level4_L;//(int8_t)Bluecell_TestPro(AutoControl_ADC_Compare(ret,&Det_UL4.Table_Det_15_dBm_H,Det_UL4.Table_Length * 2));


//    printf("LimitData_DL_High : %d \r\n",LimitData_DL_High);    
//    printf("LimitData_DL_Low : %d \r\n",LimitData_DL_Low);   

    /*
        DL Level Alarm Check Part
    */
//    printf("======================================================\r\n");
    for(int i = 0 ; i < DET_Alarm_DL_Index_MAX; i++){
        if(LimitData_DL_High <= Res_DL_dBm[DET_Alarm_DL1_Index + i]){
            ADC_Alarm_DL_High_Set[DET_Alarm_DL1_Index + i] = true;
//            printf("ADC_Alarm_DL_High_Set %d ,Value : %d\r\n",DET_Alarm_UL1_Index + i,ADC_Alarm_DL_High_Set[DET_Alarm_UL1_Index + i]);            
        }
        else{
            if(LimitData_DL_High - 2 >= Res_DL_dBm[DET_Alarm_DL1_Index + i])
                ADC_Alarm_DL_High_Set[DET_Alarm_DL1_Index + i] = false;
//            printf("ADC_Alarm_DL_High_Set %d ,Value : %d\r\n",DET_Alarm_UL1_Index + i,ADC_Alarm_DL_High_Set[DET_Alarm_UL1_Index + i]);            
        }
#if 0 // PYJ.2020.06.22_BEGIN -- 
        printf("LimitData_DL_High: %d Res_DL_dBm [%d] : %d ,Value : %d\r\n",
        LimitData_DL_High,
        DET_Alarm_UL1_Index + i,
        Res_DL_dBm[DET_Alarm_DL1_Index + i],
        ADC_Alarm_DL_High_Set[DET_Alarm_DL1_Index + i]);            
#endif // PYJ.2020.06.22_END -- 

//        LimitData_DL_Low,
//        DET_Alarm_UL1_Index + i,
//        Res_DL_dBm[DET_Alarm_DL1_Index + i],
//        ADC_Alarm_DL_High_Set[LimitData_DL_High + i]);            
    }    
    
    /*
        DL Shutdown Alarm Check Part
    */
    uint8_t* DL_PathStatus = &bluecell_Currdatastatus.ATT_DL1_PATH ;
    uint8_t* DL_RetryCount = &bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 ;
#if 0 // PYJ.2020.06.23_BEGIN -- 
    for(int i = 0 ; i < DET_Alarm_DL_Shutdown_Index_MAX; i++){
        if(DL_PathStatus[DET_Alarm_DL1_Shutdown_Index + i] == true){
            ADC_Alarm_DL_Normal_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = true;
            if(DET_DL_Normal_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index + i] > MBIC_OFF_MAINTAIN_SEC){
                if(DL_RetryCount[DET_Alarm_DL1_Shutdown_Index + i] > 0){// Nomal Operate
                    DL_RetryCount[DET_Alarm_DL1_Shutdown_Index + i] = 0;
                }
            }
        }else{
            ADC_Alarm_DL_Normal_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = false;
        }
        
        if(LimitData_DL_Shutdown <= Res_DL_dBm[DET_Alarm_DL1_Shutdown_Index + i]){
            ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = true;
        }
        else{
             if(LimitData_DL_Shutdown - 2 >= Res_DL_dBm[DET_Alarm_DL1_Shutdown_Index + i])
                ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = false;
        }
    }
#else
    int16_t DL_Atten[DET_Alarm_DL_Shutdown_Index_MAX] = {0,};
    
    DL_Atten[DET_Alarm_DL1_Index] = bluecell_Currdatastatus.ATT_DL1_H << 8 | bluecell_Currdatastatus.ATT_DL1_L;
    DL_Atten[DET_Alarm_DL2_Index] = bluecell_Currdatastatus.ATT_DL2_H << 8 | bluecell_Currdatastatus.ATT_DL2_L;
    DL_Atten[DET_Alarm_DL3_Index] = bluecell_Currdatastatus.ATT_DL3_H << 8 | bluecell_Currdatastatus.ATT_DL3_L;
    DL_Atten[DET_Alarm_DL4_Index] = bluecell_Currdatastatus.ATT_DL4_H << 8 | bluecell_Currdatastatus.ATT_DL4_L;


    for(int i = 0 ; i < DET_Alarm_DL_Shutdown_Index_MAX; i++){
        if(DL_PathStatus[DET_Alarm_DL1_Shutdown_Index + i] == true
            && (LimitData_DL_Shutdown - 2 >= Res_DL_dBm[DET_Alarm_DL1_Shutdown_Index + i])
            && (MBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index + i] > 0)){
            ADC_Alarm_DL_Normal_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = true;
            if(DET_DL_Normal_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index + i] > MBIC_OFF_MAINTAIN_SEC){
                if(DL_RetryCount[DET_Alarm_DL1_Shutdown_Index + i] > 0){// Nomal Operate
                    DL_RetryCount[DET_Alarm_DL1_Shutdown_Index + i] = 0;
//                    printf("DL%d_PATH : %d Retry Count Initialize Start %d \r\n",i+1,DL_PathStatus[DET_Alarm_DL1_Shutdown_Index + i],bluecell_Currdatastatus.DLO_Shutdown_Retry_Count2);
                }
            }
        }else{
            ADC_Alarm_DL_Normal_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = false;
        }
    }
    for(int i = 0 ; i < DET_Alarm_DL_Shutdown_Index_MAX; i++){
        if(LimitData_DL_Shutdown <= Res_DL_dBm[DET_Alarm_DL1_Shutdown_Index + i]){
            if(DL_Atten[DET_Alarm_DL1_Index + i] / 10 <= -15)
                ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = true;
        }
        else{
//             if(LimitData_DL_Shutdown - 2 >= Res_DL_dBm[DET_Alarm_DL1_Shutdown_Index + i])
              if(MBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index + i] > 0)
                ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL1_Shutdown_Index + i] = false;
        }
    }

#endif // PYJ.2020.06.23_END -- 
//    printf("======================================================\r\n");
#if 0 // PYJ.2020.06.23_BEGIN -- 
    for(int i = 0 ; i < DET_Alarm_DL_Index_MAX; i++){
        if(LimitData_DL_Low >= Res_DL_dBm[DET_Alarm_DL1_Index + i]){
            ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index + i] = true;
        }
        else{
            if(LimitData_DL_Low + 2 <= Res_DL_dBm[DET_Alarm_DL1_Index + i])
                ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index + i] = false;

        }
#else

        for(int i = 0 ; i < DET_Alarm_DL_Index_MAX; i++){
            if(LimitData_DL_Low >= Res_DL_dBm[DET_Alarm_DL1_Index + i]){
                    ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index + i] = true;
            }
            else{
                if(LimitData_DL_Low + 2 <= Res_DL_dBm[DET_Alarm_DL1_Index + i])
                    ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index + i] = false;
        
            }


#endif // PYJ.2020.06.23_END -- 
        
#if 0 // PYJ.2020.06.22_BEGIN -- 
        printf("LimitData_DL_Low: %d Res_DL_dBm [%d] : %d ,Value : %d\r\n",
        LimitData_DL_Low,
        DET_Alarm_UL1_Index + i,
        Res_DL_dBm[DET_Alarm_DL1_Index + i],
        ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index + i]);            
#endif // PYJ.2020.06.22_END -- 
    }       
    
    /*
        UL Shutdown Alarm Check Part
    */
    
    uint8_t* UL_PathStatus = &bluecell_Currdatastatus.ATT_UL1_PATH ;
    uint8_t* UL_RetryCount = &bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 ;
        
//    printf("============================================================================\r\n");
    for(int i = 0 ; i < DET_Alarm_UL_Shutdown_Index_MAX; i++){
        if(UL_PathStatus[DET_Alarm_UL1_Shutdown_Index + i] == true){
            ADC_Alarm_UL_Normal_Shutdown_Set[DET_Alarm_UL1_Shutdown_Index + i] = true;
            if(DET_UL_Normal_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index + i] > MBIC_OFF_MAINTAIN_SEC){
                if(UL_RetryCount[DET_Alarm_UL1_Shutdown_Index + i] > 0){// Nomal Operate
                    UL_RetryCount[DET_Alarm_UL1_Shutdown_Index + i] = 0;
//                    printf("UL%d_PATH : %d Retry Count Initialize Start %d \r\n",i+1,UL_PathStatus[DET_Alarm_UL1_Shutdown_Index + i],bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2);
                }
            }
        }else{
            ADC_Alarm_UL_Normal_Shutdown_Set[DET_Alarm_UL1_Shutdown_Index + i] = false;
        }
        
        if(LimitData_UL_Shutdown <= Res_UL_dBm[DET_Alarm_UL1_Shutdown_Index + i]){
            ADC_Alarm_UL_Shutdown_Set[DET_Alarm_UL1_Shutdown_Index + i] = true;
//            printf("Shutdown threas hold : %d | Curr Meas : %d Shutdown UL %d | Alarm ON Count : %d\r\n",LimitData_UL_Shutdown,Res_UL_dBm[DET_Alarm_UL1_Shutdown_Index + i],i + 1,UL_RetryCount[DET_Alarm_UL1_Shutdown_Index + i] );
        }
        else{
            if(LimitData_UL_Shutdown - 2 >= Res_UL_dBm[DET_Alarm_UL1_Shutdown_Index + i])
                ADC_Alarm_UL_Shutdown_Set[DET_Alarm_UL1_Shutdown_Index + i] = false;
//            printf("Shutdown threas hold : %d | Curr Meas : %d Shutdown UL %d | Alarm OFF Count : %d\r\n",LimitData_UL_Shutdown,Res_UL_dBm[DET_Alarm_UL1_Shutdown_Index + i],i + 1,UL_RetryCount[DET_Alarm_UL1_Shutdown_Index + i] );
        }
    }
    /*
        UL Level Alarm Check Part
    */
    uint8_t* UL_Alarm_Timer_Status = &bluecell_Currdatastatus.ULO_Level_High_Alarm1;
    uint8_t* UL_ADC = &bluecell_Currdatastatus.ULO_P1_Level1_H;
    
//    LimitData_UL_High = -27;
//    printf("==========================================================\r\n");
    for(int i = 0 ; i < DET_Alarm_UL_Index_MAX; i++){
        if(LimitData_UL_High <= Res_UL_dBm[DET_Alarm_UL1_Index + i]){
            ADC_Alarm_UL_Set[DET_Alarm_UL1_Index + i] = true;
//            printf("ON _Limit : %d   UL%d : %d  ADC_Alarm_UL_Set %d ,Value : %d\r\n",LimitData_UL_High,i+1,Res_UL_dBm[DET_Alarm_UL1_Index + i],DET_Alarm_UL1_Index + i,ADC_Alarm_UL_Set[DET_Alarm_UL1_Index + i]);
        }
        else{
            if(LimitData_UL_High - 2 >= Res_UL_dBm[DET_Alarm_UL1_Index + i])
                ADC_Alarm_UL_Set[DET_Alarm_UL1_Index + i] = false;
//            printf("OFF _Limit : %d   UL%d : %d  ADC_Alarm_UL_Set %d ,Value : %d\r\n",LimitData_UL_High,i+1,Res_UL_dBm[DET_Alarm_UL1_Index + i],DET_Alarm_UL1_Index + i,ADC_Alarm_UL_Set[DET_Alarm_UL1_Index + i]);            
        }
#if 1 // PYJ.2020.06.22_BEGIN -- 
#if 0 // PYJ.2020.06.22_BEGIN --
                printf("ADC : %f LimitData_UL_High: %d Res_UL_dBm [%d] : %d ,Value : %d\r\n",
                (UL_ADC[i * 2] << 8 | UL_ADC[i * 2 + 1]) * 0.001,
                LimitData_UL_High,
                DET_Alarm_UL1_Index + i,
                Res_UL_dBm[DET_Alarm_UL1_Index + i],
                ADC_Alarm_UL_Set[DET_Alarm_UL1_Index + i]);            
#endif // PYJ.2020.06.22_END -- 
#endif // PYJ.2020.06.22_END -- 
        
    }    

}

void Temp_AttenCheck(int8_t Temp){
    int8_t CurrentTemp = 0;

    

}
void ADC_Check(void){
    //static uint8_t Cnt = 0;
   double ADC1DoubleRet[4];
   double ADC3DoubleRet[5];
   uint32_t ADC1_Average_value[ADC1_CNT];
   uint32_t ADC3_Average_value[ADC3_CNT];

   // double ret = 0;
    uint16_t MIN_ADC[ADC3_CNT] = {0,};
    uint16_t cnt[ADC3_CNT] = {0,};
    double temp;
//    if(AdcTimerCnt > 10){
    // 정렬할 배열, 요소 개수, 요소 크기, 비교 함수를 넣어줌



    if(adc3cnt >= ADC_AVERAGECNT){
//        printf("%f\r\n",ADC3value[4]*3.3/4095);
#if 1 // PYJ.2020.05.25_BEGIN -- 
      for(int i = 0; i < ADC3_CNT; i++){
                DascendigFunc(&ADC3valuearray[i][0],ADC_AVERAGECNT);
                MIN_ADC[i] = ADC3valuearray[i][0] - 12;
                for(int a = 0; a < ADC_AVERAGECNT; a++){
//                     printf("ADC3valuearray[%d][%d] : %d \r\n",i,a,ADC3valuearray[i][a]);
                    if(ADC3valuearray[i][a] < MIN_ADC[i]){
                        cnt[i] = a;
    //                    printf("cnt[i] %d \r\n",cnt[i]);
                        break;
                    }else{
                        cnt[i] = ADC_AVERAGECNT;
                    }
                }
                ADC3_Average_value[i] = SumFunc(&ADC3valuearray[i][0],cnt[i]);
    //            printf("ADC3_Average_value[%d] : %d / %f \r\n",i,ADC3_Average_value[i],ADC3_Average_value[i]/cnt[i] * Volt_Calc_val);
            }

            for(int i = 0; i < ADC3_CNT; i++){
                ADC3DoubleRet[i] = (((ADC3_Average_value[i] / cnt[i]) * 3.3 /4095) * 1000);
                ADC3Ret[i] = ADC3DoubleRet[i];
                ADC3_Average_value[i] = 0;
            }
            
            bluecell_Currdatastatus.DLI_P1_Level1_H 
                = ((ADC3Ret[1] & 0xFF00) >> 8);
            bluecell_Currdatastatus.DLI_P1_Level1_L 
                = ((ADC3Ret[1] & 0x00FF));
            

            bluecell_Currdatastatus.DLI_P2_Level2_H 
                = ((ADC3Ret[2] & 0xFF00) >> 8);
            bluecell_Currdatastatus.DLI_P2_Level2_L 
                = ((ADC3Ret[2] & 0x00FF) );

                
            bluecell_Currdatastatus.DLI_P3_Level3_H 
                = ((ADC3Ret[3] & 0xFF00) >> 8);
            bluecell_Currdatastatus.DLI_P3_Level3_L 
                = ((ADC3Ret[3] & 0x00FF) );

                
            bluecell_Currdatastatus.DLI_P4_Level4_H 
                = ((ADC3Ret[4] & 0xFF00) >> 8);
            bluecell_Currdatastatus.DLI_P4_Level4_L 
                = ((ADC3Ret[4] & 0x00FF) );

                
            bluecell_Currdatastatus.ULO_P4_Level4_H 
                = ((ADC3Ret[0] & 0xFF00) >> 8);
            bluecell_Currdatastatus.ULO_P4_Level4_L 
                = ((ADC3Ret[0] & 0x00FF) );
#endif // PYJ.2020.05.25_END -- 
#if 0 // PYJ.2020.04.26_BEGIN -- 
            double ret = 0;
            ret = (ADC3Ret[0]) * 0.001;
            printf("ADC3Ret[0] : %d        UL4 : %f\r\n",ADC3Ret[0],ret);
            ret = (ADC3Ret[1]) * 0.001;
            printf("ADC3Ret[1] : %d        DL1 : %f\r\n",ADC3Ret[1],ret);
            ret = (ADC3Ret[2]) * 0.001;
            printf("ADC3Ret[2] : %d        DL2 : %f\r\n",ADC3Ret[2],ret);
            ret = (ADC3Ret[3]) * 0.001;
            printf("ADC3Ret[3] : %d        DL3 : %f\r\n",ADC3Ret[3],ret);

            ret = (ADC3Ret[4]) * 0.001;
            printf("ADC3Ret[4] : %d        DL4 : %f\r\n",ADC3Ret[4],ret);
#endif // PYJ.2020.04.26_END -- 
        adc3cnt = 0;

    }

    if(adc1cnt >= ADC_AVERAGECNT){ 
        for(int i = 0; i < ADC1_CNT; i++){
            DascendigFunc(&ADC1valuearray[i][0],ADC_AVERAGECNT);
            MIN_ADC[i] = ADC1valuearray[i][0] - 12;
            for(int a = 0; a < ADC_AVERAGECNT; a++){
//                 printf("ADC1valuearray[%d][%d] : %d \r\n",i,a,ADC1valuearray[i][a]);
                if(ADC1valuearray[i][a] < MIN_ADC[i]){
                    cnt[i] = a;
//                    printf("cnt[i] %d \r\n",cnt[i]);
                    break;
                }else{
                    cnt[i] = ADC_AVERAGECNT;
                }
            }
            ADC1_Average_value[i] = SumFunc(&ADC1valuearray[i][0],cnt[i]);
//            printf("ADC1_Average_value[%d] : %d / %f \r\n",i,ADC1_Average_value[i],ADC1_Average_value[i]/cnt[i] * Volt_Calc_val);
        }

        for(int i = 0; i < 4; i++){
            ADC1DoubleRet[i] = (((ADC1_Average_value[i] / cnt[i]) * 3.3 /4095) * 1000);
            ADC1Ret[i] = ADC1DoubleRet[i];
//            ADC1Ret[i] = ADC1_Average_value[i] / cnt[i];
            ADC1_Average_value[i] = 0;
        }
       
        bluecell_Currdatastatus.ULO_P1_Level1_H 
            = ((ADC1Ret[0] & 0xFF00) >> 8);
        bluecell_Currdatastatus.ULO_P1_Level1_L 
            = ((ADC1Ret[0] & 0x00FF) );

            
        bluecell_Currdatastatus.ULO_P2_Level2_H 
            = ((ADC1Ret[1] & 0xFF00) >> 8);
        bluecell_Currdatastatus.ULO_P2_Level2_L 
            = ((ADC1Ret[1] & 0x00FF) );

            
        bluecell_Currdatastatus.ULO_P3_Level3_H 
            = ((ADC1Ret[2] & 0xFF00) >> 8);
        bluecell_Currdatastatus.ULO_P3_Level3_L 
            = ((ADC1Ret[2] & 0x00FF) );

//            printf("ADC1_Average_value[%d] : %d / %f \r\n",i,ADC1_Average_value[i],ADC1_Average_value[i]/cnt[i] * Volt_Calc_val);            
#if 0 // PYJ.2020.05.14_BEGIN -- 
        bluecell_Currdatastatus.DET_TEMP_H  
            =((ADC1Ret[3] & 0xFF00) >> 8);
        bluecell_Currdatastatus.DET_TEMP_L  
            =((ADC1Ret[3] & 0x00FF) );
#else    
/* *
온도 소수점 제거
*/
    temp = (ADC1Ret[3] * 0.001);
//    printf("DetEnd");
//    printf("temp %f \r\n",temp  );
    bluecell_Currdatastatus.DET_TEMP = (uint8_t)( (temp - 0.5) * 100);
//    printf("DET_TEMP %d \r\n",bluecell_Currdatastatus.DET_TEMP  );
    bluecell_Currdatastatus.DET_TEMP  += bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET;
//    printf("DET_TEMP + bluecell_Currdatastatus.bluecell_User_TEMP_OFFSET  :  %d \r\n",bluecell_Currdatastatus.DET_TEMP  );
//     temp= ((( - 0.5 ) * 100)  + );

#endif // PYJ.2020.05.14_END -- 


//        ret = ((ADC1Ret[0]) * Volt_Calc_val);
      //  printf("UL1 : %f\r\n",ret);

//        ret = ((ADC1Ret[1]) * Volt_Calc_val);
//        printf("UL2 : %f\r\n",ret);

//        ret = ((ADC1Ret[2]) * Volt_Calc_val);
////        printf("UL3 : %f\r\n",ret);
//        bluecell_Currdatastatus.DET_TEMP_H  
//            =((ADC1Ret[3] & 0xFF00) >> 8);
//        bluecell_Currdatastatus.DET_TEMP_L  
//            =((ADC1Ret[3] & 0x00FF) );
//    printf("bluecell_Currdatastatus.DET_TEMP  %d \r\n",bluecell_Currdatastatus.DET_TEMP  );
//    printf(" ADC1Ret[3]    %x \r\n",  ADC1Ret[3]   );


//        ret = ((ADC1Ret[3]) * Volt_Calc_val);
//        printf("Temp : %f\r\n",ret);
        adc1cnt = 0;

    }
    /*
        After ADC calculation is completed, start Det Alarm check.
    */
    DET_LevelAlarmCheck();/*DL UL Alarm Check*/
}



#if 1 // PYJ.2020.05.12_BEGIN -- 

/*One Point  round*/
double Bluecell_round( double value )
{
    unsigned short data = value * 1000;
    unsigned short temp = 0;
    double ret = 0;
//    printf("1: %d\r\n",data);
    temp = data % 10;
    if(temp >= 5){
        data = data - temp + 10;
    }else{
        data = data - temp;
    }
//    printf("2: %d\r\n",data);
    temp = (data % 100) / 10;
//    printf("3: %d\r\n",temp);
    if(temp >= 5){
        data = data - (temp * 10)+ 100;
    }else{
        data = data - (temp * 10);
    }
//     printf("4: %d\r\n",data);
     ret = data;
       // printf("ret : %f\r\n",ret / 1000);

     return ret / 1000;
}
/*One Point  round*/
double Bluecell_TestPro(double value )
{
    int16_t temp = (value * 10);
    double temp_float = 0;
    bool minus_set = 0;
//    printf("0. temp  : %d\r\n",temp);

    temp = temp % 10;
//    printf("1. temp  : %d\r\n",temp);
    if((int16_t)temp == 0)
        return value;
    if(temp < 0){
        temp *= -1;//Convert Minus To plus
//        printf("MInus set : 2. temp  : %d\r\n",temp);        
        minus_set = true; 
    }
    temp_float = temp * 0.1;
//    printf("3. temp_float: %f    temp  : %f\r\n",temp_float,temp);            
//    printf("4. value : %f temp : %d   temp_float : %f \r\n",value,temp,temp_float);        
    
    if(temp >= 5){

        if(minus_set == true){
            value -= 1;
            value += temp_float;
        }else{
            value += 1;
            value -= temp_float;
        }
            
//    printf("temp_float : %f \r\n",temp_float);
    }
    else{
#if 1 // PYJ.2020.05.25_BEGIN -- 
         if(minus_set == true){
             value += temp_float;
         }
         else{
            value -= temp_float;
         }
#else         
        value -= temp_float;

#endif // PYJ.2020.05.25_END -- 
//        printf("temp_float : %f \r\n",temp_float);
    }
    
    
//    printf("temp  : %f \r\n",value);
     return (value);
}


#endif // PYJ.2020.05.12_END -- 

#if 0 // PYJ.2020.05.12_BEGIN -- 
double AutoControl_ADC_Compare(double CurrentAdc,uint8_t* CompareAdc,uint8_t size){
    double ret = 0xFF,CurrRet = 0,TableAdc,TableAdcret; 
    uint8_t LastIndex = 0;
    double Max_ADC = 0,Min_ADC = 0;
    double step = 0;
    double dot = 0,tempret = 0xFF;
    for(int i =0; i < size / 2; i++){
        TableAdc = CompareAdc[i * 2] << 8;
        TableAdc += CompareAdc[i * 2 + 1];
        TableAdc /= 1000;
//        printf("TableAdc[%d] : %f \r\n",i,TableAdc);
        CurrRet = TableAdc - CurrentAdc;
        
        if(CurrRet < 0){ // plus 공식
            CurrRet = (CurrRet * -2) + CurrRet;
        }
        if(ret > CurrRet){
            ret = CurrRet;
            TableAdcret = TableAdc;
            LastIndex = i;
        }
    }
    
    /*MIN*/
    TableAdc = CompareAdc[LastIndex * 2] << 8;
    TableAdc += CompareAdc[LastIndex * 2 + 1];
    TableAdc /= 1000;
    Min_ADC = TableAdc;
    /*MAX*/
    TableAdc = CompareAdc[LastIndex * 2 - 2 ] << 8;
    TableAdc += CompareAdc[LastIndex * 2 - 1];
    TableAdc /= 1000;
    Max_ADC = TableAdc;

    
    step = ((Max_ADC - Min_ADC) / 10);
    
//    Min_ADC = Bluecell_round(Min_ADC);
//    CurrentAdc = Bluecell_round(CurrentAdc);
//    printf("1:STEP : %f , %f  > %f > %f \r\n",step,Max_ADC,CurrentAdc,Min_ADC);
    for(double d = 0; d < 1; d += 0.1){
        CurrRet = CurrentAdc - Min_ADC;
        if(tempret >= CurrRet & CurrRet > 0){
//            printf("(%f >= %f)\r\n",tempret,CurrRet);
            tempret = CurrRet;
//            printf("2:STEP : %f , %f  > %f > %f \r\n",step,Max_ADC,CurrentAdc,Min_ADC);
            Min_ADC += step;
            dot = d;
        }
   }    
//    printf("dot : %f \r\n",dot);
    dot = AutoControl_Save[LastIndex] - dot;
    
//    printf("AutoControl_Save[LastIndex]:%d + dot:%f : %f \r\n",AutoControl_Save[LastIndex] + dot);
//    printf(" %f  > %f > %f \r\n",Max_ADC,CurrentAdc,Min_ADC);
    return dot;
}
#else
#if 0 // PYJ.2020.05.15_BEGIN -- 
int8_t AutoControl_ADC_Compare(double CurrentAdc,uint8_t* CompareAdc,uint8_t size){
    double ret = 0xFF,CurrRet = 0,TableAdc; 
    uint8_t LastIndex = 0;
    for(int i =0; i < size / 2; i++){
        TableAdc = CompareAdc[i * 2] << 8;
        TableAdc += CompareAdc[i * 2 + 1];
        TableAdc /= 1000;
//        printf("TableAdc[%d] : %f \r\n",i,TableAdc);
        CurrRet = TableAdc - CurrentAdc;

        if(CurrRet < 0){
            CurrRet = (CurrRet * -2) + CurrRet;
        }
        
        if(ret > CurrRet){
            ret = CurrRet;
            LastIndex = i;
        }
        
    }
    return AutoControl_Save[LastIndex];
}
#else
double AutoControl_ADC_Compare(double CurrentAdc,uint8_t* CompareAdc,uint8_t size,int8_t* RefTable_Data){
    double ret = 0xFF,CurrRet = 0,TableAdc,NextTableAdc; 
    double Vitual_array[10] = {0,};
    double step = 0;
    uint8_t LastIndex = 0;
    uint8_t dot = 0;
    double Lastdata = 0;    
//   printf("size: %d \r\n",size);
    for(int i =0; i < size / 2; i++){
        TableAdc = CompareAdc[i * 2] << 8;
        TableAdc += CompareAdc[i * 2 + 1];
        if(TableAdc == 0)
            continue;
        TableAdc /= 1000;

        NextTableAdc = CompareAdc[i * 2 + 2] << 8;
        NextTableAdc += CompareAdc[i * 2 + 3];
        NextTableAdc /= 1000;        
        Lastdata = TableAdc;
//        printf("TableAdc[%d] : %f \r\n",i,TableAdc);
//        CurrRet = TableAdc - CurrentAdc;

        step = (TableAdc - NextTableAdc) / 10;
        for(int a = 0; a < 10; a++){
            Vitual_array[a] = TableAdc - (step * a);
            if(Vitual_array[a] >= CurrentAdc){
                CurrRet = (Vitual_array[a]) - (CurrentAdc);
            }else{
                CurrRet = (CurrentAdc) - (Vitual_array[a]);
            }
//            printf("Vitual_array[%d] : %f \r\n",a,Vitual_array[a]);
//            if(CurrRet < 0){
//                CurrRet = (CurrRet * -2) + CurrRet;
//            }
            
            if(ret > CurrRet){
                ret = CurrRet;
                LastIndex = i;
                dot = a;
            }
        }
        
    }
    TableAdc = CompareAdc[0] << 8;
    TableAdc += CompareAdc[1];
    TableAdc /= 1000;    

    if(Lastdata > CurrentAdc)
        return RefTable_Data[(size / 2) - 1]; 
    
    if(CurrentAdc < TableAdc){
        for(int i = 0; i < sizeof(ALC_dBm_t); i++)
//            printf("ref Tabe: %d \r\n",RefTable_Data[i]);
//        printf("LastIndex : %d  / dot : %d  TableAdc : %f \r\n",LastIndex,dot,(RefTable_Data[LastIndex] - (dot * 0.1)));
        return (RefTable_Data[LastIndex] - (dot * 0.1));
    }
    else{
//        printf("CurrentAdc : %f     TableAdc : %f \r\n",CurrentAdc,TableAdc);
        return (RefTable_Data[0]);
    }
}

double AGC_AutoControl_ADC_Compare(double CurrentAdc,uint8_t* CompareAdc,uint8_t size,int8_t* RefTable_Data){
    double ret = 3.3,CurrRet = 0,TableAdc,NextTableAdc; 
    double Vitual_array[10] = {0,};
    double step = 0;
    uint8_t LastIndex = 0;
    uint8_t dot = 0;
    double Lastdata = 0;
    double Compare_Data = 0;
    double first_data = CompareAdc[0] << 8 | CompareAdc[1];
    for(int i =0; i < size / 2; i++){
        TableAdc = CompareAdc[i * 2] << 8;
        TableAdc += CompareAdc[i * 2 + 1];
        TableAdc /= 1000;
        Lastdata = TableAdc;
        
        NextTableAdc = CompareAdc[i * 2 + 2] << 8;
        NextTableAdc += CompareAdc[i * 2 + 3];
        NextTableAdc /= 1000;        
//        printf("TableAdc[%d] : %f \r\n",i,TableAdc);
//        CurrRet = TableAdc - CurrentAdc;
        if(TableAdc >= NextTableAdc){
            step = (TableAdc - NextTableAdc)/10;
        }else{
            step = (NextTableAdc - TableAdc) / 10;
        }
        for(int a = 0; a < 10; a++){
            Vitual_array[a] = TableAdc - (step * a);
            if(Vitual_array[a] >= CurrentAdc){
                CurrRet = Vitual_array[a] - CurrentAdc;
            }else{
                CurrRet = CurrentAdc - Vitual_array[a];
            }
//            CurrRet = (Vitual_array[a]) - (CurrentAdc);
//            printf("Vitual_array[%d] : %f \r\n",a,Vitual_array[a]);
//            Compare_Data *= 1000;

//            if(CurrRet < 0){
//                CurrRet = (CurrRet * -2) + CurrRet;
//            }
            
            if(ret > CurrRet){
                ret = CurrRet;
                LastIndex = i;
//                if(LastIndex == 30 && CurrentAdc != 0 ){
//                    printf("TableAdc[%d] : %f  step : %f  x a : %d\r\n",i,TableAdc,step,a);
//                    for(int k = 0; k < 10; k++)
//                        printf("Vitual_array[%d] : %f \r\n",k,Vitual_array[k]);
//                }
//                printf("ret : %f CurrRet  : %f  CurrentAdc : %f  %d.Vitual_array[a] : %f dot : %d\r\n",ret,CurrRet,CurrentAdc,i,Vitual_array[a],dot);
                dot = a;
            }
        }
        
    }
    if(Lastdata > CurrentAdc){
//        printf("RefTable_Data[(size / 2) - 1] : %d \r\n",RefTable_Data[(size / 2) - 1]);
        return RefTable_Data[(size / 2) - 1];
    }
//        printf("CurrentAdc : %f     TableAdc : %f \r\n",CurrentAdc,TableAdc);
//        for(int a = 0; a < sizeof(AGC_dBm_t); a++)
//            printf("AutoControl_Save[%d] : %d \r\n",a,AutoControl_Save[a]);
    if(first_data < CurrentAdc){
//        printf("(RefTable_Data[0]) : %d \r\n",(RefTable_Data[0]));
        return (RefTable_Data[0]);
    }else{
//        printf("Nomal _Table Data %f",(RefTable_Data[LastIndex] - (dot * 0.1)));
//        printf("LastIndex : %d  / dot : %d  TableAdc : %f \r\n",LastIndex,dot,(RefTable_Data[LastIndex] - (dot * 0.1)));
        return (RefTable_Data[LastIndex] - (dot * 0.1));
    }
}

#endif // PYJ.2020.05.15_END -- 

#endif // PYJ.2020.05.12_END -- 
#if 0 // PYJ.2020.05.21_BEGIN -- 
int32_t MinusConvert(uint8_t Temp_h, int32_t Value){
    int32_t ret;
    if((((bluecell_Currdatastatus.ATT_ALC1_MAX_H << 8) & 0xFF00) & 0xF000) == 0xF000){
        
        Value = 0x0000FFFF - (Value & 0x0000FFFF);
        Value += 0x01;
        Value *= -1;
    }
    Value /= 100;
    ret = Value;
     
    return ret;
}
#endif // PYJ.2020.05.21_END -- 


uint8_t ALC_AlarmSet[ALC_Alarm_UL_Index_MAX] = {0,};
int16_t ALC_Calc(uint8_t num,double CurrAtten ,int8_t threshold,double CurrDet){
    double ret = 0;
//    CurrAtten *= -1;
    if(CurrDet == threshold){
        return 0;
    }
//    ret = CurrDet - threshold;
#if 0 // PYJ.2020.05.25_BEGIN -- 
    if(CurrAtten >= 20){
        if(CurrDet - threshold < 0){
            if(CurrAtten + (CurrDet - threshold) > 0){
//                ret = CurrAtten + (CurrDet - threshold);
                printf("5. %f : %f  %d\r\n",ret,CurrDet,threshold);
            }
            else{
                ret = CurrAtten * -1;
                printf("6. %f : %f  %d\r\n",ret,CurrDet,threshold);
            }
        }else{
            ALC_AlarmSet[num] = true;
        }
        printf("4. %f : %f  %d\r\n",ret,CurrDet,threshold);
        return ret;
    }
#endif // PYJ.2020.05.25_END -- 
    
#if 0 // PYJ.2020.06.20_BEGIN -- 
    if(CurrDet < threshold){
          ret =  CurrDet - threshold;
          printf("1. %f : %f - %d\r\n",ret,CurrDet,threshold);
//          if(((ret * 10) % 10 ) != 0)
              ret = Bluecell_TestPro(ret);
        printf("ret = %f \r\n",ret);

    }
    else if(CurrDet > threshold){
        ret = CurrDet - threshold;
        printf("2. %f : %f  %d\r\n",ret,CurrDet,threshold);    
//        if(((ret * 10) % 10 ) != 0)
           ret = Bluecell_TestPro(ret);
        
    }
   
    printf("Result : ret = %f \r\n",ret);
#if 0 // PYJ.2020.06.20_BEGIN -- 
    if(CurrAtten + ret >= 0){
        ret = CurrAtten * -1;
        printf("3. ret0 : %f \r\n",ret);
    }else{
        ALC_AlarmSet[num] = false;  
    }
#endif // PYJ.2020.06.20_END -- 
    ret *= -1;
    if(CurrAtten < 0){
        if(CurrAtten < ret){
            ret += CurrAtten;
        }
    }
#endif // PYJ.2020.06.20_END -- 
    if(threshold < CurrDet){
        ret = CurrDet - threshold ;
//        printf("1. %f : %f - %d\r\n",ret,CurrDet,threshold);

        
//        printf("2. %f : %f - %d\r\n",ret,CurrDet,threshold);
        ret = Bluecell_TestPro(ret +( CurrAtten * -1));
        ret *= -1;  
//        if(CurrAtten <= -20)
//            ALC_AlarmSet[num] = true;
    }
    else if(threshold -2 > CurrDet){
        if(CurrAtten < 0){
            ret =  (threshold - 2) - CurrDet ;// -27 ///// - 29
//            printf("%f  =  %d - %f\r\n",ret,(threshold - 2),CurrDet);
//            printf("CurrAtten : %f\r\n",CurrAtten);
            ret = Bluecell_TestPro(ret);
//            printf("3.ret : %f\r\n",ret);
            ret += CurrAtten;
//             ret *= -1;     
             
//            printf("4.ret : %f\r\n",ret);
//            ALC_AlarmSet[num] = false; 

        }
         
    }


    
    if(ret < -20){
//        printf("5. ret1 : %f \r\n",ret);
        ret = -20;
        
    }else{
        if(ret > 0){
//            printf("6. ret1 : %f \r\n",ret);
            ret = 0;
        }
       
    }
//    printf("ret7 : %f \r\n",ret);
    return ret * 10;
}
#define UL_DL_DATASEND_MULTIPLE 10
void ALC_Alarm_TimerSet(uint8_t num,int16_t threshold,int16_t CurrDet,int16_t Atten){
    int16_t ret =0;
    int16_t CurrAtten = 0;
    CurrAtten = Atten * 0.1;
        if(threshold < CurrDet){
            ret = CurrDet - threshold ;
            ret = Bluecell_TestPro(ret +( CurrAtten / 10 * -1));
            ret *= -1;  
            if(CurrAtten <= -20)
                ALC_AlarmSet[num] = true;
        }
        else if(threshold -2 > CurrDet){
            if(CurrAtten < 0){
                ret =  (threshold - 2) - CurrDet ;// -27 ///// - 29
                ret = Bluecell_TestPro(ret);
                ret += CurrAtten;
                ALC_AlarmSet[num] = false; 
            }
        }
}
void ALC_Function(){ //DL 
    //double Volt_Calc_val = 3.3 / 4095;
    //double tempret = 0;
    double ret = 0;
   // double CurrnALCValue = 0;
    double CurrentATTENVALUE = 0;
   uint8_t tempcurratten = 0;
  //  int32_t CurrnALC_MAX_SettingValue = 0;
  //  int32_t CurrnALC_MIN_SettingValue = 0;
  //  int32_t ALC_Result = 0;
    int16_t Limitdata = 0;
    static int16_t PrevLimitdata[ALC_Alarm_UL_Index_MAX] = {0,};
    double ResdBm = 0;
    int16_t ResultData = 0;
    int16_t PrevResultData[ALC_Alarm_UL_Index_MAX] = {0,};    
    int16_t IwillGiveAtten = 0;
    static int8_t* ALC_Table;
    int16_t tmpcalc = 0;
    int16_t UL_Atten[ALC_Alarm_UL_Index_MAX] = {0,};
    int16_t Main_Atten[ALC_Alarm_UL_Index_MAX] = {0,};
    static int16_t PrevIwillGiveAtten[ALC_Alarm_UL_Index_MAX] ={0,};
//    uint8_t tempadc[256];

    
//    Bluecell_StructCpy(&AutoControl_Save[0],&ALC_Table_ref.DET_UL_0,sizeof(ALC_dBm_t));
//    Bluecell_StructCpy(&tempadc[0],&Det_UL1.Table_Det_15_dBm_H,sizeof(DET_TABLEUL_st));
    if(ALCTimerCnt > 500){
        if(bluecell_Currdatastatus.ULO_ALC_ON_OFF == true){
            UL_Atten[ALC_Alarm_UL1_Index] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L;
            UL_Atten[ALC_Alarm_UL2_Index] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L;
            UL_Atten[ALC_Alarm_UL3_Index] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L;
            UL_Atten[ALC_Alarm_UL4_Index] = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L;
            Main_Atten[ALC_Alarm_UL1_Index] = bluecell_Currdatastatus.ATT_UL1_H << 8 | bluecell_Currdatastatus.ATT_UL1_L;
            Main_Atten[ALC_Alarm_UL2_Index] = bluecell_Currdatastatus.ATT_UL2_H << 8 | bluecell_Currdatastatus.ATT_UL2_L;
            Main_Atten[ALC_Alarm_UL3_Index] = bluecell_Currdatastatus.ATT_UL3_H << 8 | bluecell_Currdatastatus.ATT_UL3_L;
            Main_Atten[ALC_Alarm_UL4_Index] = bluecell_Currdatastatus.ATT_UL4_H << 8 | bluecell_Currdatastatus.ATT_UL4_L;
            
#if 1 // PYJ.2020.06.20_BEGIN -- 
            /***************************************************************************************************************/
            /*                                             ALC 1                                                           */
            /***************************************************************************************************************/            
            //ret 현재 Voltage 값 출력
            //ret 에 따른 현재 DBM 구현
            ALC_Table = &ALC_Table_ref[ALC_Table_UL1_Ref_Index].DET_UL_0;
            ret  = bluecell_Currdatastatus.ULO_P1_Level1_H << 8;
            ret += bluecell_Currdatastatus.ULO_P1_Level1_L;   
            ret *= 0.001;

            
            Limitdata = (( bluecell_Currdatastatus.ULO_ALC_Threshold_H << 8) & 0xFF00) ;
            Limitdata +=  bluecell_Currdatastatus.ULO_ALC_Threshold_L ;
            
            Limitdata /= 10;
            
            
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL1.Table_Det_15_dBm_H,Det_UL1.Table_Length * 2,ALC_Table);
            ResultData = ResdBm;
#if 1 // PYJ.2020.06.20_BEGIN -- 
            printf("=============ALC ON =====START================\r\n");
            printf("Current UL ADC VALUE : %f \r\n",ret);
            printf("ResdBm : UL1 : %f \r\n",ResdBm);
//            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level1_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level1_H);
//            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level1_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level1_L);            

#endif // PYJ.2020.06.20_END -- 

            
//            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.ATT_UL1_H,bluecell_Currdatastatus.ATT_UL1_L);

            bluecell_Currdatastatus.ULO_Level1_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level1_L = ((int16_t)ResultData & 0x00FF);


            
//            tempret = (((bluecell_Currdatastatus.ATT_UL1_H << 8) & 0xFF00) | (bluecell_Currdatastatus.ATT_UL1_L & 0x00FF)) / 10;
          
            //if(ResdBm < Limitdata && CurrentATTENVALUE != 0){
                    
    //            printf("db %f \r\n",ResdBm);
               
//                            printf("==================START================\r\n");
                //            printf("Current UL ADC VALUE : %f \r\n",ret);
    //                        printf("ResdBm : UL1 : %f \r\n",ResdBm);
     

                //CurrnALC_MIN_SettingValue = MinusConvert(bluecell_Currdatastatus.ULO_P1_Level1_H,CurrnALC_MIN_SettingValue);
    //            Limitdata = (( bluecell_Currdatastatus.ATT_UL1_H << 8) & 0xFF00) ;
    //            Limitdata +=  bluecell_Currdatastatus.ATT_UL1_L ;
    //
    //            Limitdata /= 10;

//                printf("CurrentATTENVALUE : %f\r\n",CurrentATTENVALUE);  
//                CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L);

////                CurrentATTENVALUE = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H;
                bluecell_Currdatastatus.ULO_Level1_H = ((int16_t)ResultData & 0xFF00) >> 8;
                bluecell_Currdatastatus.ULO_Level1_L = ((int16_t)ResultData & 0x00FF);       
                printf("ORIGIN ResdBm : %f  %d\r\n",ResdBm,Main_Atten[ALC_Alarm_UL1_Index]);
                printf("After ResdBm : %f \r\n",ResdBm);
                Limitdata += (Main_Atten[ALC_Alarm_UL1_Index] / 10);
                ALC_Alarm_TimerSet(ALC_Alarm_UL1_Index,Limitdata,ResdBm,UL_Atten[ALC_Alarm_UL1_Index]);
                IwillGiveAtten = ALC_Calc(ALC_Alarm_UL1_Index,CurrentATTENVALUE,Limitdata,ResdBm);
                //IwillGiveAtten = ALC_Calc(ALC_Alarm_UL1_Index,CurrentATTENVALUE,Limitdata,ResdBm);                
                
//                ResdBm = ResdBm + (Main_Atten[ALC_Alarm_UL1_Index] / 10);
                IwillGiveAtten += UL_Atten[ALC_Alarm_UL1_Index];
                if(ResdBm <= (Limitdata - 3)){
                    if(IwillGiveAtten <= -10){
                        IwillGiveAtten += 10;
                    }
                }
//
//                if(UL_Atten[ALC_Alarm_UL1_Index] + IwillGiveAtten <= -200){
//
//                    tmpcalc = (-200 + (Main_Atten[ALC_Alarm_UL1_Index] * -1));
//                    IwillGiveAtten =tmpcalc;
//                }
//                    
                printf(" UL_Atten[ALC_Alarm_UL1_Index] : %d I WILL GIVE ATTEN  : %d \r\n", UL_Atten[ALC_Alarm_UL1_Index],IwillGiveAtten);  
                printf("ADC : %f CURR ATT : %f  Threas : %d : I WILL GIVE ATTEN  : %d \r\n",ret,ResdBm , Limitdata,IwillGiveAtten);  
                printf("==================END================\r\n");                

//                printf("CurrentATTENVALUE : %f \r\n",CurrentATTENVALUE);
//                if(CurrentATTENVALUE < 20 && IwillGiveAtten != 0){
//                    IwillGiveAtten = CurrentATTENVALUE + IwillGiveAtten ;
//                }
//                printf("ADC : %f Current : %f  Threas : %d : I WILL GIVE ATTEN  : %d \r\n",ret,ResdBm , Limitdata,IwillGiveAtten);  
//                printf("CurrentATTENVALUE : %f I WILL GIVE ATTEN  : %d \r\n",CurrentATTENVALUE,IwillGiveAtten);  
//                printf("CurrentATTENVALUE : %f I WILL GIVE ATTEN  : %d \r\n",CurrentATTENVALUE,IwillGiveAtten);  
//                if(IwillGiveAtten < 0)
//                printf("CurrentATTENVALUE %f \r\n",CurrentATTENVALUE);
/*                if(ALC_Level_Save[ALC_Alarm_UL1_Index] != ResultData
                    || Limitdata != PrevLimitdata[ALC_Alarm_UL1_Index])*/
                        if( (IwillGiveAtten > 0) || (CurrentATTENVALUE >= (IwillGiveAtten))){
                            bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H = (((uint16_t)((IwillGiveAtten)) & 0xFF00) >> 8);
                            bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L = ((uint16_t)(IwillGiveAtten)) & 0x00FF;
                            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                        }
//                CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_L);             

//                 printf("CurrentATTENVALUE : %f\r\n",CurrentATTENVALUE );  
//                 printf("==================END================\r\n");


#endif // PYJ.2020.05.23_END -- 
//                 HAL_Delay(1000);
            //}
#if 1 // PYJ.2020.05.22_BEGIN -- 
             /***************************************************************************************************************/
             /*                                             ALC 2                                                           */
             /***************************************************************************************************************/  

            //ret 현재 Voltage 값 출력
            //ret 에 따른 현재 DBM 구현
            ALC_Table = &ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_0;
            ret  = bluecell_Currdatastatus.ULO_P2_Level2_H << 8;
            ret += bluecell_Currdatastatus.ULO_P2_Level2_L;   
            ret *= 0.001;
            
            
            Limitdata = (( bluecell_Currdatastatus.ULO_ALC_Threshold_H << 8) & 0xFF00) ;
            Limitdata +=  bluecell_Currdatastatus.ULO_ALC_Threshold_L ;
            Limitdata /= 10;
            
//             printf("Det_UL2.Table_Length: %d \r\n",Det_UL2.Table_Length);
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL2.Table_Det_15_dBm_H,Det_UL2.Table_Length* 2,ALC_Table);
             ResultData = ResdBm;
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L);
            bluecell_Currdatastatus.ULO_Level2_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level2_L = ((int16_t)ResultData & 0x00FF); 
//            CurrentATTENVALUE = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H;
            IwillGiveAtten = ALC_Calc(ALC_Alarm_UL2_Index,CurrentATTENVALUE,Limitdata,ResdBm);
//            printf("CurrentATTENVALUE : %f I WILL GIVE ATTEN  : %d \r\n",CurrentATTENVALUE,IwillGiveAtten);  
//            printf("ADC : %f Current : %f  Threas : %d : I WILL GIVE ATTEN  : %d \r\n",ret,ResdBm , Limitdata,IwillGiveAtten);  
            
            if( (IwillGiveAtten > 0) || (CurrentATTENVALUE >= (IwillGiveAtten))){
                tempcurratten = CurrentATTENVALUE * 10;
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H = (((uint16_t)((tempcurratten + IwillGiveAtten)) & 0xFF00) >> 8);
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L = ((uint16_t)(tempcurratten + IwillGiveAtten)) & 0x00FF;
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
            }
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L);             
#if 0 // PYJ.2020.06.20_BEGIN -- 
            printf("==================START================\r\n");
            printf("Current UL ADC VALUE : %f \r\n",ret);
            printf("ResdBm : UL2 : %f \r\n",ResdBm);
            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level2_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level2_H);
            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level2_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level2_L);            
            printf("==================END================\r\n");
            printf("bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H : %x \r\n",bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H);
            printf("bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L : %x \r\n",bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L);            
#endif // PYJ.2020.06.20_END --
#if 1 // PYJ.2020.06.20_BEGIN -- 
#else
            printf("bluecell_Currdatastatus.ATT_UL2_H : %x \r\n",bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H);
            printf("bluecell_Currdatastatus.ATT_UL2_L : %x \r\n",bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L);            
#endif // PYJ.2020.06.20_END -- 


                
            /***************************************************************************************************************/
            /*                                             ALC 3                                                           */
            /***************************************************************************************************************/              
            //ret 현재 Voltage 값 출력
            //ret 에 따른 현재 DBM 구현
            ALC_Table = &ALC_Table_ref[ALC_Table_UL3_Ref_Index].DET_UL_0;
            ret  = bluecell_Currdatastatus.ULO_P3_Level3_H << 8;
            ret += bluecell_Currdatastatus.ULO_P3_Level3_L;   
            ret *= 0.001;
            
            
            Limitdata = (( bluecell_Currdatastatus.ULO_ALC_Threshold_H << 8) & 0xFF00) ;
            Limitdata +=  bluecell_Currdatastatus.ULO_ALC_Threshold_L ;
            Limitdata /= 10;
//            bluecell_Currdatastatus.ULO_Level3_H = ((int16_t)ResdBm & 0xFF00) >> 8;
//            bluecell_Currdatastatus.ULO_Level3_L = ((int16_t)ResdBm & 0x00FF);         
//                              printf("==================START================\r\n");          
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL3.Table_Det_15_dBm_H,Det_UL3.Table_Length* 2,ALC_Table);
            ResultData = ResdBm;

            
            
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L);
            bluecell_Currdatastatus.ULO_Level3_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level3_L = ((int16_t)ResultData & 0x00FF);     
//            CurrentATTENVALUE = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H;
            IwillGiveAtten = ALC_Calc(ALC_Alarm_UL3_Index,CurrentATTENVALUE,Limitdata,ResdBm);
//                printf("ADC : %f Current : %f  Threas : %d : I WILL GIVE ATTEN  : %d \r\n",ret,ResdBm , Limitdata,IwillGiveAtten);              
            if( (IwillGiveAtten > 0) || (CurrentATTENVALUE >= (IwillGiveAtten * -1))){
                tempcurratten = CurrentATTENVALUE * 10;
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H = (((tempcurratten + IwillGiveAtten) & 0xFF00) >> 8);
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L = (tempcurratten + IwillGiveAtten) & 0x00FF;
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
            }
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten3_L);
//                 printf("CurrentATTENVALUE : %f\r\n",CurrentATTENVALUE );              
//                             printf("==================END================\r\n");

            /***************************************************************************************************************/
            /*                                             ALC 4                                                           */
            /***************************************************************************************************************/  

            //ret 현재 Voltage 값 출력
            //ret 에 따른 현재 DBM 구현
            ALC_Table = &ALC_Table_ref[ALC_Table_UL4_Ref_Index].DET_UL_0;
            ret  = bluecell_Currdatastatus.ULO_P4_Level4_H << 8;
            ret += bluecell_Currdatastatus.ULO_P4_Level4_L;   
            ret *= 0.001;
            
            
            Limitdata = (( bluecell_Currdatastatus.ULO_ALC_Threshold_H << 8) & 0xFF00) ;
            Limitdata +=  bluecell_Currdatastatus.ULO_ALC_Threshold_L ;
            Limitdata /= 10;
            
            
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL4.Table_Det_15_dBm_H,Det_UL4.Table_Length* 2,ALC_Table);
            ResultData = ResdBm;
#if 0 // PYJ.2020.06.20_BEGIN -- 
            printf("==================START================\r\n");
            printf("Current UL ADC VALUE : %f \r\n",ret);
            printf("ResdBm : UL4 : %f \r\n",ResdBm);
            printf("ResultData  : %x /////  bluecell_Currdatastatus.ULO_Level4_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level4_H);
            printf("ResultData  : %x /////  bluecell_Currdatastatus.ULO_Level4_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level4_L);            
            printf("==================END================\r\n");
#endif // PYJ.2020.06.20_END -- 
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L);
             bluecell_Currdatastatus.ULO_Level4_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level4_L = ((int16_t)ResultData & 0x00FF);           
////            CurrentATTENVALUE = bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H << 8 | bluecell_Currdatastatus.MBIC_ULO_ALC_Atten1_H;
            IwillGiveAtten = ALC_Calc(ALC_Alarm_UL4_Index,CurrentATTENVALUE,Limitdata,ResdBm);
            if( (IwillGiveAtten > 0) || (CurrentATTENVALUE >= (IwillGiveAtten * -1))){
                tempcurratten = CurrentATTENVALUE * 10;
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H = (((tempcurratten + IwillGiveAtten) & 0xFF00) >> 8);
                bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L = (tempcurratten + IwillGiveAtten) & 0x00FF;
                CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
            }
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten4_L);

#endif // PYJ.2020.05.22_END -- 
            ALCTimerCnt = 0;
        }else{
#if 1 // PYJ.2020.06.21_BEGIN -- 
            ALC_Table = &ALC_Table_ref[ALC_Table_UL1_Ref_Index].DET_UL_0;
            ret  = bluecell_Currdatastatus.ULO_P1_Level1_H << 8;
            ret += bluecell_Currdatastatus.ULO_P1_Level1_L;   
            ret *= 0.001;

            
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL1.Table_Det_15_dBm_H,Det_UL1.Table_Length* 2,ALC_Table);
            ResultData = ResdBm;
//            if(ResultData == 0){
//                ResultData =
//            }
            bluecell_Currdatastatus.ULO_Level1_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level1_L = ((int16_t)ResultData & 0x00FF);
#if 1 // PYJ.2020.06.20_BEGIN -- 
            printf("============ALC_OFF======START================\r\n");
            printf("Current UL ADC VALUE : %f \r\n",ret);
            printf("ResdBm : UL1 : %f \r\n",ResdBm);
            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level1_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level1_H);
            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level1_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level1_L);            
            printf("==================END================\r\n");
#endif // PYJ.2020.06.20_END -- 

#endif // PYJ.2020.06.21_END -- 

#if 1 // PYJ.2020.06.22_BEGIN -- 
            ret  = bluecell_Currdatastatus.ULO_P2_Level2_H << 8;
            ret += bluecell_Currdatastatus.ULO_P2_Level2_L;   
            ret *= 0.001;

            ALC_Table = &ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_0;
            Bluecell_StructCpy(&AutoControl_Save[0][0],&ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_0,sizeof(ALC_dBm_t));
//            for(int i = 0; i < sizeof(ALC_dBm_t); i++)
//                printf("ref Tabe: %d \r\n",ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_0      );      
//            printf("ref Tabe: %d \r\n",ALC_Table_ref[ALC_Table_UL2_Ref_Index].DET_UL_1      );      

            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL2.Table_Det_15_dBm_H,Det_UL2.Table_Length * 2,ALC_Table);
            ResultData = ResdBm;
            CurrentATTENVALUE = PE43711_Double(bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_H,bluecell_Currdatastatus.MBIC_ULO_ALC_Atten2_L);

//            printf("CurrentATTENVALUE : %f I WILL GIVE ATTEN  : %d \r\n",CurrentATTENVALUE,IwillGiveAtten);  
            bluecell_Currdatastatus.ULO_Level2_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level2_L = ((int16_t)ResultData & 0x00FF);
#endif // PYJ.2020.06.22_END -- 

#if 0 // PYJ.2020.06.21_BEGIN -- 
                        printf("=========ALC_OFF=========START================\r\n");
                        printf("Current UL ADC VALUE : %f \r\n",ret);
                        printf("ResdBm : UL2 : %f \r\n",ResdBm);
                        printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level2_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level2_H);
                        printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level2_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level2_L);            
                        printf("==================END================\r\n");
#endif // PYJ.2020.06.21_END -- 

#if 1 // PYJ.2020.06.20_BEGIN -- 
#if 1 // PYJ.2020.06.21_BEGIN -- 
            ALC_Table = &ALC_Table_ref[ALC_Table_UL3_Ref_Index].DET_UL_0;

            ret  = bluecell_Currdatastatus.ULO_P3_Level3_H << 8;
            ret += bluecell_Currdatastatus.ULO_P3_Level3_L;   
            ret *= 0.001;

            
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL3.Table_Det_15_dBm_H,Det_UL3.Table_Length* 2,ALC_Table);
            ResultData = ResdBm;
            bluecell_Currdatastatus.ULO_Level3_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level3_L = ((int16_t)ResultData & 0x00FF);
#if 0 // PYJ.2020.06.21_BEGIN -- 
                        printf("=========ALC_OFF=========START================\r\n");
                        printf("Current UL ADC VALUE : %f \r\n",ret);
                        printf("ResdBm : UL3 : %f \r\n",ResdBm);
                        printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level3_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level3_H);
                        printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level3_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level3_L);            
                        printf("==================END================\r\n");
#endif // PYJ.2020.06.21_END --             
#if 0 // PYJ.2020.06.21_BEGIN -- 
            printf("=========ALC_OFF=========START================\r\n");
            printf("Current UL ADC VALUE : %f \r\n",ret);
            printf("ResdBm : UL3 : %f \r\n",ResdBm);
            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level3_H : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level3_H);
            printf("ResdBm  : %x /////  bluecell_Currdatastatus.ULO_Level3_L : %x \r\n",ResultData,bluecell_Currdatastatus.ULO_Level3_L);            
            printf("==================END================\r\n");
#endif // PYJ.2020.06.21_END -- 

#if 1 // PYJ.2020.06.22_BEGIN -- 
            ALC_Table = &ALC_Table_ref[ALC_Table_UL4_Ref_Index].DET_UL_0;

            ret  = bluecell_Currdatastatus.ULO_P4_Level4_H << 8;
            ret += bluecell_Currdatastatus.ULO_P4_Level4_L;   
            ret *= 0.001;

            
            ResdBm = AutoControl_ADC_Compare(ret,&Det_UL4.Table_Det_15_dBm_H,Det_UL4.Table_Length* 2,ALC_Table);
            ResultData = ResdBm;; 
            bluecell_Currdatastatus.ULO_Level4_H = ((int16_t)ResultData & 0xFF00) >> 8;
            bluecell_Currdatastatus.ULO_Level4_L = ((int16_t)ResultData & 0x00FF);            
#endif // PYJ.2020.06.22_END -- 
#endif // PYJ.2020.06.21_END -- 
#endif // PYJ.2020.06.20_END -- 

        }
    }
}
//-15 ~ -5
int8_t AGC_Calc(int8_t threshold,int8_t CurrDet){
    int8_t Attenret = 0;
    if(threshold > CurrDet)
        return Attenret;
    
    if(threshold != CurrDet){
        Attenret = (threshold - CurrDet) * -1;
    }
//    printf("Attenret : %d \r\n",Attenret);
    return Attenret;
}

bool AGC_AlarmSet[AGC_Alarm_DL_Index_MAX] = {0,};
//bool AGC_AlarmTimerSet[AGC_Alarm_DL_Index_MAX] = {false,};
void AGC_Alarm_Check(uint8_t Path_Index,double AGC_Det,uint8_t* AGC_Table,uint16_t CurrAtten){
    double TableAdc = 0;
    double PrevTableAdc = 0;
    double step = 0;

    TableAdc = AGC_Table[0] << 8;
    TableAdc += AGC_Table[1];
    TableAdc /= 1000;

    
    PrevTableAdc = AGC_Table[2] << 8;
    PrevTableAdc += AGC_Table[3];
    PrevTableAdc /= 1000;
    
    step = TableAdc - PrevTableAdc;
    

    if(AGC_Det > TableAdc + step){/*Alarm Enable*/
        AGC_AlarmSet[Path_Index] = true; /*Alarm Check val*/
//        AGC_AlarmTimerSet[Path_Index] = false;/*Alarm Timer Setting Val*/
//        AGCAlarmTimerCnt[Path_Index] = 0;/*Alarm Time Cnt Val*/
//        printf("AGC ALARM ON %d \r\n",Path_Index + 1);
    }else{/*Alarm Disable*/
//        if(AGC_AlarmSet[Path_Index] == true && CurrAtten <= 18){/*Alarm Disalbe Condition*/
//            AGC_AlarmTimerSet[Path_Index] = true;
//        }else{/*Alarm Disalbe Fail*/
//            AGC_AlarmTimerSet[Path_Index] = false;
//            AGCAlarmTimerCnt[Path_Index] = 0;
//        }
//        if(AGCAlarmTimerCnt[Path_Index] > 3000){
            AGC_AlarmSet[Path_Index] = false;
//        }
    }

}



void AGC_Function(){//DL
    double ret = 0;
    int8_t ResdBm = 0;

    int16_t Levelret = 0;
    int i = 0;
    int16_t Limitdata = 0;
    int16_t IwillgiveAtten = 0;
    int16_t ResultData = 0;
    static int8_t* AGC_Table;// = &AGC_Table_ref[MBIC_Table_DL1_INDEX].DET_DL_0;
    int16_t DL_Curr_Atten[AGC_Alarm_DL_Index_MAX] = {
        bluecell_Currdatastatus.ATT_DL1_H  << 8 | bluecell_Currdatastatus.ATT_DL1_L,
        bluecell_Currdatastatus.ATT_DL2_H  << 8 | bluecell_Currdatastatus.ATT_DL2_L,
        bluecell_Currdatastatus.ATT_DL3_H  << 8 | bluecell_Currdatastatus.ATT_DL3_L,
        bluecell_Currdatastatus.ATT_DL4_H  << 8 | bluecell_Currdatastatus.ATT_DL4_L,        
    };
         
    if(AGCTimerCnt > 100){
        

        if(bluecell_Currdatastatus.DLI_AGC_ON_OFF == true){
            Limitdata = (( bluecell_Currdatastatus.DLI_AGC_Threshold_H << 8) & 0xFF00) ;
            Limitdata +=  bluecell_Currdatastatus.DLI_AGC_Threshold_L ;
            Limitdata /= 10; 
        DL_Curr_Atten[AGC_Alarm_DL1_Index] = 
            bluecell_Currdatastatus.ATT_DL1_H  << 8 | bluecell_Currdatastatus.ATT_DL1_L;
        DL_Curr_Atten[AGC_Alarm_DL2_Index] = 
            bluecell_Currdatastatus.ATT_DL2_H  << 8 | bluecell_Currdatastatus.ATT_DL2_L;
        DL_Curr_Atten[AGC_Alarm_DL3_Index] = 
            bluecell_Currdatastatus.ATT_DL3_H  << 8 | bluecell_Currdatastatus.ATT_DL3_L;        
        DL_Curr_Atten[AGC_Alarm_DL4_Index] = 
            bluecell_Currdatastatus.ATT_DL4_H  << 8 | bluecell_Currdatastatus.ATT_DL4_L;            
#if 0 // PYJ.2020.06.23_BEGIN -- 
            AGC_Table = &AGC_Table_ref[MBIC_Table_DL1_INDEX].DET_DL_0;
            ret  = bluecell_Currdatastatus.DLI_P1_Level1_H << 8;
            ret += bluecell_Currdatastatus.DLI_P1_Level1_L;   
            ret *= 0.001;
            
            ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL1.Table_Det5_dBm_H,Det_DL1.Table_Length * 2,AGC_Table));
             bluecell_Currdatastatus.DLI_Level1_H = ((int16_t)ResdBm & 0xFF00) >> 8;
             bluecell_Currdatastatus.DLI_Level1_L = ((int16_t)ResdBm & 0x00FF);
            for(i = 0; i < sizeof(AGC_dBm_t); i++){
                if(AGC_Table[i] == ResdBm)
                    break;
            }
            IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
            IwillgiveAtten *= -1; 
            AGC_Alarm_Check(AGC_Alarm_DL1_Index,ret,&Det_DL1.Table_Det5_dBm_H,IwillgiveAtten);
            bluecell_Currdatastatus.ATT_DL1_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
            bluecell_Currdatastatus.ATT_DL1_L = (((IwillgiveAtten * 10) & 0x00FF));     
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
#endif // PYJ.2020.06.23_END -- 




#if 1 // PYJ.2020.05.25_BEGIN -- 
#if 1// PYJ.2020.06.22_BEGIN -- 
#if 0 // PYJ.2020.06.23_BEGIN -- 
       AGC_Table = &AGC_Table_ref[MBIC_Table_DL2_INDEX].DET_DL_0;
       ret  = bluecell_Currdatastatus.DLI_P2_Level2_H << 8;
       ret += bluecell_Currdatastatus.DLI_P2_Level2_L;   
       ret *= 0.001;
      
       ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL2.Table_Det5_dBm_H,Det_DL2.Table_Length * 2,AGC_Table));
        bluecell_Currdatastatus.DLI_Level2_H = ((int16_t)ResdBm & 0xFF00) >> 8;
        bluecell_Currdatastatus.DLI_Level2_L = ((int16_t)ResdBm & 0x00FF);
       for(i = 0; i < sizeof(AGC_dBm_t); i++){
           if(AGC_Table[i] == ResdBm)
               break;
       }
   

       IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
       AGC_Alarm_Check(AGC_Alarm_DL2_Index,ret,&Det_DL2.Table_Det5_dBm_H,IwillgiveAtten);

       bluecell_Currdatastatus.ATT_DL2_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
       bluecell_Currdatastatus.ATT_DL2_L = (((IwillgiveAtten * 10) & 0x00FF));     
       CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
#endif // PYJ.2020.06.23_END -- 
        AGC_Table = &AGC_Table_ref[MBIC_Table_DL1_INDEX].DET_DL_0;
        ret  = bluecell_Currdatastatus.DLI_P1_Level1_H << 8;
        ret += bluecell_Currdatastatus.DLI_P1_Level1_L;   
        ret *= 0.001;
        ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL1.Table_Det5_dBm_H,Det_DL1.Table_Length * 2,AGC_Table));
        bluecell_Currdatastatus.DLI_Level1_H = ((int16_t)ResdBm & 0xFF00) >> 8;
        bluecell_Currdatastatus.DLI_Level1_L = ((int16_t)ResdBm & 0x00FF);
        for(i = 0; i < sizeof(AGC_dBm_t); i++){
            if(AGC_Table[i] == ResdBm)
                break;
        }
        IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
        IwillgiveAtten *= -1;
//        printf("==================================\r\n");
//        printf("Current DL1 ADC VALUE : %f \r\n",ret);
//        printf("ResdBm : DL1 : %d \r\n",ResdBm);
//        printf("I WILL GIVE YOU ATTEN  : %d\r\n",IwillgiveAtten);            
//        printf("AGC  : %d\r\n",Limitdata);            


        if(ResdBm > Limitdata &&  DL_PrevIwillgiveAtten[AGC_Alarm_DL1_Index] > IwillgiveAtten){
//            printf("IwillgiveAtten : %d \r\n",IwillgiveAtten);
            DL_PrevIwillgiveAtten[AGC_Alarm_DL1_Index] = IwillgiveAtten;
            IwillgiveAtten = (DL_AGC_StartAtten[AGC_Alarm_DL1_Index]/10) + IwillgiveAtten;
//            printf("WILLATTEN RET : %d \r\n",IwillgiveAtten);
            AGC_Alarm_Check(AGC_Alarm_DL1_Index,ret,&Det_DL1.Table_Det5_dBm_H,IwillgiveAtten);
            bluecell_Currdatastatus.ATT_DL1_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
            bluecell_Currdatastatus.ATT_DL1_L = (((IwillgiveAtten * 10) & 0x00FF));     
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
        AGC_Table = &AGC_Table_ref[MBIC_Table_DL2_INDEX].DET_DL_0;
        ret  = bluecell_Currdatastatus.DLI_P2_Level2_H << 8;
        ret += bluecell_Currdatastatus.DLI_P2_Level2_L;   
        ret *= 0.001;
        ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL2.Table_Det5_dBm_H,Det_DL2.Table_Length * 2,AGC_Table));
        bluecell_Currdatastatus.DLI_Level2_H = ((int16_t)ResdBm & 0xFF00) >> 8;
        bluecell_Currdatastatus.DLI_Level2_L = ((int16_t)ResdBm & 0x00FF);
        for(i = 0; i < sizeof(AGC_dBm_t); i++){
            if(AGC_Table[i] == ResdBm)
                break;
        }
        IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
        IwillgiveAtten *= -1;
//        printf("==================================\r\n");
//        printf("Current DL2 ADC VALUE : %f \r\n",ret);
//        printf("ResdBm : DL2 : %d \r\n",ResdBm);
//        printf("I WILL GIVE YOU ATTEN  : %d\r\n",IwillgiveAtten);            
//        printf("AGC  : %d\r\n",Limitdata);            


        if(ResdBm > Limitdata &&  DL_PrevIwillgiveAtten[AGC_Alarm_DL2_Index] > IwillgiveAtten){
//            printf("IwillgiveAtten : %d \r\n",IwillgiveAtten);
            DL_PrevIwillgiveAtten[AGC_Alarm_DL2_Index] = IwillgiveAtten;
            IwillgiveAtten = (DL_AGC_StartAtten[AGC_Alarm_DL2_Index]/10) + IwillgiveAtten;
//            printf("WILLATTEN RET : %d \r\n",IwillgiveAtten);
            AGC_Alarm_Check(AGC_Alarm_DL2_Index,ret,&Det_DL2.Table_Det5_dBm_H,IwillgiveAtten);
            bluecell_Currdatastatus.ATT_DL2_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
            bluecell_Currdatastatus.ATT_DL2_L = (((IwillgiveAtten * 10) & 0x00FF));     
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }


        AGC_Table = &AGC_Table_ref[MBIC_Table_DL3_INDEX].DET_DL_0;
        ret  = bluecell_Currdatastatus.DLI_P3_Level3_H << 8;
        ret += bluecell_Currdatastatus.DLI_P3_Level3_L;   
        ret *= 0.001;
        ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL3.Table_Det5_dBm_H,Det_DL3.Table_Length * 2,AGC_Table));
        bluecell_Currdatastatus.DLI_Level3_H = ((int16_t)ResdBm & 0xFF00) >> 8;
        bluecell_Currdatastatus.DLI_Level3_L = ((int16_t)ResdBm & 0x00FF);
        for(i = 0; i < sizeof(AGC_dBm_t); i++){
            if(AGC_Table[i] == ResdBm)
                break;
        }
        IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
        IwillgiveAtten *= -1;
//        printf("==================================\r\n");
//        printf("Current DL3 ADC VALUE : %f \r\n",ret);
//        printf("ResdBm : DL3 : %d \r\n",ResdBm);
//        printf("I WILL GIVE YOU ATTEN  : %d\r\n",IwillgiveAtten);            
//        printf("AGC  : %d\r\n",Limitdata);            


        if(ResdBm > Limitdata &&  DL_PrevIwillgiveAtten[AGC_Alarm_DL3_Index] > IwillgiveAtten){
//            printf("IwillgiveAtten : %d \r\n",IwillgiveAtten);
            DL_PrevIwillgiveAtten[AGC_Alarm_DL3_Index] = IwillgiveAtten;
            IwillgiveAtten = (DL_AGC_StartAtten[AGC_Alarm_DL3_Index]/10) + IwillgiveAtten;
//            printf("WILLATTEN RET : %d \r\n",IwillgiveAtten);
            AGC_Alarm_Check(AGC_Alarm_DL3_Index,ret,&Det_DL3.Table_Det5_dBm_H,IwillgiveAtten);
            bluecell_Currdatastatus.ATT_DL3_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
            bluecell_Currdatastatus.ATT_DL3_L = (((IwillgiveAtten * 10) & 0x00FF));     
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }
//        printf("DL_Curr_Atten[AGC_Alarm_DL3_Index] : %d  \r\n",DL_Curr_Atten[AGC_Alarm_DL3_Index]);
//            IwillgiveAtten < DL_Curr_Atten[AGC_Alarm_DL3_Index];
        
        AGC_Table = &AGC_Table_ref[MBIC_Table_DL4_INDEX].DET_DL_0;
        ret  = bluecell_Currdatastatus.DLI_P4_Level4_H << 8;
        ret += bluecell_Currdatastatus.DLI_P4_Level4_L;   
        ret *= 0.001;
        ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL4.Table_Det5_dBm_H,Det_DL4.Table_Length * 2,AGC_Table));
        bluecell_Currdatastatus.DLI_Level4_H = ((int16_t)ResdBm & 0xFF00) >> 8;
        bluecell_Currdatastatus.DLI_Level4_L = ((int16_t)ResdBm & 0x00FF);
        for(i = 0; i < sizeof(AGC_dBm_t); i++){
            if(AGC_Table[i] == ResdBm)
                break;
        }
        IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
        IwillgiveAtten *= -1;
//        printf("==================================\r\n");
//        printf("Current DL4 ADC VALUE : %f \r\n",ret);
//        printf("ResdBm : DL4 : %d \r\n",ResdBm);
//        printf("I WILL GIVE YOU ATTEN  : %d\r\n",IwillgiveAtten);            
//        printf("AGC  : %d\r\n",Limitdata);            


        if(ResdBm > Limitdata &&  DL_PrevIwillgiveAtten[AGC_Alarm_DL4_Index] > IwillgiveAtten){
//            printf("IwillgiveAtten : %d \r\n",IwillgiveAtten);
            DL_PrevIwillgiveAtten[AGC_Alarm_DL4_Index] = IwillgiveAtten;
            IwillgiveAtten = (DL_AGC_StartAtten[AGC_Alarm_DL4_Index]/10) + IwillgiveAtten;
//            printf("WILLATTEN RET : %d \r\n",IwillgiveAtten);
            AGC_Alarm_Check(AGC_Alarm_DL4_Index,ret,&Det_DL4.Table_Det5_dBm_H,IwillgiveAtten);
            bluecell_Currdatastatus.ATT_DL4_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
            bluecell_Currdatastatus.ATT_DL4_L = (((IwillgiveAtten * 10) & 0x00FF));     
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
        }




#if 0 // PYJ.2020.06.23_BEGIN -- 
            AGC_Table = &AGC_Table_ref[MBIC_Table_DL4_INDEX].DET_DL_0;

            ret  = bluecell_Currdatastatus.DLI_P4_Level4_H << 8;
            ret += bluecell_Currdatastatus.DLI_P4_Level4_L;   
            ret *= 0.001;
            ResdBm = (int8_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL4.Table_Det5_dBm_H,Det_DL4.Table_Length * 2,AGC_Table));
            bluecell_Currdatastatus.DLI_Level4_H = ((int16_t)ResdBm & 0xFF00) >> 8;
            bluecell_Currdatastatus.DLI_Level4_L = ((int16_t)ResdBm & 0x00FF);            
            for(i = 0; i < sizeof(AGC_dBm_t); i++){
                if(AGC_Table[i] == ResdBm)
                    break;
            }
            Limitdata = (( bluecell_Currdatastatus.DLI_AGC_Threshold_H << 8) & 0xFF00) ;
            Limitdata +=  bluecell_Currdatastatus.DLI_AGC_Threshold_L ;
            Limitdata /= 10;
        
//            printf("==================================\r\n");
//            printf("Current DL4 ADC VALUE : %f \r\n",ret);
//            printf("ResdBm : DL4 : %d \r\n",ResdBm);
//            printf("AGC_Table[%d] : %d\r\n",i,AGC_Table[i]);
//            printf("I WILL GIVE YOU ATTEN  : %d\r\n",AGC_Calc(Limitdata,ResdBm));            
//            printf("AGC  : %d\r\n",Limitdata);            
//            HAL_Delay(1000);

            IwillgiveAtten = AGC_Calc(Limitdata,ResdBm);
            AGC_Alarm_Check(AGC_Alarm_DL4_Index,ret,&Det_DL4.Table_Det5_dBm_H,IwillgiveAtten);
            bluecell_Currdatastatus.ATT_DL4_H = (((IwillgiveAtten * 10) & 0xFF00) >> 8) ;
            bluecell_Currdatastatus.ATT_DL4_L = (((IwillgiveAtten * 10) & 0x00FF));     
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
#endif // PYJ.2020.06.23_END -- 
#endif // PYJ.2020.05.25_END --             
#endif // PYJ.2020.06.22_END -- 
        }else{

            AGC_Table = &AGC_Table_ref[MBIC_Table_DL1_INDEX].DET_DL_0;
            ret  = bluecell_Currdatastatus.DLI_P1_Level1_H << 8;
            ret += bluecell_Currdatastatus.DLI_P1_Level1_L;   
            ret *= 0.001;
            
            Levelret = (int16_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL1.Table_Det5_dBm_H,Det_DL2.Table_Length * 2,AGC_Table));
            bluecell_Currdatastatus.DLI_Level1_H = (Levelret & 0xFF00) >> 8;
            bluecell_Currdatastatus.DLI_Level1_L = (Levelret & 0x00FF);
#if 1 // PYJ.2020.06.20_BEGIN -- 
            printf("========AGC OFF==========START================\r\n");
            printf(ANSI_COLOR_YELLOW"Current DL1 ADC VALUE : %f \r\n"ANSI_COLOR_RESET,ret);
            printf("AGC_Table[%d] : %d\r\n",i,AGC_Table[i]);
            printf(ANSI_COLOR_YELLOW "Levelret : DL1 : %d \r\n"ANSI_COLOR_RESET ,Levelret);
            printf("Levelret  : %x /////  bluecell_Currdatastatus.DLI_Level1_H : %x \r\n",Levelret,bluecell_Currdatastatus.DLI_Level1_H);
            printf("Levelret  : %x /////  bluecell_Currdatastatus.DLI_Level1_L : %x \r\n",Levelret,bluecell_Currdatastatus.DLI_Level1_L);            
            printf("==================END================\r\n");        
#endif // PYJ.2020.06.20_END -- 

//            printf("==================================\r\n");
//            printf("Current DL1 ADC VALUE : %f \r\n",ret);
//              printf("ResdBm : DL1 : %d \r\n",ResdBm);
//            HAL_Delay(1000);

#if 1 // PYJ.2020.06.23_BEGIN -- 
            AGC_Table = &AGC_Table_ref[MBIC_Table_DL2_INDEX].DET_DL_0;
            ret  = bluecell_Currdatastatus.DLI_P2_Level2_H << 8;
            ret += bluecell_Currdatastatus.DLI_P2_Level2_L;   
            ret *= 0.001;
            
            Levelret = (int16_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL2.Table_Det5_dBm_H,Det_DL1.Table_Length * 2,AGC_Table));
            bluecell_Currdatastatus.DLI_Level2_H = (Levelret & 0xFF00) >> 8;
            bluecell_Currdatastatus.DLI_Level2_L = (Levelret & 0x00FF);
            AGC_Table = &AGC_Table_ref[MBIC_Table_DL3_INDEX].DET_DL_0;
            ret  = bluecell_Currdatastatus.DLI_P3_Level3_H << 8;
            ret += bluecell_Currdatastatus.DLI_P3_Level3_L;   
            ret *= 0.001;
            
            
            
            Levelret = (int16_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL3.Table_Det5_dBm_H,Det_DL3.Table_Length * 2,AGC_Table));
            bluecell_Currdatastatus.DLI_Level3_H = (Levelret & 0xFF00) >> 8;
            bluecell_Currdatastatus.DLI_Level3_L = (Levelret & 0x00FF);
            
            ret  = bluecell_Currdatastatus.DLI_P3_Level3_H << 8;
            ret += bluecell_Currdatastatus.DLI_P3_Level3_L;   
            ret *= 0.001;
            
            
            
            Levelret = (int16_t)Bluecell_TestPro(AGC_AutoControl_ADC_Compare(ret,&Det_DL4.Table_Det5_dBm_H,Det_DL4.Table_Length * 2,AGC_Table));
            AGC_Table = &AGC_Table_ref[MBIC_Table_DL4_INDEX].DET_DL_0;
            bluecell_Currdatastatus.DLI_Level4_H = (Levelret & 0xFF00) >> 8;
            bluecell_Currdatastatus.DLI_Level4_L = (Levelret & 0x00FF);
#endif // PYJ.2020.06.23_END -- 


        }
        AGCTimerCnt = 0;
    }
}

void DLI_LevelAlarmCheck(){
    //double temp = 0;
    //double ret = 0;
    //int8_t ResdBm[4] = {0,};

    /*********************DL LEVEL LOW START****************************/
    if(DET_DL_Low_On_AlarmTimerCnt[DET_Alarm_DL1_Index] >= MBIC_ON_MAINTAIN_SEC
     &&ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P1_LEVEL_LOW;
        bluecell_Currdatastatus.DLI_Level_Low_Alarm1 = true;

    }else{/**/
        if(DET_DL_Low_Off_AlarmTimerCnt[DET_Alarm_DL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P1_LEVEL_LOW;
            bluecell_Currdatastatus.DLI_Level_Low_Alarm1 = false;
        }
    }

    if(DET_DL_Low_On_AlarmTimerCnt[DET_Alarm_DL2_Index] >= MBIC_ON_MAINTAIN_SEC
     &&ADC_Alarm_DL_Low_Set[DET_Alarm_DL2_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P2_LEVEL_LOW;
        bluecell_Currdatastatus.DLI_Level_Low_Alarm2 = true;

    }else{
        if(DET_DL_Low_Off_AlarmTimerCnt[DET_Alarm_DL2_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P2_LEVEL_LOW;
            bluecell_Currdatastatus.DLI_Level_Low_Alarm2 = false;
        }
    }

    if(DET_DL_Low_On_AlarmTimerCnt[DET_Alarm_DL3_Index] >= MBIC_ON_MAINTAIN_SEC
     &&ADC_Alarm_DL_Low_Set[DET_Alarm_DL3_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P3_LEVEL_LOW;
        bluecell_Currdatastatus.DLI_Level_Low_Alarm3 = true;

    }else{
        if(DET_DL_Low_Off_AlarmTimerCnt[DET_Alarm_DL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P3_LEVEL_LOW;
            bluecell_Currdatastatus.DLI_Level_Low_Alarm3 = false;
        }
    }

    if(DET_DL_Low_On_AlarmTimerCnt[DET_Alarm_DL4_Index] >= MBIC_ON_MAINTAIN_SEC
     &&ADC_Alarm_DL_Low_Set[DET_Alarm_DL4_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P4_LEVEL_LOW;
        bluecell_Currdatastatus.DLI_Level_Low_Alarm4 = true;

    }else{
        if(DET_DL_Low_Off_AlarmTimerCnt[DET_Alarm_DL4_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P4_LEVEL_LOW;
            bluecell_Currdatastatus.DLI_Level_Low_Alarm4 = false;
        }
    }     

    /*********************DL LEVEL HIGH START***************************/
    if(DET_DL_High_On_AlarmTimerCnt[DET_Alarm_DL1_Index] >= MBIC_ON_MAINTAIN_SEC
    &&ADC_Alarm_DL_High_Set[DET_Alarm_DL1_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P1_LEVEL_HIGH;
        bluecell_Currdatastatus.DLI_Level_High_Alarm1 = true;

    }else{/**/
        if(DET_DL_High_Off_AlarmTimerCnt[DET_Alarm_DL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P1_LEVEL_HIGH;
            bluecell_Currdatastatus.DLI_Level_High_Alarm1 = false;
        }
    }

    if(DET_DL_High_On_AlarmTimerCnt[DET_Alarm_DL2_Index] >= MBIC_ON_MAINTAIN_SEC
    &&ADC_Alarm_DL_High_Set[DET_Alarm_DL2_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P2_LEVEL_HIGH;
        bluecell_Currdatastatus.DLI_Level_High_Alarm2 = true;

    }else{
        if(DET_DL_High_Off_AlarmTimerCnt[DET_Alarm_DL2_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P2_LEVEL_HIGH;
            bluecell_Currdatastatus.DLI_Level_High_Alarm2 = false;
        }
    }

    if(DET_DL_High_On_AlarmTimerCnt[DET_Alarm_DL3_Index] >= MBIC_ON_MAINTAIN_SEC
    &&ADC_Alarm_DL_High_Set[DET_Alarm_DL3_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P3_LEVEL_HIGH;
        bluecell_Currdatastatus.DLI_Level_High_Alarm3 = true;

    }else{
        if(DET_DL_High_Off_AlarmTimerCnt[DET_Alarm_DL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P3_LEVEL_HIGH;
            bluecell_Currdatastatus.DLI_Level_High_Alarm3 = false;
        }
    }

    if(DET_DL_High_On_AlarmTimerCnt[DET_Alarm_DL4_Index] >= MBIC_ON_MAINTAIN_SEC
    &&ADC_Alarm_DL_High_Set[DET_Alarm_DL4_Index] == true){
        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P4_LEVEL_HIGH;
        bluecell_Currdatastatus.DLI_Level_High_Alarm4 = true;

    }else{
        if(DET_DL_High_Off_AlarmTimerCnt[DET_Alarm_DL4_Index] >= MBIC_OFF_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P4_LEVEL_HIGH;
            bluecell_Currdatastatus.DLI_Level_High_Alarm4 = false;
        }
    }      
        

  

}
void ULO_LevelAlarmCheck(){
    //double temp = 0;
    //double ret = 0;
    //int8_t ResdBm[4] = {0,};
#if 1 // PYJ.2020.05.27_BEGIN -- 
        if(DET_UL_On_AlarmTimerCnt[DET_Alarm_UL1_Index] >= MBIC_ON_MAINTAIN_SEC
         &&ADC_Alarm_UL_Set[DET_Alarm_UL1_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P1_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm1 = true;
        
        }else{/**/
            if(DET_UL_Off_AlarmTimerCnt[DET_Alarm_UL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P1_LEVEL_HIGH;
                bluecell_Currdatastatus.ULO_Level_High_Alarm1 = false;
            }
        }
        
        if(DET_UL_On_AlarmTimerCnt[DET_Alarm_UL2_Index] >= MBIC_ON_MAINTAIN_SEC
         &&ADC_Alarm_UL_Set[DET_Alarm_UL2_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P2_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm2 = true;
        
        }else{
            if(DET_UL_Off_AlarmTimerCnt[DET_Alarm_UL2_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P2_LEVEL_HIGH;
                bluecell_Currdatastatus.ULO_Level_High_Alarm2 = false;
            }
        }
        
        if(DET_UL_On_AlarmTimerCnt[DET_Alarm_UL3_Index] >= MBIC_ON_MAINTAIN_SEC
         &&ADC_Alarm_UL_Set[DET_Alarm_UL3_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P3_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm3 = true;
        
        }else{
            if(DET_UL_Off_AlarmTimerCnt[DET_Alarm_UL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P3_LEVEL_HIGH;
                bluecell_Currdatastatus.ULO_Level_High_Alarm3 = false;
            }
        }
        
        if(DET_UL_On_AlarmTimerCnt[DET_Alarm_UL4_Index] >= MBIC_ON_MAINTAIN_SEC
         &&ADC_Alarm_UL_Set[DET_Alarm_UL4_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P4_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm4 = true;
        
        }else{
            if(DET_UL_Off_AlarmTimerCnt[DET_Alarm_UL4_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P4_LEVEL_HIGH;
                bluecell_Currdatastatus.ULO_Level_High_Alarm4 = false;
            }
        }   

#else
        if(ADC_Alarm_UL_Set[DET_Alarm_UL1_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P1_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm1 = true;
    
        }else{
            bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P1_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm1 = false;
        }
    
        if(ADC_Alarm_UL_Set[DET_Alarm_UL2_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P2_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm2 = true;
    
        }else{
            bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P2_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm2 = false;
        }
    
        if(ADC_Alarm_UL_Set[DET_Alarm_UL3_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P3_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm3 = true;
    
        }else{
            bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P3_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm3 = false;
        }
    
        if(ADC_Alarm_UL_Set[DET_Alarm_UL4_Index] == true){
            bluecell_Currdatastatus.ALARM_ULO_Level |= ALARM_ULO_P4_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm4 = true;
    
        }else{
            bluecell_Currdatastatus.ALARM_ULO_Level &= ~ALARM_ULO_P4_LEVEL_HIGH;
            bluecell_Currdatastatus.ULO_Level_High_Alarm4 = false;
        }
    
#endif // PYJ.2020.05.27_END -- 

}
#if 0 // PYJ.2020.06.21_BEGIN -- 
void DLI_ShutdownAlarmCheck(){
    //double temp = 0;
    //double ret = 0;
    //int8_t ResdBm[4] = {0,};
    /*DLI Shutdown On*/
    if(bluecell_Currdatastatus.DLI_Shutdown_ON_OFF == true){    
       if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
              {
               
                   if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 >= RETRYCNT_MAX){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P1;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm1       = true;
                       HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_RESET);//CLOCK            
                       bluecell_Currdatastatus.ATT_DL1_PATH = false;
                   }else{
                       if(HAL_GPIO_ReadPin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin) == GPIO_PIN_SET)
                       {//CLOCK;        
                           bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1++;
                       }
                       /*ShutDown Inc*/
                       if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                           /*Path Recovery*/
                           HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_SET);//CLOCK
                           bluecell_Prevdatastatus.ATT_DL1_H = 0xFF;
                           bluecell_Prevdatastatus.ATT_DL1_L =0xFF;
                           bluecell_Currdatastatus.ATT_DL1_PATH = true;
                           CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                           DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] = 0;
                       }else{
                           /*Path Off*/
                           HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_RESET);//CLOCK
                           bluecell_Currdatastatus.ATT_DL1_PATH = false;
                       }
                   }
               }
               else{
                   if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P1;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm1       = false;            
                   }
               } 
               
#if 0 // PYJ.2020.06.19_BEGIN -- 
               if(ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL2_Shutdown_Index] == true
                &&DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
              {
               
                   if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 >= RETRYCNT_MAX){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P2;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm2       = true;
                       HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_RESET);//CLOCK            
                       bluecell_Currdatastatus.ATT_DL2_PATH = false;
                   }else{
                       if(HAL_GPIO_ReadPin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin) == GPIO_PIN_SET)
                       {//CLOCK;        
                           bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2++;
                       }
                       /*ShutDown Inc*/
                       if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                           /*Path Recovery*/
                           HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_SET);//CLOCK
                           bluecell_Prevdatastatus.ATT_DL2_H = 0xFF;
                           bluecell_Prevdatastatus.ATT_DL2_L = 0xFF;
                           bluecell_Currdatastatus.ATT_DL2_PATH = true;
                           CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                           DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] = 0;
                       }else{
                           /*Path Off*/
                           HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_RESET);//CLOCK
                           bluecell_Currdatastatus.ATT_DL2_PATH = false;
                       }
                   }
               }
               else{
                   if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P2;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm2       = false;            
                   }
               } 
       
               if(ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL3_Shutdown_Index] == true
                &&DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
              {
               
                   if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 >= RETRYCNT_MAX){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P3;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm3       = true;
                       HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_RESET);//CLOCK            
                       bluecell_Currdatastatus.ATT_DL3_PATH = false;
                   }else{
                       if(HAL_GPIO_ReadPin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin) == GPIO_PIN_SET)
                       {//CLOCK;        
                           bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3++;
                       }
                       /*ShutDown Inc*/
                       if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                           /*Path Recovery*/
                           HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_SET);//CLOCK
                           bluecell_Prevdatastatus.ATT_DL3_H = 0xFF;
                           bluecell_Prevdatastatus.ATT_DL3_L = 0xFF;
                           bluecell_Currdatastatus.ATT_DL3_PATH = true;
                           CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                           DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] = 0;
                       }else{
                           /*Path Off*/
                           HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_RESET);//CLOCK
                           bluecell_Currdatastatus.ATT_DL3_PATH = false;
                       }
                   }
               }
               else{
                   if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P3;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm3       = false;            
                   }
               } 
       
       
               if(ADC_Alarm_DL_Shutdown_Set[DET_Alarm_DL4_Shutdown_Index] == true
                &&DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
              {
               
                   if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 >= RETRYCNT_MAX){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P4;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm4       = true;
                       HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_RESET);//CLOCK            
                       bluecell_Currdatastatus.ATT_DL4_PATH = false;
                   }else{
                       if(HAL_GPIO_ReadPin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin) == GPIO_PIN_SET)
                       {//CLOCK;        
                           bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4++;
                       }
                       /*ShutDown Inc*/
                       if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                           /*Path Recovery*/
                           HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_SET);//CLOCK
                           bluecell_Prevdatastatus.ATT_DL4_H =0xFF;
                           bluecell_Prevdatastatus.ATT_DL4_L = 0xFF;
                           bluecell_Currdatastatus.ATT_DL4_PATH = true;
                           CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                           DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] = 0;
                       }else{
                           /*Path Off*/
                           HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_RESET);//CLOCK
                           bluecell_Currdatastatus.ATT_DL4_PATH = false;
                       }
                   }
               }
               else{
                   if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                       bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P4;
                       bluecell_Currdatastatus.DLI_Shutdown_Alarm4       = false;            
                   }
               }         
#endif // PYJ.2020.06.19_END -- 
   }
    else{
        bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN = 0;
        bluecell_Currdatastatus.DLI_Shutdown_Alarm1 = false;            
        bluecell_Currdatastatus.DLI_Shutdown_Alarm2 = false;            
        bluecell_Currdatastatus.DLI_Shutdown_Alarm3 = false;            
        bluecell_Currdatastatus.DLI_Shutdown_Alarm4 = false;

    }

}
#else
void DLI_ShutdownAlarmCheck()
{



    
    if(bluecell_Currdatastatus.DLI_Shutdown_ON_OFF == true){
         
        /***************************************************************************************************************/
        /*                                             SHUTDOWN DL1                                                    */
        /***************************************************************************************************************/  
        if(MBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index] < 3
            && (PrevMBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index] != MBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index])
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL1_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL1_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL1_L = 0xFF;
            PrevMBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index] = MBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }
        else if(MBIC_DL_ShutdownCount[DET_Alarm_DL1_Shutdown_Index] == 3
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL1_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL1_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL1_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }                  
        if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_DL1_PATH = false;
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1++;
            DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] = 0;
            
            if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P1;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm1       = true;
            }
        }
        else{
            if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL1_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P1;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm1       = false;            
            }
        } 

        /***************************************************************************************************************/
        /*                                             SHUTDOWN DL2                                                    */
        /***************************************************************************************************************/  
        if(MBIC_DL_ShutdownCount[DET_Alarm_DL2_Shutdown_Index] < 3
            && (PrevMBIC_DL_ShutdownCount[DET_Alarm_DL2_Shutdown_Index] != MBIC_DL_ShutdownCount[DET_Alarm_DL2_Shutdown_Index])
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL2_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL2_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL2_L = 0xFF;
            PrevMBIC_DL_ShutdownCount[DET_Alarm_DL2_Shutdown_Index] = MBIC_DL_ShutdownCount[DET_Alarm_DL2_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
            printf("DL SHUTDOWN RECOVERY OPERATE PATH ON\r\n");
        }
        else if(MBIC_DL_ShutdownCount[DET_Alarm_DL2_Shutdown_Index] == 3
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL2_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL2_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL2_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
            printf("DL SHUTDOWN RECOVERY OPERATE PATH ON\r\n");
        }                  
        if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
        
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_DL2_PATH = false;
             printf("DL SHUTDOWN RECOVERY OPERATE PATH OFF\r\n");
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2++;
            DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] = 0;
            printf("Shutdown Operate DL2 ============== bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 : %d \r\n",bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2);
            
            if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P2;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm2       = true;
            }
        }
        else{
            if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL2_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.DLI_Shutdown_Retry_Count2 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P2;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm2       = false;            
            }
        } 
        /***************************************************************************************************************/
        /*                                             SHUTDOWN DL3                                                    */
        /***************************************************************************************************************/  
        if(MBIC_DL_ShutdownCount[DET_Alarm_DL3_Shutdown_Index] < 3
            && (PrevMBIC_DL_ShutdownCount[DET_Alarm_DL3_Shutdown_Index] != MBIC_DL_ShutdownCount[DET_Alarm_DL3_Shutdown_Index])
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL3_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL3_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL3_L = 0xFF;
            PrevMBIC_DL_ShutdownCount[DET_Alarm_DL3_Shutdown_Index] = MBIC_DL_ShutdownCount[DET_Alarm_DL3_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }
        else if(MBIC_DL_ShutdownCount[DET_Alarm_DL3_Shutdown_Index] == 3
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL3_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL3_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL3_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }                  
        if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_DL3_PATH = false;
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3++;
            DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] = 0;
            
            if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P3;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm3       = true;
            }else{
            }

        }
        else{
            if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL3_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.DLI_Shutdown_Retry_Count3 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P3;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm3       = false;            
            }
        } 
        /***************************************************************************************************************/
        /*                                             SHUTDOWN DL4                                                    */
        /***************************************************************************************************************/  
        if(MBIC_DL_ShutdownCount[DET_Alarm_DL4_Shutdown_Index] < 3
            && (PrevMBIC_DL_ShutdownCount[DET_Alarm_DL4_Shutdown_Index] != MBIC_DL_ShutdownCount[DET_Alarm_DL4_Shutdown_Index])
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL4_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL4_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL4_L = 0xFF;
            PrevMBIC_DL_ShutdownCount[DET_Alarm_DL4_Shutdown_Index] = MBIC_DL_ShutdownCount[DET_Alarm_DL4_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }
        else if(MBIC_DL_ShutdownCount[DET_Alarm_DL4_Shutdown_Index] == 3
            && DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_DL4_PATH = true;            
            bluecell_Prevdatastatus.ATT_DL4_H = 0xFF;
            bluecell_Prevdatastatus.ATT_DL4_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }                  
        if(DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_DL4_PATH = false;
            bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4++;
            DET_DL_Shutdown_On_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] = 0;
            
            if(bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN |= ALARM_DLI_SHUTDOWN_P4;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm4       = true;
            }else{
            }

        }
        else{
            if(DET_DL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_DL4_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.DLI_Shutdown_Retry_Count4 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN &= ~ALARM_DLI_SHUTDOWN_P4;
                bluecell_Currdatastatus.DLI_Shutdown_Alarm4       = false;            
            }
        } 
    }
    else{
            bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN = 0;
            bluecell_Currdatastatus.DLI_Shutdown_Alarm1 = false;            
            bluecell_Currdatastatus.DLI_Shutdown_Alarm2 = false;            
            bluecell_Currdatastatus.DLI_Shutdown_Alarm3 = false;            
            bluecell_Currdatastatus.DLI_Shutdown_Alarm4 = false;
    }
}

#endif // PYJ.2020.06.21_END -- 


void ULO_ShutdownAlarmCheck(){
   // double temp = 0;
    //double ret = 0;
   // int8_t ResdBm[4] = {0,};


//        if(ALC_AlarmSet[ALC_Alarm_UL1_Index] == true
//         &&ALC_On_AlarmTimerCnt[ALC_Alarm_UL1_Index] >= MBIC_ON_MAINTAIN_SEC){
//            printf("ALARM_ALC_P1 OCCUR\r\n");
//            bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm |= ALARM_ALC_P1;
//            bluecell_Currdatastatus.ULO_ALC_Alarm1       = true;
//        }else{
//            if(ALC_On_AlarmTimerCnt[ALC_Alarm_UL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
//                bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm &= ~ALARM_ALC_P1;
//                bluecell_Currdatastatus.ULO_ALC_Alarm1       = false;            
//            }
//        }
        
    if(bluecell_Currdatastatus.ULO_Shutdown_ON_OFF == true){
#if 0 // PYJ.2020.06.21_BEGIN -- 
       if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
       {
        
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P1;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm1       = true;
                bluecell_Currdatastatus.ATT_UL1_PATH = false;
                HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,bluecell_Currdatastatus.ATT_UL1_PATH);//CLOCK            
            }else{
                if(HAL_GPIO_ReadPin(PATH_EN_DL1_GPIO_Port,PATH_EN_DL1_Pin) == GPIO_PIN_SET)
                {//CLOCK;        
                    bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1++;
                }
                /*ShutDown Inc*/
                if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                    /*Path Recovery*/
                    HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_SET);//CLOCK
                    bluecell_Prevdatastatus.ATT_UL1_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL1_L = 0xFF;
                    bluecell_Currdatastatus.ATT_UL1_PATH = true;
                    CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                    DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] = 0;
                }else{
                    /*Path Off*/
                    HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_RESET);//CLOCK
                    bluecell_Currdatastatus.ATT_UL1_PATH = false;
                }
            }
        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P1;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm1       = false;            
            }
        } 
        
#endif // PYJ.2020.06.21_END -- 
#if 1 // PYJ.2020.06.19_BEGIN --
        /***************************************************************************************************************/
        /*                                             SHUTDOWN UL1                                                    */
        /***************************************************************************************************************/  
        if(MBIC_UL_ShutdownCount[DET_Alarm_UL1_Shutdown_Index] < 3
            && (PrevMBIC_UL_ShutdownCount[DET_Alarm_UL1_Shutdown_Index] != MBIC_UL_ShutdownCount[DET_Alarm_UL1_Shutdown_Index])
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL1_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL1_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL1_L = 0xFF;
            PrevMBIC_UL_ShutdownCount[DET_Alarm_UL1_Shutdown_Index] = MBIC_UL_ShutdownCount[DET_Alarm_UL1_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }
        else if(MBIC_UL_ShutdownCount[DET_Alarm_UL1_Shutdown_Index] == 3
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL1_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL1_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL1_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }                  
        if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_UL1_GPIO_Port,PATH_EN_UL1_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_UL1_PATH = false;
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1++;
            DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] = 0;
            
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P1;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm1       = true;
            }else{
            }

        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL1_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.ULO_Shutdown_Retry_Count1 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P1;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm1       = false;            
            }
        } 

        /***************************************************************************************************************/
        /*                                             SHUTDOWN UL2                                                    */
        /***************************************************************************************************************/  
        if(MBIC_UL_ShutdownCount[DET_Alarm_UL2_Shutdown_Index] < 3
            && (PrevMBIC_UL_ShutdownCount[DET_Alarm_UL2_Shutdown_Index] != MBIC_UL_ShutdownCount[DET_Alarm_UL2_Shutdown_Index])
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL2_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL2_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL2_L = 0xFF;
            PrevMBIC_UL_ShutdownCount[DET_Alarm_UL2_Shutdown_Index] = MBIC_UL_ShutdownCount[DET_Alarm_UL2_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
            printf("UL SHUTDOWN RECOVERY OPERATE PATH ON\r\n");
        }
        else if(MBIC_UL_ShutdownCount[DET_Alarm_UL2_Shutdown_Index] == 3
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL2_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL2_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL2_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
            printf("UL SHUTDOWN RECOVERY OPERATE PATH ON\r\n");
        }                  
        if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
        
#if 0 // PYJ.2020.06.21_BEGIN -- 
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P2;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm2       = true;
                HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_RESET);//CLOCK            
                bluecell_Currdatastatus.ATT_UL2_PATH = false;
            }else{
                if(HAL_GPIO_ReadPin(PATH_EN_DL2_GPIO_Port,PATH_EN_DL2_Pin) == GPIO_PIN_SET)
                {//CLOCK;        
                    bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2++;
                }
                /*ShutDown Inc*/
                if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                    /*Path Recovery*/
                    HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_SET);//CLOCK
                    bluecell_Prevdatastatus.ATT_UL2_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL2_L = 0xFF;
                    bluecell_Currdatastatus.ATT_UL2_PATH = true;
                    CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                    DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] = 0;
                }else{
                    /*Path Off*/
                    HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_RESET);//CLOCK
                    bluecell_Currdatastatus.ATT_UL2_PATH = false;
                }
            }
#else
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_UL2_GPIO_Port,PATH_EN_UL2_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_UL2_PATH = false;
             printf("UL SHUTDOWN RECOVERY OPERATE PATH OFF\r\n");
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2++;
            DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] = 0;
            printf("Shutdown Operate UL2 ============== bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2 : %d \r\n",bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2);
            
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P2;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm2       = true;
            }

#endif // PYJ.2020.06.21_END -- 
        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL2_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.ULO_Shutdown_Retry_Count2!= RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P2;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm2       = false;            
            }
        } 
        /***************************************************************************************************************/
        /*                                             SHUTDOWN UL3                                                    */
        /***************************************************************************************************************/  
        if(MBIC_UL_ShutdownCount[DET_Alarm_UL3_Shutdown_Index] < 3
            && (PrevMBIC_UL_ShutdownCount[DET_Alarm_UL3_Shutdown_Index] != MBIC_UL_ShutdownCount[DET_Alarm_UL3_Shutdown_Index])
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL3_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL3_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL3_L = 0xFF;
            PrevMBIC_UL_ShutdownCount[DET_Alarm_UL3_Shutdown_Index] = MBIC_UL_ShutdownCount[DET_Alarm_UL3_Shutdown_Index];
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }
        else if(MBIC_UL_ShutdownCount[DET_Alarm_UL3_Shutdown_Index] == 3
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL3_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL3_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL3_L = 0xFF;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }                  
        if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_UL3_PATH = false;
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3++;
            DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] = 0;
            
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P3;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm3       = true;
            }else{
            }

        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P3;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm3       = false;            
            }
        } 
        /***************************************************************************************************************/
        /*                                             SHUTDOWN UL4                                                    */
        /***************************************************************************************************************/  
        if(MBIC_UL_ShutdownCount[DET_Alarm_UL4_Shutdown_Index] < 3
            && (PrevMBIC_UL_ShutdownCount[DET_Alarm_UL4_Shutdown_Index] != MBIC_UL_ShutdownCount[DET_Alarm_UL4_Shutdown_Index])
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL4_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL4_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL4_L = 0xFF;
            PrevMBIC_UL_ShutdownCount[DET_Alarm_UL4_Shutdown_Index] = MBIC_UL_ShutdownCount[DET_Alarm_UL4_Shutdown_Index];
            printf("Current Line : %d\r\n",__LINE__);
            DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] = 0;
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }
        else if(MBIC_UL_ShutdownCount[DET_Alarm_UL4_Shutdown_Index] == 3
            && DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_RECOVERY_LAST_SHUTDOWN_MAINTAIN_SEC){
            HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_SET);//CLOCK
            bluecell_Currdatastatus.ATT_UL4_PATH = true;            
            bluecell_Prevdatastatus.ATT_UL4_H = 0xFF;
            bluecell_Prevdatastatus.ATT_UL4_L = 0xFF;
            printf("Current Line : %d\r\n",__LINE__);            
            CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus); 
        }                  
        if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
        {
            /*Shutdown 5sec Alarm*/
            HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_RESET);//CLOCK      
            bluecell_Currdatastatus.ATT_UL4_PATH = false;
            bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4++;
            DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] = 0;
            
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4 = RETRYCNT_MAX;
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P4;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm4       = true;
            }

        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC
                && bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4 != RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P4;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm4       = false;            
            }
        } 
        

#if 0 // PYJ.2020.06.21_BEGIN -- 
        if(ADC_Alarm_UL_Shutdown_Set[DET_Alarm_UL3_Shutdown_Index] == true
         &&DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
       {
        
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P3;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm3       = true;
                HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_RESET);//CLOCK            
                bluecell_Currdatastatus.ATT_UL3_PATH = false;
            }else{
                if(HAL_GPIO_ReadPin(PATH_EN_DL3_GPIO_Port,PATH_EN_DL3_Pin) == GPIO_PIN_SET)
                {//CLOCK;        
                    bluecell_Currdatastatus.ULO_Shutdown_Retry_Count3++;
                }
                /*ShutDown Inc*/
                if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                    /*Path Recovery*/
                    HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_SET);//CLOCK
                    bluecell_Prevdatastatus.ATT_UL3_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL3_L =0xFF;
                    bluecell_Currdatastatus.ATT_UL3_PATH = true;
                    CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                    DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] = 0;
                }else{
                    /*Path Off*/
                    HAL_GPIO_WritePin(PATH_EN_UL3_GPIO_Port,PATH_EN_UL3_Pin,GPIO_PIN_RESET);//CLOCK
                    bluecell_Currdatastatus.ATT_UL3_PATH = false;
                }
            }
        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL3_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P3;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm3       = false;            
            }
        } 


        if(ADC_Alarm_UL_Shutdown_Set[DET_Alarm_UL4_Shutdown_Index] == true
         &&DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_ON_SHUTDOWN_MAINTAIN_SEC)
       {
        
            if(bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4 >= RETRYCNT_MAX){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN |= ALARM_ULO_SHUTDOWN_P4;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm4       = true;
                HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_RESET);//CLOCK            
                bluecell_Currdatastatus.ATT_UL4_PATH = false;
            }else{
                if(HAL_GPIO_ReadPin(PATH_EN_DL4_GPIO_Port,PATH_EN_DL4_Pin) == GPIO_PIN_SET)
                {//CLOCK;        
                    bluecell_Currdatastatus.ULO_Shutdown_Retry_Count4++;
                }
                /*ShutDown Inc*/
                if(DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_RECOVERY_SHUTDOWN_MAINTAIN_SEC){
                    /*Path Recovery*/
                    HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_SET);//CLOCK
                    bluecell_Prevdatastatus.ATT_UL4_H = 0xFF;
                    bluecell_Prevdatastatus.ATT_UL4_L = 0xFF;
                    bluecell_Currdatastatus.ATT_UL4_PATH = true;
                    CompareAttenData(bluecell_Currdatastatus,bluecell_Prevdatastatus);
                    DET_UL_Shutdown_On_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] = 0;
                }else{
                    /*Path Off*/
                    HAL_GPIO_WritePin(PATH_EN_UL4_GPIO_Port,PATH_EN_UL4_Pin,GPIO_PIN_RESET);//CLOCK
                    bluecell_Currdatastatus.ATT_UL4_PATH = false;
                }
            }
        }
        else{
            if(DET_UL_Shutdown_Off_AlarmTimerCnt[DET_Alarm_UL4_Shutdown_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN &= ~ALARM_ULO_SHUTDOWN_P4;
                bluecell_Currdatastatus.ULO_Shutdown_Alarm4       = false;            
            }
        }         
#endif // PYJ.2020.06.21_END -- 
#endif // PYJ.2020.06.19_END -- 

    }
    else{
            bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN = 0;
            bluecell_Currdatastatus.ULO_Shutdown_Alarm1 = false;            
            bluecell_Currdatastatus.ULO_Shutdown_Alarm2 = false;            
            bluecell_Currdatastatus.ULO_Shutdown_Alarm3 = false;            
            bluecell_Currdatastatus.ULO_Shutdown_Alarm4 = false;

    }
}


void ULO_ALCAlarmCheck(){
    //double temp = 0;
    //double ret = 0;
    //int8_t ResdBm[4] = {0,};
    if(bluecell_Currdatastatus.ULO_ALC_ON_OFF == true){
        if(ALC_AlarmSet[ALC_Alarm_UL1_Index] == true
         &&ALC_On_AlarmTimerCnt[ALC_Alarm_UL1_Index] >= MBIC_ON_MAINTAIN_SEC){
//            printf("ALARM_ALC_P1 OCCUR\r\n");

                bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm |= ALARM_ALC_P1;
                bluecell_Currdatastatus.ULO_ALC_Alarm1       = true;
        }else{
            if(ALC_Off_AlarmTimerCnt[ALC_Alarm_UL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm &= ~ALARM_ALC_P1;
                bluecell_Currdatastatus.ULO_ALC_Alarm1       = false;            
            }
        }
        if(ALC_AlarmSet[ALC_Alarm_UL2_Index] == true
         &&ALC_On_AlarmTimerCnt[ALC_Alarm_UL2_Index] >= MBIC_ON_MAINTAIN_SEC){
//            printf("ALARM_ALC_P2 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm |= ALARM_ALC_P2;
            bluecell_Currdatastatus.ULO_ALC_Alarm2       = true;
        }else{
            if(ALC_Off_AlarmTimerCnt[ALC_Alarm_UL2_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm &= ~ALARM_ALC_P2;
                bluecell_Currdatastatus.ULO_ALC_Alarm2       = false;            
            }
        }
        if(ALC_AlarmSet[ALC_Alarm_UL3_Index] == true
         &&ALC_On_AlarmTimerCnt[ALC_Alarm_UL3_Index] >= MBIC_ON_MAINTAIN_SEC){
//            printf("ALARM_ALC_P3 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm |= ALARM_ALC_P3;
            bluecell_Currdatastatus.ULO_ALC_Alarm3       = true;
        }else{
            if(ALC_Off_AlarmTimerCnt[ALC_Alarm_UL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm &= ~ALARM_ALC_P3;
                bluecell_Currdatastatus.ULO_ALC_Alarm3       = false;            
            }
        }
        if(ALC_AlarmSet[ALC_Alarm_UL3_Index] == true
         &&ALC_On_AlarmTimerCnt[ALC_Alarm_UL3_Index] >= MBIC_ON_MAINTAIN_SEC){
//            printf("ALARM_ALC_P4 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm |= ALARM_ALC_P4;
            bluecell_Currdatastatus.ULO_ALC_Alarm4       = true;
        }else{
            if(ALC_Off_AlarmTimerCnt[ALC_Alarm_UL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm &= ~ALARM_ALC_P4;
                bluecell_Currdatastatus.ULO_ALC_Alarm4       = false;            
            }
        }        
    }

}
void DLI_AGCAlarmCheck(){
    //double temp = 0;
    //double ret = 0;
    //int8_t ResdBm[4] = {0,};


//    if(DET_DL_Low_On_AlarmTimerCnt[DET_Alarm_DL1_Index] >= MBIC_ON_MAINTAIN_SEC
//     &&ADC_Alarm_DL_Low_Set[DET_Alarm_DL1_Index] == true){
//        bluecell_Currdatastatus.ALARM_DLI_Level |= ALARM_DLI_P1_LEVEL_LOW;
//        bluecell_Currdatastatus.DLI_Level_Low_Alarm1 = true;
//
//    }else{/**/
//        if(DET_DL_Low_Off_AlarmTimerCnt[DET_Alarm_DL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
//            bluecell_Currdatastatus.ALARM_DLI_Level &= ~ALARM_DLI_P1_LEVEL_LOW;
//            bluecell_Currdatastatus.DLI_Level_Low_Alarm1 = false;
//        }
//    }

    if(bluecell_Currdatastatus.DLI_AGC_ON_OFF == true){
        if(AGC_AlarmSet[AGC_Alarm_DL1_Index] == true
         &&AGC_On_AlarmTimerCnt[AGC_Alarm_DL1_Index] >= MBIC_ON_MAINTAIN_SEC){
            printf("ALARM_AGC_P1 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm |= ALARM_AGC_P1;
            bluecell_Currdatastatus.DLI_AGC_Alarm1       = true;
        }else{
            if(AGC_Off_AlarmTimerCnt[AGC_Alarm_DL1_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm &= ~ALARM_AGC_P1;
                bluecell_Currdatastatus.DLI_AGC_Alarm1       = false;            
            }
        }
        if(AGC_AlarmSet[AGC_Alarm_DL2_Index] == true
         &&AGC_On_AlarmTimerCnt[AGC_Alarm_DL2_Index] >= MBIC_ON_MAINTAIN_SEC){
            printf("ALARM_AGC_P2 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm |= ALARM_AGC_P2;
            bluecell_Currdatastatus.DLI_AGC_Alarm2       = true;
        }else{
            if(AGC_Off_AlarmTimerCnt[AGC_Alarm_DL2_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm &= ~ALARM_AGC_P2;
                bluecell_Currdatastatus.DLI_AGC_Alarm2       = false;            
            }
        }
        if(AGC_AlarmSet[AGC_Alarm_DL3_Index] == true
         &&AGC_On_AlarmTimerCnt[AGC_Alarm_DL3_Index] >= MBIC_ON_MAINTAIN_SEC){
            printf("ALARM_AGC_P3 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm |= ALARM_AGC_P3;
            bluecell_Currdatastatus.DLI_AGC_Alarm3       = true;
        }else{
            if(AGC_Off_AlarmTimerCnt[AGC_Alarm_DL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm &= ~ALARM_AGC_P3;
                bluecell_Currdatastatus.DLI_AGC_Alarm3       = false;            
            }
        }
        if(AGC_AlarmSet[AGC_Alarm_DL3_Index] == true
         &&AGC_On_AlarmTimerCnt[AGC_Alarm_DL3_Index] >= MBIC_ON_MAINTAIN_SEC){
            printf("ALARM_AGC_P4 OCCUR\r\n");
            bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm |= ALARM_AGC_P4;
            bluecell_Currdatastatus.DLI_AGC_Alarm4       = true;
        }else{
            if(AGC_Off_AlarmTimerCnt[AGC_Alarm_DL3_Index] >= MBIC_OFF_MAINTAIN_SEC){
                bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm &= ~ALARM_AGC_P4;
                bluecell_Currdatastatus.DLI_AGC_Alarm4       = false;            
            }
        }        
    }
}
void Temp_HighAlarmCheck(){
    //double temp = 0;
    //double ret = 0;
    //int8_t ResdBm[4] = {0,};
    if(bluecell_Currdatastatus.Temp_High_Threshold <= (bluecell_Currdatastatus.DET_TEMP )){
        bluecell_Currdatastatus.Temp_High_Alarm = true;
        if(Alarm_Temp_TimerOnCnt > MBIC_ON_MAINTAIN_SEC){
            bluecell_Currdatastatus.ALARM_TEMP_HIGH |= ENVIRONMENT_TEMPHIGH;
        }
    }else{
        if(bluecell_Currdatastatus.ALARM_TEMP_HIGH == ENVIRONMENT_TEMPHIGH){
            if(bluecell_Currdatastatus.Temp_High_Threshold - 2 >= (bluecell_Currdatastatus.DET_TEMP )){
                bluecell_Currdatastatus.Temp_High_Alarm = false;        
                if( Alarm_Temp_TimerOffCnt > MBIC_OFF_MAINTAIN_SEC){
                    printf("1 Alarm TEMP OFF Curr : %d  Limit : %d \r\n",(bluecell_Currdatastatus.DET_TEMP ),bluecell_Currdatastatus.Temp_High_Threshold);
                    bluecell_Currdatastatus.ALARM_TEMP_HIGH &= ~ENVIRONMENT_TEMPHIGH;
                }
            }
        }
    }

}

void Alarm_Check(){
   // double temp = 0;
   // double ret = 0;
   // int8_t ResdBm[4] = {0,};

#if 0 // PYJ.2020.06.21_BEGIN -- 
    if(bluecell_Currdatastatus.ALARM_MASK1 == true){
        bluecell_Currdatastatus.ALARM_TEMP_HIGH             = 0;
        bluecell_Currdatastatus.ALARM_DLI_Level             = 0;
        bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN         = 0;
        bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm         = 0;
        bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm         = 0;
        bluecell_Currdatastatus.ALARM_ULO_Level             = 0;
        bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN         = 0;
        Alarm_LED_OnSet = false;
        return;
    }
#endif // PYJ.2020.06.21_END -- 

    if(bluecell_Currdatastatus.ALARM_TEMP_HIGH             > 0 
    || bluecell_Currdatastatus.ALARM_DLI_Level             > 0
    || bluecell_Currdatastatus.ALARM_DLI_SHTUTDOWN         > 0
    || bluecell_Currdatastatus.ALARM_DLI_AGC_Alarm         > 0
    || bluecell_Currdatastatus.ALARM_ULO_ALC_Alarm         > 0
    || bluecell_Currdatastatus.ALARM_ULO_Level             > 0
    || bluecell_Currdatastatus.ALARM_ULO_SHTUTDOWN         > 0)
    {
//        AlarmTimerOnSet = true;
//        AlarmTimerOffSet = false;
//        if(AlarmTimerOnCnt > 3000){
            Alarm_LED_OnSet  = true;
//        }
    }else{
//        AlarmTimerOffSet = true;// False 
//        if(AlarmTimerOffCnt > 10000){
//            AlarmTimerOnSet = false;
            Alarm_LED_OnSet = false;
//            printf("=================AlarmTimerOffCnt=================\r\n");
//        }
    }

    Temp_HighAlarmCheck();
    DLI_AGCAlarmCheck();
    ULO_ALCAlarmCheck();
    ULO_ShutdownAlarmCheck();
    DLI_ShutdownAlarmCheck();   
    DLI_LevelAlarmCheck();
    ULO_LevelAlarmCheck();
}
#if 0 // PYJ.2020.06.19_BEGIN -- 
void MBIC_ShutDown_Check(){
    
    /*
        - Timer Cnt Init 
        - Shutdown inc
        - Path Off 
        
    */
    if(ShutdownCnt_Get(MBIC_Shutdown_DL1) > MBIC_ON_SHUTDOWN_MAINTAIN_SEC){
        bluecell_Currdatastatus.DLI_Shutdown_Retry_Count1++;
        bluecell_Currdatastatus.ATT_DL1_PATH = true;
    }
    ShutdownCnt_Get(MBIC_Shutdown_DL2);
    ShutdownCnt_Get(MBIC_Shutdown_DL3);
    ShutdownCnt_Get(MBIC_Shutdown_DL4);

    ShutdownCnt_Get(MBIC_Shutdown_UL1);
    ShutdownCnt_Get(MBIC_Shutdown_UL2);
    ShutdownCnt_Get(MBIC_Shutdown_UL3);
    ShutdownCnt_Get(MBIC_Shutdown_UL4);
    
}
#endif // PYJ.2020.06.19_END -- 
